
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "adbhandler.h"
#include <QMessageBox>
#include <QTreeWidgetItem>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QCoreApplication>

AdbHandler::AdbHandler( QTreeWidget *tree){
	treeWidget = tree;
	currentText = "";
	coursesList = "";
	trimesterCount = "";
	staticCost = "";
	federationName = "";
	m_cfCurrentId = "";
	currentPictureIsEmbed=false;
	m_cfMap.clear();
#ifdef ASSUMA_DEBUG
	qDebug() << "AdbHandler-> Constructed";
#endif
}

QString AdbHandler::getCoursesList(){
	return coursesList;
}

QString AdbHandler::getStaticCost(){
	return staticCost;
}

QString AdbHandler::getTrimesterCount(){
	return trimesterCount;
}

QString AdbHandler::getFederationName(){
	return federationName;
}

void AdbHandler::setCustomFieldsMap( const QMap<QString,QString> & map ){
	m_cfMap = map;
}

bool AdbHandler::startElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &qName, const QXmlAttributes &attributes ){
	bool ok;
	if( qName == "Assuma" ){
		float version = attributes.value("adbversion").toFloat(&ok);
#ifdef ASSUMA_DEBUG
		qDebug() << "AdbHandler-> Assuma DB version: " << version;
#endif
		if( version < 3.14 ){
			m_error = AdbVersionMismatch;
			return false;
		}
		coursesList = attributes.value("courseslist");
		staticCost = attributes.value("staticcost");
		trimesterCount = attributes.value("trimestercount");
		federationName = attributes.value("federation") ;
	}
	if( qName == "Subscriber" ){
		currentItem = new QTreeWidgetItem(treeWidget);
	}
	if( qName != "Subscriber" && qName != "Assuma" ){
		currentText = "";
		if( qName == "CustomField" )
			m_cfCurrentId = attributes.value("id");
	}
	if( qName == "Picture" ){
		int isEmbed = attributes.value("embed").toInt(&ok);
		if( isEmbed == 1){
			currentPictureIsEmbed = true;
			currentText.append("embedImage:");
		}
		else
			currentPictureIsEmbed = false;
	}
	return true;
}

bool AdbHandler::endElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &qName )
{
	if( qName != "Subscriber" && qName != "Assuma" && qName != "CustomField"){
		int col = elementColumn(QCoreApplication::translate("AssumaMainWindow",qName.toAscii()));
		if( col != -1 ){
			currentItem->setText(col,currentText);
#ifdef ASSUMA_DEBUG
			qDebug() << "AdbHandler::endElement => setting " << currentText << " at column " << col << " for the " << qName << " column";
#endif
		}
	}
	else if( qName == "CustomField" ){
		int col = elementColumn( m_cfMap.value(m_cfCurrentId) );
		if( col != -1 ){
			currentItem->setText(col,currentText);
#ifdef ASSUMA_DEBUG
			qDebug() << "AdbHandler::endElement => setting CF value " << currentText << " at column " << col << " for the " << qName << " column";
#endif
		}
	}
	if( qName == "Picture" ){
		currentPictureIsEmbed = false;
	}
	return true;
}

bool AdbHandler::characters(const QString &str){
	currentText += str ;
	return true;
}

bool AdbHandler::fatalError(const QXmlParseException &exception){
	QMessageBox::warning(0,QObject::tr("Association Subscribers Manager ADB parser"),
		QObject::tr("parse error at line %1, column %2:\n%3")
		.arg( exception.lineNumber() )
		.arg( exception.columnNumber() )
		.arg( exception.message() )
		
	);
	return false;
}

QString AdbHandler::errorString() const {
	if( m_error == AdbVersionMismatch )
		return QCoreApplication::translate("AssumaMainWindow", "the ADB file version is not supported by the parser." );
	else
		return QCoreApplication::translate("AssumaMainWindow", "unknown error.");
}

int AdbHandler::elementColumn(const QString& element){
	QTreeWidgetItem *header = treeWidget->headerItem();
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		if( element == header->text(i) )
			return i;
	}
	return -1;
}
