
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "addonxmlhandler.h"
#include <QMessageBox>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

AddonXmlHandler::AddonXmlHandler( ){
	_data.clear();
#ifdef ASSUMA_DEBUG
	qDebug() << "AddonXmlHandler-> Constructed";
#endif
}

bool AddonXmlHandler::startElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &qName, const QXmlAttributes &attributes ){
	if( qName == "Locale")
		tmpLang = attributes.value("lang","");
	currentText = "";
	return true;
}

bool AddonXmlHandler::endElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &qName )
{
	if( qName == "CodeFile" || (qName == "Locale" && tmpLang == _lang) || ( qName == "Locale" && tmpLang.isEmpty() ) )
		_data.insertMulti( qName , currentText );
// 	else if (qName == "Locale" && tmpLang == _lang)
// 		_data.insertMulti( qName , currentText );
	else
		_data.insert( qName , currentText );
	return true;
}

bool AddonXmlHandler::characters(const QString &str){
	currentText += str;
	return true;
}

void AddonXmlHandler::setLanguage(const QString& l){
	_lang = l;
}

QMap<QString,QString> AddonXmlHandler::getData(){
	return _data;
}

bool AddonXmlHandler::fatalError(const QXmlParseException &exception){
	QMessageBox::warning(0,QObject::tr("Association Subscribers Manager Addon XML parser"),
		QObject::tr("parse error at line %1, column %2:\n%3")
		.arg( exception.lineNumber() )
		.arg( exception.columnNumber() )
		.arg( exception.message() )
		
	);
	return false;
}

