#include "assuma_associationsettings.h"
#include "assuma_effect.h"
#include "assuma_mainwindow.h"
#include "assuma_customwidgets.h"
#include "assuma_treewidgetitemeditordelegate.h"
#include "customfieldsxmlhandler.h"
#include <QCryptographicHash>
#include <QDateTime>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

AssumaAssociationSettings::AssumaAssociationSettings(QWidget *parent) : QWidget(parent){
	m_saveOnChange = false;
	setupUi(this);
	removeCurrentActivityGroupBox->hide();
	removeCurrentFeeGroupBox->hide();
	AddFeeGroupBox->hide();
	AddActivityGroupBox->hide();
	contactsAddEmailgroupBox->hide();
	contactsRemoveEmailgroupBox->hide();
	federationEditorFrame->hide();
	statusLabel->clear();
	contactsListWidget->setItemDelegate( new AssumaTreeWidgetItemEditorDelegate );
	addFeeFeeLineEdit->setValidator( new QDoubleValidator(addFeeFeeLineEdit) );
	if( coursesTabWidget->tabText(0) == tr("All courses") && coursesTreeWidget(0)->topLevelItemCount() == 0 )
		coursesTabWidget->removeTab(0);
	licenseTableWidget->sortByColumn(0,Qt::AscendingOrder);
	
// 	setCustomFieldsFileName("default.customfields.xml");
	booleanComboOptionsBox->hide();
	multipleValuesOptionsGroupBox->hide();
	connect( okLabelLineEdit, SIGNAL( textChanged(QString) ), this, SLOT( updatePreviewFromFields() ) );
	connect( notOkLabelLineEdit, SIGNAL( textChanged(QString) ), this, SLOT( updatePreviewFromFields() ) );
	m_previewWidget = 0;
	connect( AssumaCM, SIGNAL(configurationTokenChanged( AssumaConfigManager::Token, const QVariant & )), this, SLOT( updateConfiguration( AssumaConfigManager::Token, const QVariant & ) ) );
	m_saveOnChange = true;
}

void AssumaAssociationSettings::loadTheme( const QString & themeName ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAssociationSettings::loadTheme => loading theme: " << themeName;
#endif
	QIcon icon;
	icon.addFile(AssumaMW->getImageFromTheme(themeName,"button-erase"), QSize(), QIcon::Normal, QIcon::Off);
	removeFieldPushButton->setIcon(icon);
	removeFieldPushButton->setIconSize(QSize(22, 22));
	QIcon icon2;
	icon2.addFile(AssumaMW->getImageFromTheme(themeName,"button-add"), QSize(), QIcon::Normal, QIcon::Off);
	addFieldPushButton->setIcon(icon2);
	addFieldPushButton->setIconSize(QSize(22, 22));
	addValuePushButton->setIcon(icon2);
	addValuePushButton->setIconSize(QSize(22, 22));
	removeValuePushButton->setIcon(icon);
	removeValuePushButton->setIconSize(QSize(22, 22));
	tabWidget->setTabIcon(0, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-custom-fields" ) ) );
	tabWidget->setTabIcon(1, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-courses" ) ) );
	tabWidget->setTabIcon(2, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-global-fees" ) ) );
	tabWidget->setTabIcon(3, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-contacts" ) ) );
	
	pixLabelCustomFields->setText( QString() );
	pixLabelContacts->setText( QString() );
	pixLabelGlobalFees->setText( QString() );
	pixLabelCourses->setText( QString() );
	
	pixLabelCustomFields->setMaximumSize(QSize(35, 40));
	pixLabelContacts->setMaximumSize(QSize(35, 40));
	pixLabelGlobalFees->setMaximumSize(QSize(35, 40));
	pixLabelCourses->setMaximumSize(QSize(35, 40));
	
	pixLabelCustomFields->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-custom-fields") ).scaled(35,35) );
	pixLabelContacts->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-contacts") ));
	pixLabelGlobalFees->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-global-fees") ));
	pixLabelCourses->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-courses") ));
	
	QIcon icon3;
	icon3.addFile( AssumaMW->getImageFromTheme(themeName,"button-add") , QSize(), QIcon::Normal, QIcon::Off);
	addFeeButton->setIcon(icon3);
	addFeeButton->setIconSize(QSize(22, 22));
	addActivityButton->setIcon(icon3);
	addActivityButton->setIconSize(QSize(22, 22));
	addLicensePushButton->setIcon(icon3);
	addEmailPushButton->setIcon(icon3);
	addEmailPushButton->setIconSize(QSize(22, 22));
	addEmailAddPushButton->setIcon(icon3);
	QIcon icon4;
	icon4.addFile( AssumaMW->getImageFromTheme(themeName,"button-erase-big") , QSize(), QIcon::Normal, QIcon::Off);
	removeFeeButton->setIcon(icon4);
	removeFeeButton->setIconSize(QSize(22, 22));
	removeActivityButton->setIcon(icon4);
	removeActivityButton->setIconSize(QSize(22, 22));
	removeEmailPushButton->setIcon(icon4);
	removeEmailPushButton->setIconSize(QSize(22, 22));
	
	QIcon icon6;
	icon6.addFile( AssumaMW->getImageFromTheme(themeName, "edit" ) , QSize(), QIcon::Normal, QIcon::Off);
	editFederationPushButton->setIcon(icon6);
	QIcon icon7;
	icon7.addFile( AssumaMW->getImageFromTheme(themeName, "file-save" ) , QSize(), QIcon::Normal, QIcon::Off);
	saveFederationPushButton->setIcon(icon7);
	QIcon icon8;
	icon8.addFile( AssumaMW->getImageFromTheme(themeName, "button-erase" ) , QSize(), QIcon::Normal, QIcon::Off);
	removeLicensePushButton->setIcon(icon8);

        QIcon icon_ok;
        icon_ok.addFile( AssumaMW->getImageFromTheme(themeName,"button-ok") , QSize(), QIcon::Normal, QIcon::Off);
        removeCurrentActivityConfirmButton->setIcon(icon_ok);
        removeCurrentFeeConfirmButton->setIcon(icon_ok);
        removeEmailConfirmPushButton->setIcon(icon_ok);
        addEmailAddPushButton->setIcon(icon_ok);
        addActivityAddButton->setIcon(icon_ok);
        addFeeAddButton->setIcon(icon_ok);

        QIcon icon_nok;
        icon_nok.addFile( AssumaMW->getImageFromTheme(themeName,"button-nok") , QSize(), QIcon::Normal, QIcon::Off);
        removeCurrentActivityCancelButton->setIcon(icon_nok);
        removeCurrentFeeCancelButton->setIcon(icon_nok);
        removeEmailCancelPushButton->setIcon(icon_nok);
        addFeeCancelButton->setIcon(icon_nok);
        addActivityCancelButton->setIcon(icon_nok);
        addEmailCancelPushButton->setIcon(icon_nok);
}

void AssumaAssociationSettings::updateConfiguration( AssumaConfigManager::Token token, const QVariant & value ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAssociationSettings::updateConfiguration : " << token << value;
#endif
	QString value_string = value.toString();
	QStringList tmp_list;
	QStringList activities;
	switch( token ) {
		case AssumaConfigManager::AssociationName : 
			if( value != AssociationNameLineEdit->text() ){
				AssociationNameLineEdit->setText( value_string ); 
				setCustomFieldsFileName( value_string );
			}
			break;
		case AssumaConfigManager::FederationList : federationComboBox->clear(); federationComboBox->addItems( value_string.split(";") ); break;
		case AssumaConfigManager::TrimesterCount : trimesterSpinBox->setValue( value.toInt() ) ; break;
		case AssumaConfigManager::StaticCost : taxSpinBox->setValue( value.toInt() ) ; break;
		case AssumaConfigManager::Currency : taxSpinBox->setSuffix( " "+value_string ); break;
                case AssumaConfigManager::Federation : federationComboBox->setCurrentIndex( federationComboBox->findText(value_string, Qt::MatchExactly | Qt::MatchCaseSensitive ) ); break ;
                case AssumaConfigManager::ContactsList :
			tmp_list.clear();
                        contactsListWidget->clear();
			tmp_list = value_string.split(";");
			for ( int i = 0 ; i < tmp_list.count(); ++i ){
				if( ! tmp_list.at(i).isEmpty() ){
					QListWidgetItem *it = new QListWidgetItem( tmp_list.at(i) );
					it->setFlags(it->flags() | Qt::ItemIsEditable);
					contactsListWidget->addItem( it );
				}
			}
			break;
		case AssumaConfigManager::CoursesList :
			activities = value_string.split(";");
			if( activities.size() > 0 ){
				for (int i = 0 ; i < activities.count(); ++i){
					QStringList data1 = activities.at(i).split(":");
					if( data1.size() > 1 ){
						int tab_index = -1;
						for ( int k = 0 ; k < coursesTabWidget->count(); ++k){
							if( coursesTabWidget->tabText(k) == data1.at(0) )
								tab_index = k;
						}
						if( tab_index == -1){
							addActivity( data1.at(0) );
							tab_index = coursesTabWidget->count() - 1;
						}
						QTreeWidget* tw = coursesTreeWidget(tab_index);
						if( tw ){
							if( tw->topLevelItemCount() > 0 )
								tw->clear();
							QStringList data2 = data1.at(1).split(",");
							for ( int j = 0 ; j < data2.count(); ++j){
								QStringList data3 = data2.at(j).split("=");
								if ( data3.size() > 1 ){
									QTreeWidgetItem *it = new QTreeWidgetItem ;
									it->setText(0, data3.at(0) );
									it->setText(1,data3.at(1) );
									it->setFlags(it->flags() | Qt::ItemIsEditable);
									tw->addTopLevelItem( it );
								}
							}
						}
					}
				}
			}
			break;
		default : break;
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAssociationSettings::updateConfiguration : done";
#endif
}

void AssumaAssociationSettings::setCustomFieldsFileName(const QString & fileName) {
	m_customFieldsFileName = fileName;
	if( ! QFile::exists( fileName ) )
		return;
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAssociationSettings::setCustomFieldsFileName : custom field file name : " << m_customFieldsFileName;
#endif
	customFieldsTable->clear();
	m_saveOnChange = false;
	QFile file(fileName);
	QXmlInputSource inputSource(&file);
	QXmlSimpleReader reader;
	CustomFieldsXmlHandler handler;
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	reader.parse(inputSource);
	foreach (CustomFieldItem *item, handler.getData()){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaAssociationSettings::setCustomFieldsFileName : add item " << item->id() << "," << item->label() << "," << item->type() ;
#endif
		int row = customFieldsTable->rowCount();
		customFieldsTable->insertRow( row );
		
		QTableWidgetItem *item0 = new QTableWidgetItem( item->id() );
		item0->setToolTip( tr("The ID cell is not editable.") );
		item0->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
		customFieldsTable->setItem(row, 0, item0);
		
		QTableWidgetItem *item1 = new QTableWidgetItem( item->label() );
		item1->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsEditable );
		customFieldsTable->setItem(row, 1, item1 );
		
		QTableWidgetItem *item2 = new QTableWidgetItem( tr(item->type().toUtf8()) );
		item2->setToolTip( tr("The \"Field type\" cell is not editable.") );
		item2->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
		customFieldsTable->setItem(row, 2, item2 );
		
		if( item->type() == "Ok/Not ok combobox" ){
			QStringList extra_arguments = item->extraString().split(";");
			customFieldsTable->setItem(row, 3, new QTableWidgetItem( extra_arguments.at(0) ));
			customFieldsTable->setItem(row, 4, new QTableWidgetItem( extra_arguments.at(1) ));
		}
		else{
			customFieldsTable->setItem(row, 3, new QTableWidgetItem( "" ));
			customFieldsTable->setItem(row, 4, new QTableWidgetItem( "" ));
		}
		if( item->type() == "Multiple value selector" ){
			QTableWidgetItem *item5 = new QTableWidgetItem( item->extraString() );
			item5->setToolTip( tr("This cell contains values separated by a \";\", please take this in account during edition") );
			customFieldsTable->setItem(row, 5, item5);
		}
		else {
			QTableWidgetItem *item5 = new QTableWidgetItem( "" );
			item5->setToolTip( tr("The \"Value list\" cell is not editable for this type of field.") );
			item5->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
			customFieldsTable->setItem(row, 5, item5);
		}
	}
	m_saveOnChange = true;
}

QString AssumaAssociationSettings::getCustomFieldFileName(){
	return m_customFieldsFileName;
}

void AssumaAssociationSettings::on_fieldTypeLineEdit_textChanged( const QString & text ){
	if( m_previewWidget )
		m_previewWidget->setCustomFieldLabelString( text );
}

void AssumaAssociationSettings::on_fieldTypeComboBox_currentIndexChanged(const QString & newValue){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAssociationSettings::on_fieldTypeComboBox_currentIndexChanged() : new value="<<newValue;
#endif
	if( tr("Ok/Not ok combobox") == newValue ){
		// Prepare the boolean's special options box effect
		AssumaEffect *box_effect = new AssumaEffect(0,booleanComboOptionsBox, AssumaEffect::EXPAND_VERTICALLY);
		box_effect->setVSizeDelta( 102 );
		box_effect->setReversing(false);
		booleanComboOptionsBox->resize( booleanComboOptionsBox->size().width(),0 );
		box_effect->doEffect();
	}
	else if( tr("Ok/Not ok combobox") != newValue && booleanComboOptionsBox->isVisible() ){
		AssumaEffect *box_effect = new AssumaEffect(0,booleanComboOptionsBox, AssumaEffect::SHRINK_VERTICALLY);
		box_effect->setVSizeDelta( 102 );
		box_effect->setReversing(false);
		connect( box_effect, SIGNAL( AssumaEffectFinished() ), booleanComboOptionsBox, SLOT( hide() ) );
		box_effect->doEffect();
		okLabelLineEdit->setText("");
		notOkLabelLineEdit->setText("");
	}
	if( tr("Multiple value selector") == newValue ){
		// Prepare the boolean's special options box effect
		AssumaEffect *box_effect = new AssumaEffect(0,multipleValuesOptionsGroupBox, AssumaEffect::EXPAND_VERTICALLY);
		box_effect->setVSizeDelta( 238 );
		box_effect->setReversing(false);
		multipleValuesOptionsGroupBox->resize( multipleValuesOptionsGroupBox->size().width(),0 );
		box_effect->doEffect();
	}
	else if( tr("Multiple value selector") != newValue && multipleValuesOptionsGroupBox->isVisible() ){
		AssumaEffect *box_effect = new AssumaEffect(0,multipleValuesOptionsGroupBox, AssumaEffect::SHRINK_VERTICALLY);
		box_effect->setVSizeDelta( 238 );
		box_effect->setReversing(false);
		connect( box_effect, SIGNAL( AssumaEffectFinished() ), multipleValuesOptionsGroupBox, SLOT( hide() ) );
		box_effect->doEffect();
		multipleValuesListWidget->clear();
                multipleValueLineEdit->setText("");
	}
	if( tr("<please select one...>") != newValue )
		updatePreviewFromFields();
}

void AssumaAssociationSettings::on_removeFieldPushButton_clicked(){
	customFieldsTable->removeRow( customFieldsTable->currentRow() );
	saveCustomFields( getCustomFieldFileName() );
}

void AssumaAssociationSettings::on_addFieldPushButton_clicked(){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaAssociationSettings::on_addFieldPushButton_clicked() => clicked";
#endif
	if( tr("<please select one...>") == fieldTypeComboBox->currentText() )
		return;
	m_saveOnChange = false;
	if( ! fieldTypeLineEdit->text().isEmpty() ){
		QString hashBase = fieldTypeLineEdit->text() + fieldTypeComboBox->currentText() + QDateTime::currentDateTime().toString("ddMMyyyyhhmmsszzz");
		QString id( QCryptographicHash::hash(hashBase.toLocal8Bit(), QCryptographicHash::Sha1).toHex() );
		int row = customFieldsTable->rowCount();
		customFieldsTable->insertRow( row );
		
		QTableWidgetItem *item0 = new QTableWidgetItem( id );
		item0->setToolTip( tr("The ID cell is not editable.") );
		item0->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
		customFieldsTable->setItem(row, 0, item0);
		
		QTableWidgetItem *item1 = new QTableWidgetItem( fieldTypeLineEdit->text() );
		item1->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled | Qt::ItemIsEditable );
		customFieldsTable->setItem(row, 1, item1 );
		
		QTableWidgetItem *item2 = new QTableWidgetItem( fieldTypeComboBox->currentText() );
		item2->setToolTip( tr("The \"Field type\" cell is not editable.") );
		item2->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
		customFieldsTable->setItem(row, 2, item2 );
		if( fieldTypeComboBox->currentText() == tr("Ok/Not ok combobox") ){
			customFieldsTable->setItem(row, 3, new QTableWidgetItem( okLabelLineEdit->text() ));
			customFieldsTable->setItem(row, 4, new QTableWidgetItem( notOkLabelLineEdit->text() ));
		}
		else{
			customFieldsTable->setItem(row, 3, new QTableWidgetItem( "" ));
			customFieldsTable->setItem(row, 4, new QTableWidgetItem( "" ));
		}
		if( fieldTypeComboBox->currentText() == tr("Multiple value selector") ){
			QTableWidgetItem *item = new QTableWidgetItem( listWidgetToStringList( multipleValuesListWidget ).join(";") );
			item->setToolTip( tr("This cell contains values separated by a ';', please take this in account during edition") );
			customFieldsTable->setItem(row, 5, item);
		}
		else {
			QTableWidgetItem *item = new QTableWidgetItem( "" );
			item->setToolTip( tr("The \"Value list\" cell is not editable for this type of field.") );
			item->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
			customFieldsTable->setItem(row, 5, item);
		}
		saveCustomFields( getCustomFieldFileName() );
		if(m_previewWidget){
			m_previewWidget->setVisible(false);
			delete(m_previewWidget);
			m_previewWidget = 0;
		}
		fieldTypeLineEdit->setText("");
		fieldTypeComboBox->setCurrentIndex(0);
	}
	m_saveOnChange = true;
}

void AssumaAssociationSettings::on_removeValuePushButton_clicked(){
	m_saveOnChange = false;
	QList<QListWidgetItem *>list;
	for(int k=0; k < multipleValuesListWidget->count(); k++ ){
		QListWidgetItem *it = multipleValuesListWidget->item(k);
		if( it && it->checkState() == Qt::Checked ){
			list << it;
		}
	}
	foreach (QListWidgetItem *i, list){
		QListWidgetItem *item = multipleValuesListWidget->takeItem( multipleValuesListWidget->row(i) );
		delete(item);
	}
	updatePreviewFromFields();
	m_saveOnChange = true;
}

void AssumaAssociationSettings::on_addValuePushButton_clicked() {
	QString text = multipleValueLineEdit->text();
	if (!text.isEmpty()){
		QListWidgetItem *item = new QListWidgetItem( text ) ;
		item->setFlags( Qt::ItemIsSelectable | Qt::ItemIsUserCheckable | Qt::ItemIsEnabled );
		item->setCheckState( Qt::Unchecked );
		multipleValuesListWidget->addItem(item);
		AssumaCustomWidgetComboBox *cb = qobject_cast<AssumaCustomWidgetComboBox *>( m_previewWidget );
		if( cb ){
			cb->addItem( text );
		}
		multipleValueLineEdit->setText("");
	}
}

void AssumaAssociationSettings::saveCustomFields(const QString & fileName){
	qDebug() << "AssumaAssociationSettings::saveCustomFields : fileName=" << fileName;
	QString xml("<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?>\n<AssumaCustomFields>\n");
	for(int i=0; i < customFieldsTable->rowCount(); i++ ){
		QTableWidgetItem *item0 = customFieldsTable->item(i,0);
		QTableWidgetItem *item1 = customFieldsTable->item(i,1);
		QTableWidgetItem *item2 = customFieldsTable->item(i,2);
		QTableWidgetItem *item3 = customFieldsTable->item(i,3);
		QTableWidgetItem *item4 = customFieldsTable->item(i,4);
		QTableWidgetItem *item5 = customFieldsTable->item(i,5);
		QString cfType;
		if( item2->text() == tr("Multiple line text") ){
			cfType = "Multiple line text";
		}
		else if( item2->text() == tr("Date selector") ){
			cfType = "Date selector";
		}
		else if( item2->text() == tr("Ok/Not ok combobox") ){
			cfType = "Ok/Not ok combobox";
		}
		else if( item2->text() == tr("Multiple value selector") ){
			cfType = "Multiple value selector";
		}
		else if( item2->text() == tr("Fee field") ){
			cfType = "Fee field";
		}
		else if( item2->text() == tr("One line text") ){
			cfType = "One line text";
		}
		if( item0 && item1 && item2 ){
			QString extra_string;
			if( item3 )
				extra_string.append(item3->text());
			if( item4 ){
				if( ! extra_string.isEmpty() )
					extra_string.append(";");
				extra_string.append(item4->text());
			}
			if( item5 ){
				if( ! extra_string.isEmpty() )
					extra_string.append(";");
				extra_string.append(item5->text());
			}
			xml.append( QString("\t<CustomField id=\"%1\" label=\"%2\" type=\"%3\" extra_string=\"%4\" />\n").arg(item0->text(),item1->text(),cfType,extra_string) );
		}
	}
	xml.append("</AssumaCustomFields>\n");
	QFile file( fileName );
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
		AssumaMW->showPassiveMessage( tr("Unable to save custom fields list at %1").arg(fileName), AssumaConfigManager::Warning );
		return;
	}
	QTextStream out(&file);
	out.setCodec("UTF-8");
	out << xml;
	out.flush();
	file.close();
	emit( customFieldsChanged() );
}

void AssumaAssociationSettings::updatePreview( const QString & type, const QString & text, const QStringList & extra_arguments ){
	if(m_previewWidget){
		m_previewWidget->setVisible(false);
		delete(m_previewWidget);
		m_previewWidget = 0;
	}
	if( type == tr("One line text") ){
		AssumaCustomWidgetOneLine *w = new AssumaCustomWidgetOneLine(previewFrame);
		w->setCustomFieldLabelString( text );
		w->show();
		previewFrame->layout()->addWidget(w);
		m_previewWidget = w;
	}
	else if( type == tr("Multiple line text") ){
		AssumaCustomWidgetMultipleLine *w = new AssumaCustomWidgetMultipleLine(previewFrame);
		w->setCustomFieldLabelString( text );
		w->show();
		previewFrame->layout()->addWidget(w);
		m_previewWidget = w;
	}
	else if( type == tr("Date selector") ){
		AssumaCustomWidgetDateEdit *w = new AssumaCustomWidgetDateEdit(previewFrame);
		w->setCustomFieldLabelString( text );
		w->show();
		previewFrame->layout()->addWidget(w);
		m_previewWidget = w;
	}
	else if( type == tr("Ok/Not ok combobox") ){
		AssumaCustomWidgetComboBox *w = new AssumaCustomWidgetComboBox(previewFrame);
		w->setCustomFieldLabelString( text );
		w->addItem( extra_arguments.at(0), QIcon( AssumaMW->getImageFromTheme("button-ok") ) );
		w->addItem( extra_arguments.at(1), QIcon( AssumaMW->getImageFromTheme("button-nok") ) );
		w->show();
		previewFrame->layout()->addWidget(w);
		m_previewWidget = w;
	}
	else if( type == tr("Multiple value selector") ){
		AssumaCustomWidgetComboBox *w = new AssumaCustomWidgetComboBox(previewFrame);
		w->setCustomFieldLabelString( text );
		foreach( QString str, extra_arguments ){
			if( ! str.isEmpty() )
				w->addItem(str);
		}
		w->show();
		previewFrame->layout()->addWidget(w);
		m_previewWidget = w;
	}
	else if( type == tr("Fee field") ){
		AssumaCustomWidgetFeeEditor *w = new AssumaCustomWidgetFeeEditor(previewFrame);
		w->setCustomFieldLabelString( text );
		w->show();
		previewFrame->layout()->addWidget(w);
		m_previewWidget = w;
	}
}

QStringList AssumaAssociationSettings::listWidgetToStringList( QListWidget *w ){
	if( ! w )
		return QStringList();
	
	QStringList list;
	for(int k=0; k < w->count(); k++ ){
		list << w->item(k)->text();
	}
	return list;
}

void AssumaAssociationSettings::updatePreviewFromFields(){
	QStringList list;
	list << okLabelLineEdit->text() << notOkLabelLineEdit->text() << listWidgetToStringList( multipleValuesListWidget );
	updatePreview(fieldTypeComboBox->currentText(), fieldTypeLineEdit->text(), list );
}

void AssumaAssociationSettings::on_customFieldsTable_cellClicked ( int row, int /*column*/ ){
	QTableWidgetItem *item1 = customFieldsTable->item(row,1);
	QTableWidgetItem *item2 = customFieldsTable->item(row,2);
	QTableWidgetItem *item3 = customFieldsTable->item(row,3);
	QTableWidgetItem *item4 = customFieldsTable->item(row,4);
	QTableWidgetItem *item5 = customFieldsTable->item(row,5);
	if( item1 && item2 ){
		QStringList extra_arguments;
		if( item3 )
			extra_arguments << item3->text() ;
		if( item4 )
			extra_arguments << item4->text() ;
		if( item5 )
			extra_arguments << item5->text().split(";") ;
		updatePreview(QString(item2->text()), QString(item1->text()), extra_arguments );
	}
}

void AssumaAssociationSettings::on_customFieldsTable_itemChanged ( QTableWidgetItem * /*item*/ ) {
	if( m_saveOnChange ){
		saveCustomFields( getCustomFieldFileName() );
	}
}

// Start federation editor management

void AssumaAssociationSettings::on_federationComboBox_currentIndexChanged ( const QString & text ){
	QString fileName = AssumaCM->getConfigToken(AssumaConfigManager::DataDir).toString()+"/Federations/"+text+".federation.xml";
	if( QFile::exists( fileName ) ){
		federationNameLineEdit->setText(text);
		// Load federation description file
		QFile file(fileName);
		QXmlInputSource inputSource(&file);
		QXmlSimpleReader reader;
		FederationHandler handler ;
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		reader.parse(inputSource);
		
		// Set the license list
		licenseTableWidget->setSortingEnabled(false);
		while( licenseTableWidget->rowCount() > 0 ){
			licenseTableWidget->removeRow(0);
		}
		QList<FederationLicenseFee *> tmp_fees = handler.getLicenceFees();
		int row = 0;
		foreach( FederationLicenseFee *key, tmp_fees ){
			licenseTableWidget->insertRow(row);
			
			QTableWidgetItem *c0 = licenseTableWidget->item(row,0);
			if( c0 )
				c0->setText( key->getId() );
			else
				licenseTableWidget->setItem(row,0, new QTableWidgetItem( key->getId() ) );
			
			QTableWidgetItem *c1 = licenseTableWidget->item(row,1);
			if( c1 )
				c1->setText( key->getName() );
			else
				licenseTableWidget->setItem(row,1, new QTableWidgetItem( key->getName() ) );
			
			QTableWidgetItem *c2 = licenseTableWidget->item(row,2);
			if( c2 )
				c2->setText( QString::number( key->getFee() ) );
			else
				licenseTableWidget->setItem(row,2, new QTableWidgetItem( QString::number( key->getFee() ) ) );
			
			row++;
		}
		licenseTableWidget->setSortingEnabled(true);
		licenseTableWidget->sortByColumn(1,Qt::AscendingOrder);
		if( m_saveOnChange )
				AssumaCM->setConfigToken( AssumaConfigManager::Federation, text );
	}
	else{
		statusLabel->setText( tr("<font color='red'><strong>Unable to load the federation file for %1</strong></font>").arg(text) );
	}
}

void AssumaAssociationSettings::on_addLicensePushButton_clicked(){
	licenseTableWidget->insertRow( licenseTableWidget->rowCount() );
	statusLabel->setText( tr("<font color=\"green\">New row added to licenses.</font>") );
}

void AssumaAssociationSettings::on_removeLicensePushButton_clicked(){
	licenseTableWidget->removeRow( licenseTableWidget->currentRow() );
	statusLabel->setText( tr("<font color=\"red\">Row removed from licenses.</font>") );
}

void AssumaAssociationSettings::on_saveFederationPushButton_clicked(){
	if( saveCurrentFederation() ){
		int idx = federationComboBox->findText( federationNameLineEdit->text() , Qt::MatchExactly | Qt::MatchCaseSensitive );
		// If we are dealing with a new federation, add it to the combobox
		if( idx == -1 )
			federationComboBox->addItem( federationNameLineEdit->text() );
		federationComboBox->setCurrentIndex( federationComboBox->findText( federationNameLineEdit->text() , Qt::MatchExactly | Qt::MatchCaseSensitive ) );
		statusLabel->setText( tr("<font color=\"green\">Federation saved in %1/%2.federation.xml.</font>").arg(AssumaCM->getConfigToken(AssumaConfigManager::DataDir).toString()+"/Federations/" ).arg( federationNameLineEdit->text() ) );
	}
}

QString AssumaAssociationSettings::escapeXml( const QString & xml){
	QString text(xml);
	text.replace("&","&amp;");
	text.replace("\"","&quot;");
	return text;
}

bool AssumaAssociationSettings::saveCurrentFederation(){
	QString xml;
	xml.append("<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?>\n<federation version=\"1.0\">\n");
	xml.append("\t<licences>\n");
	for( int i=0; i < licenseTableWidget->rowCount(); i++ ){
		QTableWidgetItem *id = licenseTableWidget->item(i,0);
		QTableWidgetItem *name = licenseTableWidget->item(i,1);
		QTableWidgetItem *fee = licenseTableWidget->item(i,2);
		if( ! id || id->text() == "" ){
			statusLabel->setText( tr("<font color=\"red\">Row %1 have no tax id. Federation was not saved</font>").arg(i) );
			return false;
		}
		if( ! name || name->text() == "" ){
			statusLabel->setText( tr("<font color=\"red\">Row %1 have no tax name. Federation was not saved</font>").arg(i) );
			return false;
		}
		if( ! fee || fee->text() == "" ){
			statusLabel->setText( tr("<font color=\"red\">Row %1 have no tax fee. Federation was not saved</font>").arg(i) );
			return false;
		}
		xml.append("\t\t<licence id=\"" + escapeXml( id->text() ) + "\" label=\"" + escapeXml(name->text()) + "\" fee=\"" + fee->text() + "\" />\n");
	}
	xml.append("\t</licences>\n</federation>\n");
	QString fileName = AssumaCM->getConfigToken(AssumaConfigManager::DataDir).toString() + "/Federations/" + federationNameLineEdit->text() + ".federation.xml";
	QFile file( fileName );
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
		statusLabel->setText( tr("<font color=\"red\">Unable to open file %1 for writing.</font>").arg( fileName ));
		return false;
	}
	QTextStream out(&file);
	out.setCodec("UTF-8");
	out << xml;
	return true;
}

// Stop federation editor management

QTreeWidget* AssumaAssociationSettings::coursesTreeWidget(int w_id){
	QWidget *widget = coursesTabWidget->widget(w_id);
	if( ! widget )
		return 0;
	QObjectList ol = widget->children();
	for (int i=0; i < ol.size(); i++){
		QString classname(ol.at(i)->metaObject()->className());
		if(  classname.contains("QTreeWidget",Qt::CaseInsensitive) ){
				QTreeWidget *tw = qobject_cast<QTreeWidget*>( ol.at(i) );
				return tw;
		}
	}
	return 0 ;
}

QTreeWidget* AssumaAssociationSettings::currentCoursesTreeWidget(){
	QObjectList ol = coursesTabWidget->currentWidget()->children();
	for (int i=0; i < ol.size(); i++){
		QString classname(ol.at(i)->metaObject()->className());
		if(  classname.contains("QTreeWidget",Qt::CaseInsensitive) ){
				QTreeWidget *tw = qobject_cast<QTreeWidget*>( ol.at(i) );
				return tw;
		}
	}
	return 0 ;
}

int AssumaAssociationSettings::getMaxCoursesPerWeek(QString tab){
	int tab_index=-1;
	for (int k = 0 ; k < coursesTabWidget->count(); ++k){
		if ( coursesTabWidget->tabText(k) == tab )
			tab_index = k;
	}
	int max = 0;
	if(tab_index != -1){
		QTreeWidget *tw = coursesTreeWidget(tab_index);
		if( tw ){
			for ( int i = 0 ; i < tw->topLevelItemCount(); ++i ){
				int tmp = tw->topLevelItem(i)->text(0).toInt();
				if( tmp > max ){
					max = tmp;
				}
			}
		}
	}
	return max;
}

QMap<QString, QStringList> AssumaAssociationSettings::getCoursesLessonsNumberData(){
	QMap<QString, QStringList> cln_map;
	for (int k = 0 ; k < coursesTabWidget->count(); ++k){
		QTreeWidget* tmp_tw = coursesTreeWidget(k);
		if( tmp_tw ){
			QStringList l ;
			for ( int i = 0 ; i < tmp_tw->topLevelItemCount(); ++i ){
				l << tmp_tw->topLevelItem(i)->text(0) ;
			}
			cln_map.insert(coursesTabWidget->tabText(k),l);
		}
	}
	return cln_map;
}

QStringList AssumaAssociationSettings::getCoursesList(){
	QStringList list;
	for (int k = 0 ; k < coursesTabWidget->count(); ++k){
		list << coursesTabWidget->tabText(k);
	}
	return list;
}

QString AssumaAssociationSettings::getCourseFee( QString course, int number_of_courses ){
	int k = 0;
	bool stop = false;
	while(!stop){
		if( coursesTabWidget->tabText(k) == course || k >= coursesTabWidget->count() )
			stop = true;
		else
			k++;
	}
// 	qDebug() << "AssumaAssociationSettings::getCourseFee( QString course, int number_of_courses ) : k is " << k;
	if( k < coursesTabWidget->count()){
		QTreeWidget* tmp_tw = coursesTreeWidget(k);
		if( tmp_tw ){
			stop = false;
			int i = 0;
			while(!stop){
				if( i >= tmp_tw->topLevelItemCount() || tmp_tw->topLevelItem(i)->text(0) == QString::number(number_of_courses) )
					stop = true;
				else
					i++;
			}
			if( i < tmp_tw->topLevelItemCount() )
				return tmp_tw->topLevelItem(i)->text(1) ;
		}
	}
	
	return QString();
}

QString AssumaAssociationSettings::serializeCourses(){
	QStringList tmp_list;
	for (int k = 0 ; k < coursesTabWidget->count(); ++k){
		QTreeWidget* tmp_tw = coursesTreeWidget(k);
		if( tmp_tw ){
			QStringList l ;
			for ( int i = 0 ; i < tmp_tw->topLevelItemCount(); ++i ){
				l << tmp_tw->topLevelItem(i)->text(0) + "=" + tmp_tw->topLevelItem(i)->text(1) ;
			}
			tmp_list << coursesTabWidget->tabText(k) + ":" + l.join(",");
		}
	}
	return tmp_list.join(";");
}

QString AssumaAssociationSettings::serializeContacts(){
	QStringList tmp_list;
	for ( int i = 0 ; i < contactsListWidget->count(); ++i ){
		tmp_list << contactsListWidget->item(i)->text() ;
        }
	return tmp_list.join(";");
}

// Start Fee management ( add and remove fees )

void AssumaAssociationSettings::on_addFeeButton_clicked(){
	AddFeeGroupBox->show();
	ActionGroupBox->hide();
}

void AssumaAssociationSettings::on_addFeeAddButton_clicked(){
	// We have to had for real the fee :)
	QTreeWidget* tw = currentCoursesTreeWidget();
// 	qDebug() << "Current tree widget is " << tw;
	if(tw){
		if( ! addFeeFeeLineEdit->text().isEmpty() ){
			QTreeWidgetItem *it = new QTreeWidgetItem ;
			it->setText(0, QString::number(addFeeCoursePerWeekSpinBox->value()) );
			it->setText(1,addFeeFeeLineEdit->text() );
			it->setFlags(it->flags() | Qt::ItemIsEditable);
			tw->addTopLevelItem( it );
			addFeeCoursePerWeekSpinBox->setValue(0);
			addFeeFeeLineEdit->setText(0) ;
			if( m_saveOnChange )
				AssumaCM->setConfigToken( AssumaConfigManager::CoursesList, serializeCourses() );
		}
		else{
			AssumaMW->showPassiveMessage( tr("You have to enter a fee amount."), AssumaConfigManager::Warning );
		}
	}
	else{
		AssumaMW->showPassiveMessage( tr("There is no currently available activity to add a fee.<br>Please, create an activity first."), AssumaConfigManager::Warning );
	}
	AddFeeGroupBox->hide();
	ActionGroupBox->show();
}

void AssumaAssociationSettings::on_addFeeCancelButton_clicked(){
	// We have to clear the form here
	AddFeeGroupBox->hide();
	ActionGroupBox->show();
	addFeeCoursePerWeekSpinBox->setValue(0);
	addFeeFeeLineEdit->setText(0) ;
}

void AssumaAssociationSettings::on_removeFeeButton_clicked(){
	coursesTabWidget->setEnabled(false);
	removeCurrentFeeGroupBox->show();
	ActionGroupBox->hide();
}

void AssumaAssociationSettings::on_removeCurrentFeeConfirmButton_clicked(){
	QTreeWidget* tw = currentCoursesTreeWidget();
// 	qDebug() << "Current tree widget is " << tw;
	if(tw){
		QTreeWidgetItem *it = tw->currentItem();
		if(it){
			tw->takeTopLevelItem(tw->indexOfTopLevelItem(it));
			if( m_saveOnChange )
				AssumaCM->setConfigToken( AssumaConfigManager::CoursesList, serializeCourses() );
		}
		else
			AssumaMW->showPassiveMessage( tr("There is no currently selected fee.<br>Please, select a fee first."), AssumaConfigManager::Warning );
	}
	removeCurrentFeeGroupBox->hide();
	ActionGroupBox->show();
	coursesTabWidget->setEnabled(true);
}

void AssumaAssociationSettings::on_removeCurrentFeeCancelButton_clicked(){
	removeCurrentFeeGroupBox->hide();
	ActionGroupBox->show();
	coursesTabWidget->setEnabled(true);
}

// End Fee management

// Start Activity management ( add and remove activities )

void AssumaAssociationSettings::on_addActivityButton_clicked(){
	AddActivityGroupBox->show();
	ActionGroupBox->hide();
}

void AssumaAssociationSettings::on_addActivityAddButton_clicked(){
	addActivity( addActivityLineEdit->text() );
	if( m_saveOnChange )
		AssumaCM->setConfigToken( AssumaConfigManager::CoursesList, serializeCourses() );
	on_addActivityCancelButton_clicked();
}

void AssumaAssociationSettings::addActivity(const QString title){
	QWidget *widget = new QWidget();
	QGridLayout *gridLayout = new QGridLayout(widget);
	QTreeWidget *tw = new QTreeWidget(widget);
	tw->setItemDelegate( new AssumaTreeWidgetItemEditorDelegate(tw) );
	QStringList strl;
	strl << tr("Course(s) per week") << tr("Fee");
	tw->setHeaderLabels(strl);
	tw->setAlternatingRowColors(true);
	tw->setSortingEnabled(true);
	tw->setAnimated(true);
	tw->setAllColumnsShowFocus(true);
	tw->setColumnCount(2);
	gridLayout->addWidget(tw, 0, 0, 1, 1);
	coursesTabWidget->addTab(widget, title );
	tw->sortItems( 0, Qt::AscendingOrder );
	for( int i = 0; i < tw->headerItem()->columnCount(); i++){
		tw->resizeColumnToContents(i);
	}
}

void AssumaAssociationSettings::on_addActivityCancelButton_clicked(){
	addActivityLineEdit->setText("");
	AddActivityGroupBox->hide();
	ActionGroupBox->show();
}

void AssumaAssociationSettings::on_removeActivityButton_clicked(){
	coursesTabWidget->setEnabled(false);
	removeCurrentActivityLabel->setText( tr( "Are you sure you want to remove the <strong>%1</strong> activity ?" ).arg( coursesTabWidget->tabText( coursesTabWidget->currentIndex() ) ) );
	removeCurrentActivityGroupBox->show();
	ActionGroupBox->hide();
}

void AssumaAssociationSettings::on_removeCurrentActivityCancelButton_clicked(){
	removeCurrentActivityGroupBox->hide();
	ActionGroupBox->show();
	coursesTabWidget->setEnabled(true);
}

void AssumaAssociationSettings::on_removeCurrentActivityConfirmButton_clicked(){
	coursesTabWidget->removeTab( coursesTabWidget->currentIndex() );
	removeCurrentActivityGroupBox->hide();
	ActionGroupBox->show();
	coursesTabWidget->setEnabled(true);
	AssumaMW->showPassiveMessage( tr("Activity successfully removed."), AssumaConfigManager::Info );
	if( m_saveOnChange )
		AssumaCM->setConfigToken( AssumaConfigManager::CoursesList, serializeCourses() );
}

// Stop Activity management

// Start Contacts management

void AssumaAssociationSettings::on_addEmailPushButton_clicked(){
	contactsActionGroupBox->hide();
	contactsAddEmailgroupBox->show();
}

void AssumaAssociationSettings::on_removeEmailPushButton_clicked(){
	contactsActionGroupBox->hide();
	contactsRemoveEmailgroupBox->show();
	contactsListWidget->setEnabled(false);
}

void AssumaAssociationSettings::on_addEmailCancelPushButton_clicked(){
	contactsAddEmailgroupBox->hide();
	contactsAddEmailLineEdit->setText("");
	contactsActionGroupBox->show();
}

void AssumaAssociationSettings::on_addEmailAddPushButton_clicked(){
	if( contactsAddEmailLineEdit->text().isEmpty() ) {
                AssumaMW->showPassiveMessage( tr("Please enter an email adress."), AssumaConfigManager::Warning );
		on_addEmailCancelPushButton_clicked();
	}
	else{
		contactsListWidget->addItem( contactsAddEmailLineEdit->text() );
                if( m_saveOnChange )
                        AssumaCM->setConfigToken( AssumaConfigManager::ContactsList, serializeContacts() );
		on_addEmailCancelPushButton_clicked();
	}
}

void AssumaAssociationSettings::on_removeEmailCancelPushButton_clicked(){
	contactsRemoveEmailgroupBox->hide();
	contactsActionGroupBox->show();
	contactsListWidget->setEnabled(true);
}

void AssumaAssociationSettings::on_removeEmailConfirmPushButton_clicked(){
	QListWidgetItem * it = contactsListWidget->currentItem();
	if( it ){
		contactsListWidget->takeItem( contactsListWidget->row(it) );
                if( m_saveOnChange )
                        AssumaCM->setConfigToken( AssumaConfigManager::ContactsList, serializeContacts() );
	}
	else {
		AssumaMW->showPassiveMessage( tr("There is no currently selected email adress.<br>Please, select one first."), AssumaConfigManager::Warning );
	}
	on_removeEmailCancelPushButton_clicked();
}

// Stop Contacts management

// Updating specific labels' translation

void AssumaAssociationSettings::updateTranslations(){
	QStringList strl;
	strl << tr("Course(s) per week") << tr("Fee");
	for( int i = 0; i < coursesTabWidget->count(); i++ ){
		QTreeWidget *tw = coursesTreeWidget(i);
		if( tw )
			tw->setHeaderLabels(strl);
	}
	
	customFieldsTable->setHorizontalHeaderLabels( QStringList() << tr("Field id") << tr("Field name") << tr("Field type") << tr("\"Ok\" label") << tr("\"Not ok\" label") << tr("Value list") );
	retranslateUi(this);
	
	if (customFieldsTable->columnCount() < 6)
            customFieldsTable->setColumnCount(6);
        QFont font;
        font.setPointSize(11);
        font.setBold(true);
        font.setWeight(75);
        QTableWidgetItem *__qtablewidgetitem = new QTableWidgetItem();
        __qtablewidgetitem->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
        __qtablewidgetitem->setFont(font);
        customFieldsTable->setHorizontalHeaderItem(0, __qtablewidgetitem);
        QTableWidgetItem *__qtablewidgetitem1 = new QTableWidgetItem();
        __qtablewidgetitem1->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
        __qtablewidgetitem1->setFont(font);
        customFieldsTable->setHorizontalHeaderItem(1, __qtablewidgetitem1);
        QTableWidgetItem *__qtablewidgetitem2 = new QTableWidgetItem();
        __qtablewidgetitem2->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
        __qtablewidgetitem2->setFont(font);
        customFieldsTable->setHorizontalHeaderItem(2, __qtablewidgetitem2);
        QTableWidgetItem *__qtablewidgetitem3 = new QTableWidgetItem();
        __qtablewidgetitem3->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
        __qtablewidgetitem3->setFont(font);
        customFieldsTable->setHorizontalHeaderItem(3, __qtablewidgetitem3);
        QTableWidgetItem *__qtablewidgetitem4 = new QTableWidgetItem();
        __qtablewidgetitem4->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
        __qtablewidgetitem4->setFont(font);
        customFieldsTable->setHorizontalHeaderItem(4, __qtablewidgetitem4);
        QTableWidgetItem *__qtablewidgetitem5 = new QTableWidgetItem();
        __qtablewidgetitem5->setTextAlignment(Qt::AlignHCenter|Qt::AlignVCenter|Qt::AlignCenter);
        __qtablewidgetitem5->setFont(font);
        customFieldsTable->setHorizontalHeaderItem(5, __qtablewidgetitem5);
	
        QTableWidgetItem *___qtablewidgetitem = customFieldsTable->horizontalHeaderItem(0);
        ___qtablewidgetitem->setText(QApplication::translate("AssumaAssociationSettings", "Field id", 0, QApplication::UnicodeUTF8));
        ___qtablewidgetitem->setToolTip(QApplication::translate("AssumaAssociationSettings", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">The field id <span style=\" font-weight:600; text-decoration: underline;\">must be unique</span>. It is an id which identify it among all the field. It is also used to get the right value from the subscribers database. A bad field id might corrupt your subscribers database.</p>\n"
"<p style=\"-qt-paragraph-type:empty; margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"></p>\n"
"<p style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\">Usually"
                        ", an integer value is a good choice.</p></body></html>", 0, QApplication::UnicodeUTF8));
        QTableWidgetItem *___qtablewidgetitem1 = customFieldsTable->horizontalHeaderItem(1);
        ___qtablewidgetitem1->setText(QApplication::translate("AssumaAssociationSettings", "Field name", 0, QApplication::UnicodeUTF8));
        QTableWidgetItem *___qtablewidgetitem2 = customFieldsTable->horizontalHeaderItem(2);
        ___qtablewidgetitem2->setText(QApplication::translate("AssumaAssociationSettings", "Field type", 0, QApplication::UnicodeUTF8));
        QTableWidgetItem *___qtablewidgetitem3 = customFieldsTable->horizontalHeaderItem(3);
        ___qtablewidgetitem3->setText(QApplication::translate("AssumaAssociationSettings", "\"Ok\" label", 0, QApplication::UnicodeUTF8));
        QTableWidgetItem *___qtablewidgetitem4 = customFieldsTable->horizontalHeaderItem(4);
        ___qtablewidgetitem4->setText(QApplication::translate("AssumaAssociationSettings", "\"Not ok\" label", 0, QApplication::UnicodeUTF8));
        QTableWidgetItem *___qtablewidgetitem5 = customFieldsTable->horizontalHeaderItem(5);
        ___qtablewidgetitem5->setText(QApplication::translate("AssumaAssociationSettings", "Value list", 0, QApplication::UnicodeUTF8));
        label_8->setText(QApplication::translate("AssumaAssociationSettings", "<!DOCTYPE HTML PUBLIC \"-//W3C//DTD HTML 4.0//EN\" \"http://www.w3.org/TR/REC-html40/strict.dtd\">\n"
"<html><head><meta name=\"qrichtext\" content=\"1\" /><style type=\"text/css\">\n"
"p, li { white-space: pre-wrap; }\n"
"</style></head><body style=\" font-family:'Sans Serif'; font-size:10pt; font-weight:400; font-style:normal;\">\n"
"<p align=\"center\" style=\" margin-top:0px; margin-bottom:0px; margin-left:0px; margin-right:0px; -qt-block-indent:0; text-indent:0px;\"><span style=\" font-size:14pt; font-weight:600; color:#55aaff;\">Preview</span></p></body></html>", 0, QApplication::UnicodeUTF8));
	
}

void AssumaAssociationSettings::on_trimesterSpinBox_valueChanged(int value){
	if( m_saveOnChange )
		AssumaCM->setConfigToken( AssumaConfigManager::TrimesterCount, value );
}

void AssumaAssociationSettings::on_taxSpinBox_valueChanged(int value){
	if( m_saveOnChange )
		AssumaCM->setConfigToken( AssumaConfigManager::StaticCost, value );
}

void AssumaAssociationSettings::on_AssociationNameLineEdit_textChanged( const QString & value ){
	if( m_saveOnChange )
		AssumaCM->setConfigToken( AssumaConfigManager::AssociationName, value );
}
