
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_configmanager.h"
#include <QtCore/QSettings>
#include <QtCore/QMutexLocker>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

AssumaConfigManager *AssumaConfigManager::m_singleInstance = NULL;

AssumaConfigManager::AssumaConfigManager() : m_mutex(QMutex::Recursive){
	m_configuration.empty();
}

AssumaConfigManager* AssumaConfigManager::instance(){
	if ( NULL == m_singleInstance ){
#ifdef ASSUMA_DEBUG
		qDebug() << "\n\nWARNING: New instance of AssumaConfigManager created !!\n";
#endif
		m_singleInstance = new AssumaConfigManager ;
	}
	return m_singleInstance;
}

QString AssumaConfigManager::tokenToString( Token token ){
	if( token == TrimesterCount ){
		return QString("configTrimesterCount");
	}
	else if( token == StaticCost ){
		return QString("configStaticCost");
	}
	else if( token == Federation ){
		return QString("configFederation");
	}
	else if( token == UpdateServer ){
		return QString("configUpdateServer");
	}
	else if( token == UpdatesOnStart ){
		return QString("configUpdatesOnStart");
	}
	else if( token == Currency ){
		return QString("configCurrency");
	}
	else if( token == CurrenciesList ){
		return QString("configCurrenciesList");
	}
	else if( token == ContactsList ){
		return QString("configContactsList");
	}
	else if( token == CoursesList ){
		return QString("configCoursesList");
	}
	else if( token == Language ){
		return QString("configLanguage");
	}
	else if( token == Encoding ){
		return QString("configEncoding");
	}
	else if( token == AssociationName ){
		return QString("configAssociationName");
	}
	else if( token == Email ){
		return QString("configEmail");
	}
	else if( token == SmtpServer ){
		return QString("configSmtpServer");
	}
	else if( token == SmtpLogin ){
		return QString("configSmtpLogin");
	}
	else if( token == SmtpPassword ){
		return QString("configSmtpPassword");
	}
	else if( token == ProxyServer ){
		return QString("configProxyServer");
	}
	else if( token == ProxyLogin ){
		return QString("configProxyLogin");
	}
	else if( token == ProxyPassword ){
		return QString("configProxyPassword");
	}
	else if( token == ProxyEnabled ){
		return QString("configProxyEnabled");
	}
	else if( token == DataDir ){
		return QString("configDataDir");
	}
	else if( token == FederationList ){
		return QString("configFederationList");
	}
	else if( token == LanguageList ){
		return QString("configLanguageList");
	}
	else if( token == Country ){
		return QString("config");
	}
	else if( token == ActivatedAddonList ){
		return QString("configActivatedAddonList");
	}
	else if( token == Theme ){
		return QString("configTheme");
	}
	else if( token == MainWindowLastDatabase ){
		return QString("last_database");
	}
	else if( token == MainWindowGeometry ){
		return QString("mw_geometry");
	}
	else if( token == MainWindowSplitterGeometry ){
		return QString("splitter_geometry");
	}
	else{
		return QString();
	}
}

void AssumaConfigManager::setConfigToken( Token token, const QVariant & data){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigManager::setConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	if( data != m_configuration.value( tokenToString(token) ) ){
		m_configuration.insert( tokenToString(token), data );
		emit( configurationTokenChanged(token, data) );
	}
}

void AssumaConfigManager::setExternalConfigToken( const QString & token, const QVariant & data ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigManager::setExternalConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	if( data != m_configuration.value( "external/"+token ) ){
		m_configuration.insert( "external/"+token, data );
		emit( externalConfigurationTokenChanged(token,data) );
	}
}

void AssumaConfigManager::setTmpConfigToken( const QString & token, const QVariant & data ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigManager::setTmpConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	if( data != m_configuration.value( "tmp/"+token ) ){
		m_configuration.insert( "tmp/"+token, data );
		emit( tmpConfigurationTokenChanged(token,data) );
	}
}

QVariant AssumaConfigManager::getConfigToken(Token token){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigManager::getConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	return m_configuration.value( tokenToString(token) );
}

QVariant AssumaConfigManager::getExternalConfigToken(const QString & token){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigManager::getExternalConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	return m_configuration.value( "external/"+token );
}

QVariant AssumaConfigManager::getTmpConfigToken(const QString & token){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigManager::getTmpConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	return m_configuration.value( "tmp/"+token );
}
