
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_configwidget.h"

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QDir>
#include <QTextCodec>
#include <QTranslator>
#include <QHttp>
#include <QUrl>
#include <QFileDialog>
#include <QDir>
#include <QFileSystemWatcher>
#include <QtCore/QMutexLocker>
#include "assuma_treewidgetitemeditordelegate.h"
#include "assuma_addoninstance.h"
#include "assuma_addonengine.h"
#include "federationhandler.h"
#include "assuma_mainwindow.h"

#ifndef ASSUMA_DATADIR
#define ASSUMA_DATADIR "/usr/share/AssociationSubscribersManager"
#endif

#ifndef ASSUMA_DOCDIR
#define ASSUMA_DOCDIR "/usr/share/AssociationSubscribersManager/doc"
#endif

#ifndef ASSUMA_IMGDIR
#define ASSUMA_IMGDIR "/usr/share/AssociationSubscribersManager/images"
#endif

#ifndef ASSUMA_THEMESDIR
#define ASSUMA_THEMESDIR "/usr/share/AssociationSubscribersManager/themes"
#endif

AssumaConfigWidget::AssumaConfigWidget(QWidget *parent):QTabWidget(parent), m_mutex(QMutex::Recursive){
	m_isInitializing = true;
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigWidget::AssumaConfigWidget() : creation begin\n";
#endif
	_addonsList.empty();
	setupUi(this);
#ifndef ASSUMA_WITH_UPDATER
	removeTab(3);
#endif
// 	removeCurrentActivityGroupBox->hide();
// 	removeCurrentFeeGroupBox->hide();
// 	AddFeeGroupBox->hide();
// 	AddActivityGroupBox->hide();
// 	contactsAddEmailgroupBox->hide();
// 	contactsRemoveEmailgroupBox->hide();
	addonDeletionGroupBox->hide();
// 	federationEditorFrame->hide();
// 	statusLabel->clear();
// 	contactsListWidget->setItemDelegate( new AssumaTreeWidgetItemEditorDelegate );
// 	addFeeFeeLineEdit->setValidator( new QDoubleValidator(addFeeFeeLineEdit) );
// 	if( tabWidget->tabText(0) == tr("All courses") && coursesTreeWidget(0)->topLevelItemCount() == 0 )
// 		tabWidget->removeTab(0);
	QList<QByteArray> codecs(QTextCodec::availableCodecs());
	foreach ( QByteArray ba, codecs){
		encodingComboBox->addItem( QString(ba) );
	}
	http = new QHttp(this);
// 	licenseTableWidget->sortByColumn(0,Qt::AscendingOrder);
	
	// Loading themes list and adding filesystem watching to automatically detect new themes.
	QString themeDirectory;
	// WARNING: This is a workaround to avoid the problem of multiple instanciations of AssumaMainWindow through the use of AssumaMW.
	if( parentWidget() && qobject_cast<AssumaMainWindow *>(parentWidget()->parentWidget()) ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaConfigWidget::AssumaConfigWidget() : trying to get the theme directory from parent window\n";
#endif
		themeDirectory = qobject_cast<AssumaMainWindow *>(parentWidget()->parentWidget())->getDirectory("themes");
	}
	else{
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaConfigWidget::AssumaConfigWidget() : using the built-in ASSUMA_THEMESDIR\n";
#endif
		themeDirectory = QString(ASSUMA_THEMESDIR);
	}
	loadThemeList(themeDirectory);
	QFileSystemWatcher *fsw = new QFileSystemWatcher;
	fsw->addPath(themeDirectory);
	connect(fsw,SIGNAL(directoryChanged ( const QString &)), this, SLOT(loadThemeList( const QString & )) );
	connect( themeComboBox, SIGNAL( currentIndexChanged ( const QString & ) ), this, SIGNAL( themeChanged( const QString & ) ) );
	connect(this, SIGNAL(currentChanged ( int )), this, SLOT( currentTabChanged(int) ) );
	connect( AssumaCM, SIGNAL(configurationTokenChanged( AssumaConfigManager::Token, const QVariant & )), this, SLOT( updateConfiguration( AssumaConfigManager::Token, const QVariant & ) ) );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigWidget::AssumaConfigWidget() : creation end\n";
#endif
	m_isInitializing = false;
}

void AssumaConfigWidget::updateConfiguration( AssumaConfigManager::Token token, const QVariant & value){
	setConfigToken(token, value.toString() );
}

void AssumaConfigWidget::currentTabChanged(int tab_index){
	if( tab_index == 1 ){
		updateAddonList();
		setConfigToken( AssumaConfigManager::ActivatedAddonList, AssumaCM->getConfigToken( AssumaConfigManager::ActivatedAddonList ).toString() );
	}
}

void AssumaConfigWidget::loadThemeList(const QString & themeDirectory){
#ifdef ASSUMA_DEBUG
	qDebug() << "\n\nAssumaConfigWidget::AssumaConfigWidget() : themes directory : " << themeDirectory<< "\n\n";
#endif
	QDir td( themeDirectory );
	themeComboBox->clear();
	themeComboBox->addItems( td.entryList(QDir::Dirs|QDir::NoDotAndDotDot) );
}

void AssumaConfigWidget::loadTheme( const QString & themeName ){
	if( !AssumaMW ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaConfigWidget::loadTheme() : WARNING we don't have access to AssumaMW ! \n\tUnable to load theme " << themeName;
#endif
		return ;
	}
	QString themeDirectory = AssumaMW->getDirectory("themes");
	QIcon icon;
	icon.addFile(AssumaMW->getImageFromTheme(themeName, "settings"), QSize(), QIcon::Normal, QIcon::Off);
	setWindowIcon(icon);
	// Constructing some commons icons
	QIcon icon_button_ok;
	icon_button_ok.addFile( AssumaMW->getImageFromTheme(themeName,"button-ok") , QSize(), QIcon::Normal, QIcon::Off);
	QIcon icon_button_remove;
	icon_button_remove.addFile( AssumaMW->getImageFromTheme(themeName,"button-remove") , QSize(), QIcon::Normal, QIcon::Off);
	removeCurrencyToolButton->setIcon( icon_button_remove );
	
	QIcon icon_button_add;
	icon_button_add.addFile( AssumaMW->getImageFromTheme(themeName,"button-add") , QSize(), QIcon::Normal, QIcon::Off);
	addCurrencyToolButton->setIcon( icon_button_add );
	
	QIcon icon_button_nok;
	icon_button_nok.addFile( AssumaMW->getImageFromTheme(themeName,"button-nok") , QSize(), QIcon::Normal, QIcon::Off);
	setTabIcon(0, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-general" ) ) );
	setTabIcon(1, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-addons" ) ) );
	setTabIcon(2, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-locale" ) ) );
#ifdef ASSUMA_WITH_UPDATER
	setTabIcon(3, QIcon( AssumaMW->getImageFromTheme(themeName, "tab-icon-update" ) ) );
#endif
	QIcon icon8;
	icon8.addFile( AssumaMW->getImageFromTheme(themeName, "button-erase" ) , QSize(), QIcon::Normal, QIcon::Off);
	confirmAddonDeletionButton->setIcon(icon8);
	deleteAddonPushButton->setIcon(icon8);
	QIcon icon11;
	icon11.addFile(AssumaMW->getImageFromTheme(themeName, "button-close" ), QSize(), QIcon::Normal, QIcon::Off);
	cancelAddonDeletionButton->setIcon(icon11);
	cancelAddonDeletionButton->setIconSize(QSize(22, 22));
	QIcon icon12;
	icon12.addFile(AssumaMW->getImageFromTheme(themeName, "update-addons" ), QSize(), QIcon::Normal, QIcon::Off);
	getNewAddonsPushButton->setIcon(icon12);
	getNewAddonsPushButton->setIconSize(QSize(22, 22));
	QIcon icon14;
	icon14.addFile(AssumaMW->getImageFromTheme(themeName, "folder-icon" ), QSize(), QIcon::Normal, QIcon::Off);
	dataDirToolButton->setIcon(icon14);
	
	pixLabelUpdates->setText( QString() );
	pixLabelLocales->setText( QString() );
	pixLabelGeneral->setText( QString() );
	pixLabelAddons->setText( QString() );
	
	pixLabelUpdates->setMaximumSize(QSize(35, 40));
	pixLabelLocales->setMaximumSize(QSize(35, 40));
	pixLabelGeneral->setMaximumSize(QSize(35, 40));
	pixLabelAddons->setMaximumSize(QSize(35, 40));
	
#ifdef ASSUMA_WITH_UPDATER
	pixLabelUpdates->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-update") ));
#endif
	pixLabelLocales->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-locale") ));
	pixLabelGeneral->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-general") ));
	pixLabelAddons->setPixmap(QPixmap( AssumaMW->getImageFromTheme(themeName,"tab-icon-addons") ));
        
}

void AssumaConfigWidget::setConfigToken( AssumaConfigManager::Token token, QString value) {
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigWidget::setConfigToken() : about to lock thread.";
#endif
	QMutexLocker locker(&m_mutex);
	QStringList tmp_list;
	QStringList activities;
	switch ( token ) {
#ifdef ASSUMA_WITH_UPDATER
		case AssumaConfigManager::UpdateServer : 
			if( value != updateServerLineEdit->text() )
				updateServerLineEdit->setText(value); 
			break;
		case AssumaConfigManager::UpdatesOnStart : 
			if( value == QString::number(1) )
				checkUpdatesOnStartCheckBox->setChecked(1);
			else
				checkUpdatesOnStartCheckBox->setChecked(0);
			break;
#endif
		case AssumaConfigManager::Language : languageComboBox->setCurrentIndex( languageComboBox->findText(value, Qt::MatchExactly | Qt::MatchCaseSensitive ) ); break ;
		case AssumaConfigManager::Encoding : encodingComboBox->setCurrentIndex( encodingComboBox->findText(value, Qt::MatchExactly | Qt::MatchCaseSensitive ) ); break ;
		case AssumaConfigManager::Email : 
			if( value != emailLineEdit->text() )
				emailLineEdit->setText( value ); 
			break;
		case AssumaConfigManager::SmtpServer : 
			if( value != smtpServerEdit->text() )
			smtpServerEdit->setText( value );
			break;
		case AssumaConfigManager::SmtpLogin : 
			if( value != smtpServerLoginEdit->text() )
				smtpServerLoginEdit->setText( value ); 
			break;
		case AssumaConfigManager::SmtpPassword : 
			if( value != smtpServerPasswordEdit->text() )
				smtpServerPasswordEdit->setText( value ); 
			break;
		case AssumaConfigManager::ProxyServer : 
			if( value != proxyServerLineEdit->text() )
				proxyServerLineEdit->setText( value ); 
			break;
		case AssumaConfigManager::ProxyLogin : 
			if( value != proxyLoginLineEdit->text() )
				proxyLoginLineEdit->setText( value ); 
			break;
		case AssumaConfigManager::ProxyPassword : 
			if( value != proxyPasswordLineEdit->text() )
				proxyPasswordLineEdit->setText( value ); 
			break;
		case AssumaConfigManager::ProxyEnabled : 
			if( value == QString::number(1) )
				proxyGroupBox->setChecked(true);
			else
				proxyGroupBox->setChecked(false);
			break;
		case AssumaConfigManager::DataDir : 
			if( value != dataDirLineEdit->text() )
				dataDirLineEdit->setText( value ); 
			if( ! value.isNull() ){
				for(int i=0; i < citiesComboBox->count(); i++){
					citiesComboBox->removeItem(i);
				}
				QDir dataDir( getConfigToken( AssumaConfigManager::DataDir ) + "/PostalCodes" );
				QStringList cfilters;
				QStringList tmp_cities_list;
				cfilters << "*.cities.xml";
				dataDir.setNameFilters(cfilters);
				foreach ( QString it, dataDir.entryList() ){
					it.remove(".cities.xml");
					tmp_cities_list << it;
				}
				citiesComboBox->addItems( tmp_cities_list );
				QFileSystemWatcher *fsw_addons = new QFileSystemWatcher;
				fsw_addons->addPath(getConfigToken(AssumaConfigManager::DataDir) + "/Addons" );
				connect(fsw_addons,SIGNAL(directoryChanged ( const QString &)), this, SLOT( updateAddonList() ) );
			}
			break;
		case AssumaConfigManager::LanguageList : 
			// Here value contains a list of languages (e.g: French;English;Swedish)
			// We split the string and add each language to the combobox.
			foreach (QString it, value.split(";")){
				// Each language can contains an icon file and we have to resplit the string.
				// e.g: :/images/french_flag.png,French;images/english_flag.png,English
				// ********* First is the icon name and second is the language name *********
				if( it.contains(",") && it.length() > 0 ){
					QStringList tmp = it.split(",");
					languageComboBox->addItem( QIcon( tmp.at(0) ), tmp.at(1) );
				} else {
					languageComboBox->addItem(it);
				}
			}
			break;
		case AssumaConfigManager::Country : citiesComboBox->setCurrentIndex( citiesComboBox->findText(value, Qt::MatchExactly | Qt::MatchCaseSensitive ) ); break ;
		case AssumaConfigManager::ActivatedAddonList :
			m_dontUpdateToken = true;
			foreach(QString addon, value.split(";") ){
				QList<QTreeWidgetItem *> addonsList = addonListTreeWidget->findItems(addon,Qt::MatchExactly|Qt::MatchCaseSensitive,1);
				if( addonsList.size() > 0 ){
					addonsList.at(0)->setCheckState(0,Qt::Checked);
				}
			}
			m_dontUpdateToken = false;
			break;
		case AssumaConfigManager::CurrenciesList :
			m_dontUpdateToken = true;
			/* Removing all items in the currency box */
			while( currencyComboBox->count() > 0 ){
				currencyComboBox->removeItem(0);
			}
			/* And then adding all the serialized ones */
			foreach( QString str , value.split(";") ){
				currencyComboBox->addItem(str);
			}
			m_dontUpdateToken = false;
			break;
		case AssumaConfigManager::Currency : currencyComboBox->setCurrentIndex( currencyComboBox->findText(value, Qt::MatchExactly | Qt::MatchCaseSensitive ) );break;
		case AssumaConfigManager::Theme : themeComboBox->setCurrentIndex( themeComboBox->findText(value, Qt::MatchExactly | Qt::MatchCaseSensitive ) ); break ;
		default : break;
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaConfigWidget::setConfigToken() : about to unlock thread.";
#endif
}

QString AssumaConfigWidget::getConfigToken( AssumaConfigManager::Token token ){
	QMutexLocker locker(&m_mutex);
	QString ret;
	QStringList tmp_list;
	switch ( token ) {
#ifdef ASSUMA_WITH_UPDATER
		case AssumaConfigManager::UpdateServer : ret = updateServerLineEdit->text(); break;
		case AssumaConfigManager::UpdatesOnStart : 
			if( checkUpdatesOnStartCheckBox->isChecked() )
				ret = QString::number(1);
			else
				ret = QString::number(0);
			break;
#endif
		case AssumaConfigManager::Language : ret = languageComboBox->currentText(); break;
		case AssumaConfigManager::Encoding : ret = encodingComboBox->currentText(); break;
		case AssumaConfigManager::Email : ret = emailLineEdit->text(); break;
		case AssumaConfigManager::SmtpServer : ret = smtpServerEdit->text(); break;
		case AssumaConfigManager::SmtpLogin : ret = smtpServerLoginEdit->text(); break;
		case AssumaConfigManager::SmtpPassword : ret = smtpServerPasswordEdit->text(); break;
		case AssumaConfigManager::ProxyServer : ret = proxyServerLineEdit->text(); break;
		case AssumaConfigManager::ProxyLogin : ret = proxyLoginLineEdit->text(); break;
		case AssumaConfigManager::ProxyPassword : ret = proxyPasswordLineEdit->text(); break;
		case AssumaConfigManager::ProxyEnabled : 
			if( proxyGroupBox->isChecked() )
				ret = QString::number(1);
			else
				ret = QString::number(0);
			break;
		case AssumaConfigManager::DataDir : ret = dataDirLineEdit->text(); break;
		case AssumaConfigManager::Country : ret = citiesComboBox->currentText(); break;
		case AssumaConfigManager::ActivatedAddonList :
			ret = "";
			tmp_list.clear();
			for ( int i=0; i< addonListTreeWidget->topLevelItemCount(); i++ ){
				QTreeWidgetItem *item = addonListTreeWidget->topLevelItem(i);
				if( item->checkState(0) == Qt::Checked ){
					tmp_list << item->text(1);
				}
			}
			ret = tmp_list.join(";");
			break;
		case AssumaConfigManager::Currency : ret = currencyComboBox->currentText(); break;
		case AssumaConfigManager::CurrenciesList :
			ret="";
			tmp_list.clear();
			if( currencyComboBox->findText( currencyComboBox->currentText() , Qt::MatchExactly | Qt::MatchCaseSensitive ) == -1 )
				tmp_list << currencyComboBox->currentText();
			for( int i=0; i < currencyComboBox->count(); i++ ){
// 				qDebug() << "AssumaConfigWidget::getConfigToken( CurrenciesList ) -> item:" << currencyComboBox->itemText(i);
				tmp_list << currencyComboBox->itemText(i);
			}
			ret = tmp_list.join(";");
			break;
		case AssumaConfigManager::Theme : ret = themeComboBox->currentText(); break;
		default : break;
	}
	return ret;
}

// Start General management

void AssumaConfigWidget::on_languageComboBox_currentIndexChanged ( int index ){
	if( ! m_isInitializing ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaConfigWidget -> Language have been changed ! Trying to load the corresponding file for locale " << languageComboBox->itemText(index);
#endif
		emit( languageChanged(languageComboBox->itemText(index)) );
	}
}

void AssumaConfigWidget::on_dataDirToolButton_clicked(){
	QString dir = dataDirLineEdit->text();
	if( dir.isEmpty() )
		dir = QDir::currentPath();
	dir = QFileDialog::getExistingDirectory ( this, tr("Select a directory"), dataDirLineEdit->text() );
	if( ! dir.isEmpty() )
		dataDirLineEdit->setText(dir);
}

// Stop General management

// Start currencies management

void AssumaConfigWidget::on_addCurrencyToolButton_clicked(){
	if( ! addCurrencyLineEdit->text().isEmpty() ){
		currencyComboBox->addItem( addCurrencyLineEdit->text() );
		addCurrencyLineEdit->clear();
		AssumaCM->setConfigToken( AssumaConfigManager::CurrenciesList, getConfigToken( AssumaConfigManager::CurrenciesList ) );
	}
}

void AssumaConfigWidget::on_removeCurrencyToolButton_clicked(){
	QString item = currencyComboBox->currentText();
	currencyComboBox->removeItem( currencyComboBox->currentIndex() );
	AssumaMW->showPassiveMessage( tr("Currency \"%1\" have been removed.").arg(item) );
}

// Stop currencies management

// Start Addons management

void AssumaConfigWidget::setLoadedAddonsList(QList<AssumaAddonInstance*> list){
	_addonsList = list;
	updateAddonList();
}

void AssumaConfigWidget::updateAddonList(){
	m_dontUpdateToken = true;
	QString _addonsDirectory = getConfigToken(AssumaConfigManager::DataDir) + "/Addons" ;
	AssumaAddonEngine engine(_addonsDirectory );
	QDir pDir( _addonsDirectory );
	addonListTreeWidget->clear();
	foreach ( QString subDir, pDir.entryList( QDir::Dirs | QDir::NoDotAndDotDot ) ){
		QTreeWidgetItem *item = new QTreeWidgetItem(addonListTreeWidget);
// 		qDebug() << "AssumaConfigWidget::updateAddonList() -> loading addon's description:" << subDir;
		AssumaAddonInstance addon = engine.loadAddon(subDir);
		item->setCheckState(0,Qt::Unchecked);
		if( AssumaMW )
			item->setIcon(0, QIcon( AssumaMW->getImageFromTheme("addon-inactivate-icon") ) );
		foreach( AssumaAddonInstance *_addon, _addonsList ){
			if( _addon->name() == addon.name() ){
				if( AssumaMW )
					item->setIcon(0, QIcon(AssumaMW->getImageFromTheme("addon-activate-icon")) );
				item->setCheckState(0,Qt::Checked);
				break;
			}
		}
		if( addon.name().isEmpty() && addon.version().isEmpty() && addon.type().isEmpty() ){
			item->setText(1, subDir );
			item->setText(3, tr("This addon could not be load correctly, you should probably remove it.") );
		}
		else{
			item->setText(1, addon.name() );
			item->setText(2, addon.version() );
			item->setText(3, addon.description() );
			item->setText(4, addon.author() );
			item->setText(5, addon.license() );
		}
	}
	for( int i = 0; i < addonListTreeWidget->headerItem()->columnCount(); i++){
		addonListTreeWidget->resizeColumnToContents(i);
	}
	m_dontUpdateToken = false;
}

void AssumaConfigWidget::on_deleteAddonPushButton_clicked(){
	if( addonListTreeWidget->currentItem() ){
		addonListTreeWidget->setEnabled(false);
		addonActionsGroupBox->hide();
		addonDeletionGroupBox->show();
	}
}

void AssumaConfigWidget::on_cancelAddonDeletionButton_clicked(){
	addonDeletionGroupBox->hide();
	addonActionsGroupBox->show();
	addonListTreeWidget->setEnabled(true);
}

void AssumaConfigWidget::on_confirmAddonDeletionButton_clicked(){
	QTreeWidgetItem *item = addonListTreeWidget->currentItem();
	QString _addonDirectory = getConfigToken(AssumaConfigManager::DataDir) + "/Addons/" + item->text(1) ;
	if( removeDirectory( _addonDirectory ) ){
		emit( configMessage( tr("Addon successfully deleted.") , AssumaConfigManager::Info ) );
		updateAddonList();
	}
	else
		emit( configMessage( tr("An error occured.<br>Addon cannot be delete.") , AssumaConfigManager::Warning ) );
	on_cancelAddonDeletionButton_clicked();
}

bool AssumaConfigWidget::removeDirectory(const QString & dir){
	QDir directory( dir );
	// Deleting files in the directory
	foreach (QString file, directory.entryList(QDir::Files) ){
// 		qDebug() << "AssumaConfigWidget::removeDirectory(" << dir << ") -> removing file:" << dir + "/" + file;
		directory.remove( dir + "/" + file );
	}
	// Recursing on directories
	foreach (QString d, directory.entryList(QDir::AllDirs|QDir::NoDotAndDotDot) ){
// 		qDebug() << "AssumaConfigWidget::removeDirectory(" << dir << ") -> recursing on directory:" << dir + "/" + d;
		removeDirectory( dir + "/" + d );
	}
	// Remove the directory
// 	qDebug() << "AssumaConfigWidget::removeDirectory(" << dir << ") -> removing directory:" << dir;
	directory.cdUp();
	return directory.rmdir(dir);
}

// Stop Addons management

// Start changes forwarding to the config manager

void AssumaConfigWidget::on_emailLineEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::Email, text );
}

void AssumaConfigWidget::on_updateServerLineEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::UpdateServer, text );
}

void AssumaConfigWidget::on_checkUpdatesOnStartCheckBox_stateChanged ( int state ){
	if( state == Qt::Checked )
		AssumaCM->setConfigToken( AssumaConfigManager::UpdatesOnStart, 1 );
	else
		AssumaCM->setConfigToken( AssumaConfigManager::UpdatesOnStart, 0 );
}

void AssumaConfigWidget::on_languageComboBox_currentIndexChanged ( const QString & text ) {
	if( text != AssumaCM->getConfigToken(AssumaConfigManager::Language).toString() )
		AssumaCM->setConfigToken( AssumaConfigManager::Language, text );
}

void AssumaConfigWidget::on_encodingComboBox_currentIndexChanged ( const QString & text ) {
	if( text != AssumaCM->getConfigToken(AssumaConfigManager::Encoding).toString() )
		AssumaCM->setConfigToken( AssumaConfigManager::Encoding, text );
}

void AssumaConfigWidget::on_smtpServerEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::SmtpServer, text );
}

void AssumaConfigWidget::on_smtpServerLoginEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::SmtpLogin, text );
}

void AssumaConfigWidget::on_smtpServerPasswordEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::SmtpPassword, text );
}

void AssumaConfigWidget::on_proxyServerLineEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyServer, text );
}

void AssumaConfigWidget::on_proxyLoginLineEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyLogin, text );
}

void AssumaConfigWidget::on_proxyPasswordLineEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyPassword, text );
}

void AssumaConfigWidget::on_proxyGroupBox_toggled ( bool on ) {
	QString enabled = "0";
	if( on )
		enabled = "1";
	if( enabled != AssumaCM->getConfigToken(AssumaConfigManager::ProxyEnabled).toString() )
		AssumaCM->setConfigToken( AssumaConfigManager::ProxyEnabled, enabled );
}

void AssumaConfigWidget::on_dataDirLineEdit_textChanged(const QString & text){
	AssumaCM->setConfigToken( AssumaConfigManager::DataDir, text );
	updateAddonList();
}

void AssumaConfigWidget::on_citiesComboBox_currentIndexChanged ( const QString & text ){
	if( text != AssumaCM->getConfigToken(AssumaConfigManager::Country).toString() )
		AssumaCM->setConfigToken( AssumaConfigManager::Country, text );
}

void AssumaConfigWidget::on_addonListTreeWidget_itemChanged ( QTreeWidgetItem * /*item*/, int /*column*/ ){
	if( m_dontUpdateToken )
		return;
	QString list = getConfigToken( AssumaConfigManager::ActivatedAddonList );
	if( AssumaCM->getConfigToken( AssumaConfigManager::ActivatedAddonList ).toString() != list )
		AssumaCM->setConfigToken( AssumaConfigManager::ActivatedAddonList, list );
}

void AssumaConfigWidget::on_currencyComboBox_currentIndexChanged ( const QString & text){
	if( text != AssumaCM->getConfigToken( AssumaConfigManager::Currency ).toString() )
		AssumaCM->setConfigToken( AssumaConfigManager::Currency, text );
}

void AssumaConfigWidget::on_themeComboBox_currentIndexChanged ( const QString & text){
	if( text != AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() )
		AssumaCM->setConfigToken( AssumaConfigManager::Theme, text );
}

// Stop changes forwarding to the config manager
