#include "assuma_effect.h"

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QTimer>

AssumaEffect::AssumaEffect( QObject *parent, QWidget *widget, Effects effect) : QObject(parent) {
	m_effect = effect;
	m_widget = widget;
	m_timeLine = new QTimeLine(2500);
	m_timeLine->setDirection( QTimeLine::Forward );
	m_timeLine->setCurveShape(QTimeLine::LinearCurve);//QTimeLine::EaseInCurve
// 	connect( m_timeLine, SIGNAL( valueChanged ( qreal ) ), this, SLOT( updateEffect(qreal) ) );
	connect( m_timeLine, SIGNAL(frameChanged(int)), this, SLOT( updateEffect(int) ) );
	connect( m_timeLine, SIGNAL( finished() ), this, SLOT( effectFinished() ) );
	m_transform = new QTransform;
	m_reversing = true;
	m_timeBeforeReversing = 5000;
	m_closeWidgetOnFinish = true;
	m_wasCancelled = false;
	m_vSizeDelta=30;
}

AssumaEffect::Effects AssumaEffect::currentEffect(){
	return m_effect;
}

QWidget* AssumaEffect::widget(){
	return m_widget;
}

bool AssumaEffect::isReversing(){
	return m_reversing;
}

int AssumaEffect::timeBeforeReversing(){
	return m_timeBeforeReversing;
}

int AssumaEffect::vSizeDelta(){
	return m_vSizeDelta;
}

bool AssumaEffect::wasCancelled(){
	return m_wasCancelled;
}

// Slots definition
void AssumaEffect::setEffect(Effects effect_name){
	m_effect = effect_name;
}

void AssumaEffect::doEffect(){
	if( m_wasCancelled )
		return;
	if( m_timeLine->state() == QTimeLine::Running )
		return;
	m_timeLine->stop();
	if( m_timeLine->currentTime() > 0 )
		m_timeLine->setCurrentTime(0);
	
	if( m_effect == SLIDE_FROM_BOTTOM ){
		if( m_widget && m_widget->parentWidget() ){
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> moving widget to 0," << ((QWidget *)m_widget->parentWidget())->height();
#endif
			m_widget->move( m_widget->pos().x() ,  ((QWidget *)m_widget->parentWidget())->height() );
			if( ! m_widget->isVisible() )
				m_widget->setVisible(true);
			m_timeLine->setDuration(750);
			m_timeLine->setFrameRange(0, m_widget->height() );
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> timeline's frame range is 0," << m_widget->height();
#endif
		}
	}
	else if( m_effect == GROW_VERTICALLY ){
		if( m_widget ){
			m_timeLine->setDuration(500);
			m_timeLine->setFrameRange(0, m_widget->height() );
			QRect g = m_widget->geometry();
			e_INIT_YPOS = g.y() ;
			e_INIT_HEIGHT = g.height();
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> GROW_VERTICALLY : init geometry" << g;
#endif
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> GROW_VERTICALLY : e_INIT_HEIGHT" << e_INIT_HEIGHT;
#endif
			m_widget->setGeometry( g.x(), g.y(),g.width(),0 );
		}
	}
	else if( m_effect == SLIM_VERTICALLY ){
		if( m_widget ){
			m_timeLine->setDuration(500);
			m_timeLine->setFrameRange(0, m_widget->height() );
			QRect g = m_widget->geometry();
			e_INIT_YPOS = g.y() ;
			e_INIT_HEIGHT = g.height();
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> SLIM_VERTICALLY : e_INIT_HEIGHT" << e_INIT_HEIGHT;
#endif
		}
	}
	else if( m_effect == SHRINK_VERTICALLY ){
		m_timeLine->setDuration(500);
		m_timeLine->setFrameRange(0, m_vSizeDelta );
		if( m_widget )
			e_INIT_HEIGHT = m_widget->height();
	}
	else if( m_effect == EXPAND_VERTICALLY ){
		m_timeLine->setDuration(500);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaEffect::doEffect() EXPAND_VERTICALLY => vSizeDelta=" << m_vSizeDelta;
#endif
		m_timeLine->setFrameRange(0, m_vSizeDelta );
		if( m_widget )
			e_INIT_HEIGHT = m_widget->height();
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaEffect::doEffect() EXPAND_VERTICALLY => e_INIT_HEIGHT=" << e_INIT_HEIGHT;
#endif
	}
	else if( m_effect == SHRINK_HORIZONTALLY ){
		m_timeLine->setDuration(500);
		m_timeLine->setFrameRange(0, m_vSizeDelta );
		if( m_widget )
			e_INIT_WIDTH = m_widget->width();
	}
	else if( m_effect == EXPAND_HORIZONTALLY ){
		m_timeLine->setDuration(500);
		m_timeLine->setFrameRange(0, m_vSizeDelta );
		if( m_widget )
			e_INIT_WIDTH = m_widget->width();
	}
	m_timeLine->start();
}

void AssumaEffect::reverseEffect(){
	if( m_wasCancelled )
		return;
	m_timeLine->stop();
	m_timeLine->setDirection( QTimeLine::Backward );
	setReversing(false); // We want the effect to run backward only once !
	m_timeLine->start();
}

void AssumaEffect::setWidget( QWidget * widget ){
	m_widget = widget;
}

void AssumaEffect::setReversing( bool rev ){
	m_reversing = rev;
}

void AssumaEffect::setTimeBeforeReversing(int time){
	m_timeBeforeReversing = time;
}

void AssumaEffect::setVSizeDelta(int delta){
	m_vSizeDelta = delta;
}

void AssumaEffect::cancel(){
	m_wasCancelled = true;
}

// Private slots definition

void AssumaEffect::updateEffect(int step){
	if( m_wasCancelled )
		return;
	if( m_widget && ! m_widget->isVisible() )
		m_widget->setVisible(true);
	if( m_effect == SLIDE_FROM_BOTTOM ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaEffect::updateEffect(" << step << ") effect is : SLIDE_FROM_BOTTOM";
#endif
		if( m_widget && m_widget->parentWidget() ){
			int ypos = ( ((QWidget *)m_widget->parentWidget())->height() - step);
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> moving widget to 0," << ypos;
#endif
			if(ypos >= ( ((QWidget *)m_widget->parentWidget())->height() - m_widget->height()) )
				m_widget->move( m_widget->pos().x() ,  ypos );
		}
	}
	else if( m_effect == GROW_VERTICALLY ){
		if( m_widget ){
			QRect g = m_widget->geometry();
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() -> GROW_VERTICALLY : m_widget->geometry() : " << g << " next geometry:" << g.x()<<", "<<e_INIT_YPOS -step<<" , "<<g.width()<<" , "<<step;
#endif
			m_widget->setGeometry( g.x(), e_INIT_YPOS -step , g.width() , step );
// 			m_widget->resize( m_widget->size().width(), step );
		}
	}
	else if( m_effect == SLIM_VERTICALLY ){
		if( m_widget ){
			QRect g = m_widget->geometry();
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::doEffect() ->SLIM_VERTICALLY : m_widget->geometry() : " << g << " next geometry:" << g.x()<<", "<<e_INIT_YPOS + step<<" , "<<g.width()<<" , "<<e_INIT_HEIGHT-step;
#endif
			m_widget->setGeometry( g.x(), e_INIT_YPOS + step , g.width() , e_INIT_HEIGHT-step );
// 			m_widget->resize( m_widget->size().width(), step );
		}
	}
	else if( m_effect == SHRINK_VERTICALLY ) {
		if( m_widget ){
			int height = e_INIT_HEIGHT - step;
			if( height > (e_INIT_HEIGHT - m_vSizeDelta) ){
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaEffect::updateEffect(" << step << ") -> SHRINK_VERTICALLY next height is : " << height;
#endif
				m_widget->resize( m_widget->size().width(), height );
			}
		}
	}
	else if( m_effect == EXPAND_VERTICALLY ) {
		if( m_widget ){
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaEffect::updateEffect(" << step << ") -> EXPAND_VERTICALLY next height is : " << (e_INIT_HEIGHT + step);
#endif
			m_widget->resize( m_widget->size().width(), e_INIT_HEIGHT + step );
		}
	}
	else if( m_effect == SHRINK_HORIZONTALLY ) {
		if( m_widget ){
			int width = e_INIT_WIDTH - step;
			if( width > (e_INIT_WIDTH - m_vSizeDelta) ){
				m_widget->resize( width, m_widget->size().height() );
			}
		}
	}
	else if( m_effect == EXPAND_HORIZONTALLY ) {
		if( m_widget ){
			m_widget->resize( e_INIT_WIDTH + step , m_widget->size().height() );
		}
	}
	emit( AssumaEffectWasUpdated() );
}

void AssumaEffect::effectFinished(){
#ifdef ASSUMA_DEBUG
	qDebug() << "effect finished !";
#endif
	if( m_reversing ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaEffect::effectFinished() -> Effect finished and we will wait " << m_timeBeforeReversing << " before reversing";
#endif
		QTimer::singleShot(m_timeBeforeReversing, this, SLOT(reverseEffect()));
	}
	else{
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaEffect::effectFinished() -> emetting the AssumaEffectFinished() signal";
#endif
		emit( AssumaEffectFinished() );
		cancel();
		if( m_effect == SLIM_VERTICALLY ){
			QRect g = m_widget->geometry();
			m_widget->setGeometry( g.x(), g.y() , g.width() , e_INIT_HEIGHT );
		}
	}
}
