
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_maildialog.h"
#include "assuma_mainwindow.h"
#include "mailsender.h"
#include <QFileInfo>
#include <QFileDialog>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QWidgetAction>
#include <QAction>
#include <QMessageBox>

AssumaMailDialog::AssumaMailDialog(QWidget *parent) : QDialog(parent){
	setupUi(this);
	attachements->setVisible(false);
	addEmailLineEdit = new QLineEdit ;
	connect(addEmailLineEdit, SIGNAL(returnPressed()), this, SLOT(addDestinationEmail() ) );
	addEmailMenu = new QMenu(addAdressButton);
	QWidgetAction *actionAddEmailEdit = new QWidgetAction(this) ;
	QAction *actionAddEmailList = new QAction(this);
	actionAddEmailList->setText( tr("Add an e-mail from the current database") );
	addEmailSubMenu = new QMenu(addAdressButton);
	QAction *actionNone = new QAction(this);
	actionNone->setText( tr("(none)") );
	actionNone->setEnabled(false);
	addEmailSubMenu->addAction(actionNone);
	actionAddEmailEdit->setDefaultWidget( addEmailLineEdit );
	actionAddEmailList->setMenu(addEmailSubMenu);
	addEmailMenu->addAction(actionAddEmailEdit);
	addEmailMenu->addAction(actionAddEmailList);
	addAdressButton->setMenu(addEmailMenu);
	_smtpServer = "localhost";
	_smtpLogin = "";
	_smtpPassword = "";
	_mailEncoding = "UTF-8";
	progressBar->hide();
	label->hide();
}

// Public accessors/slots

void AssumaMailDialog::loadTheme( const QString & themeName ){
	if( ! AssumaMW )
		return;
	QIcon icon;
	icon.addFile(AssumaMW->getImageFromTheme(themeName,"mail-generic"), QSize(), QIcon::Normal, QIcon::Off);
	setWindowIcon(icon);
	QIcon icon1;
	icon1.addFile(AssumaMW->getImageFromTheme(themeName,"button-add"), QSize(), QIcon::Normal, QIcon::Off);
	addAdressButton->setIcon(icon1);
	addAdressButton->setIconSize(QSize(22, 22));
	QIcon icon2;
	icon2.addFile(AssumaMW->getImageFromTheme(themeName,"button-erase"), QSize(), QIcon::Normal, QIcon::Off);
	delAdressButton->setIcon(icon2);
	delAdressButton->setIconSize(QSize(22, 22));
	delAttachementButton->setIcon(icon2);
	delAttachementButton->setIconSize(QSize(24, 24));
	attachementButton->setIcon(icon1);
	attachementButton->setIconSize(QSize(24, 24));
	QIcon icon3;
	icon3.addFile(AssumaMW->getImageFromTheme(themeName,"mail-send"), QSize(), QIcon::Normal, QIcon::Off);
	sendButton->setIcon(icon3);
	sendButton->setIconSize(QSize(24, 24));
	QIcon icon4;
	icon4.addFile(AssumaMW->getImageFromTheme(themeName,"button-close"), QSize(), QIcon::Normal, QIcon::Off);
	closeButton->setIcon(icon4);
	closeButton->setIconSize(QSize(24, 24));
}

void AssumaMailDialog::addEmail(const QString & email) {
	toListWidget->addItem(email);
}

void AssumaMailDialog::setSender(const QString & from){
	le_from->setText(from);
}

void AssumaMailDialog::setSubject(const QString & subject){
	le_subject->setText(subject);
}

//! \todo For 3.1 version, add a setHtmlBody() function.

void AssumaMailDialog::setBody(const QString & body){
	bodyEdit->setText(body);
}

void AssumaMailDialog::addAttachement(const QString & attachement){
	if( ! attachements->isVisible() )
		attachements->setVisible(true);
	QFileInfo info(attachement);
	QTreeWidgetItem *item = new QTreeWidgetItem ;
	item->setText( 0 , info.fileName() );
	qint64 size = info.size();
	QString unit = tr("B");
	if(size > 1024){
		size = size / 1024 ;
		unit = tr("KB");
		if( size > 1024 ){
			size = size / 1024;
			unit = tr("MB");
		}
	}
	item->setText( 1 , QString("%1 %2").arg(QString::number(size)).arg(unit) );
	item->setTextAlignment(1,Qt::AlignHCenter);
	item->setText( 2 , info.absoluteFilePath() );
	attachements->addTopLevelItem(item);
}

void AssumaMailDialog::setEmailList(const QStringList & list ){
	if( list.size() > 0 ){
		foreach ( QAction *action, addEmailSubMenu->actions() )
			addEmailSubMenu->removeAction(action);
	}
	foreach( QString str, list){
		QAction *action = new QAction(this);
		action->setText( str );
		connect( action, SIGNAL( triggered(bool) ), this, SLOT( addDestinationEmail(bool) ) );
		addEmailSubMenu->addAction(action);
	}
}

void AssumaMailDialog::setSmtpServer( const QString & smtpServer){
	_smtpServer = smtpServer;
}

void AssumaMailDialog::setAuth(const QString & login, const QString & password){
	_smtpLogin = login;
	_smtpPassword = password;
}

void AssumaMailDialog::setMailEncoding( const QString & encoding ){
	_mailEncoding = encoding;
}

// Private slots

void AssumaMailDialog::addDestinationEmail(bool /*trig*/){
	QAction *action = qobject_cast<QAction*>(sender());
	addEmail( action->text() );
}

void AssumaMailDialog::addDestinationEmail(){
	addEmail( addEmailLineEdit->text() );
	addEmailLineEdit->setText("");
}

void AssumaMailDialog::on_attachementButton_clicked(){
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open attachement"),".",tr("All files (*)"));
	if( fileName.size() > 0 )
		addAttachement(fileName);
}

void AssumaMailDialog::on_delAttachementButton_clicked(){
	attachements->takeTopLevelItem( attachements->indexOfTopLevelItem( attachements->currentItem() ) );
	if( attachements->topLevelItemCount() == 0 )
		attachements->setVisible(false);
}

void AssumaMailDialog::on_delAdressButton_clicked(){
	toListWidget->takeItem( toListWidget->currentRow() );
}

void AssumaMailDialog::on_sendButton_clicked() {
	sendButton->setEnabled(false);
	progressBar->reset();
	progressBar->setMaximum( toListWidget->count()+2 );
	progressBar->show();
	label->show();
	label->setText( tr("Initializing mail") );
	QStringList attachs;
	if( _smtpServer.isEmpty() )
		_smtpServer = "127.0.0.1";
	MailSender mail(_smtpServer, le_from->text(), QStringList() , le_subject->text(), bodyEdit->toPlainText());
	mail.setContentType( MailSender::text );
	if( ! _smtpLogin.isEmpty() ){
		mail.setLogin(_smtpLogin,_smtpPassword);
	}
	if ( _mailEncoding == "ISO-8859-1" ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMailDialog::on_sendButton_clicked() => encoding set to iso8859-1";
#endif
		mail.setISO( MailSender::iso88591 );
	}
	else{
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMailDialog::on_sendButton_clicked() => encoding set to utf8";
#endif
		mail.setISO( MailSender::utf8 );
	}
	progressBar->setValue(1);
	label->setText( tr("Adding attachements") );
	// Adding attachements
	QTreeWidgetItem *rootitem = attachements->invisibleRootItem();
	for(int k=0;k<rootitem->childCount(); k++){
		QTreeWidgetItem *it = rootitem->child(k);
		if( QFile::exists( it->text( 2 ) ) )
			attachs << it->text( 2 );
	}
	if( attachs.count() > 0 )
		mail.setAttachments(attachs);
	progressBar->setValue(2);
	QString errors;
	for ( int k = 0; k < toListWidget->count(); k++ ){
		QListWidgetItem *tmp_item = toListWidget->item(k);
		mail.setTo( QStringList( tmp_item->text() ) );
		bool ok = mail.send();
		label->setText( tr("Sending to: %1").arg( tmp_item->text() ) );
		if(ok){
			label->setText( tr("Mail sent to: %1").arg( tmp_item->text() ) );
		}
		else{
			label->setText( tr("Error: %1").arg( tmp_item->text() ) );
			errors.append( tr(" * Error while sending to: %1\n").arg( tmp_item->text() ) );
			errors.append( tr("       Error is: %1\n").arg( mail.lastError()).toLatin1 () );
			errors.append( tr("       Command was: %1\n").arg( mail.lastCmd()).toLatin1 () );
			errors.append( tr("       Response was: %1\n\n").arg( mail.lastResponse()).toLatin1 () );
		}
		progressBar->setValue( progressBar->value() +1 );
	}
	if( errors.size() > 0 ){
		label->setText( tr("All e-mails sent but errors occured.") );
		QMessageBox::warning(this, tr("Association Subscribers Manager - Mail"),tr("The following errors happened during the sending process:\n\n%1").arg(errors),QMessageBox::Ok);
	}
	else{
		label->setText( tr("All e-mails successfully sent.") );
		progressBar->hide();
		label->hide();
	}
	sendButton->setEnabled(true);
}

