
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include <QtGui>
#include <QDebug>

#include "assuma_mainwindow.h"
#include "adbhandler.h"
#include "oldxmlhandler.h"
#include "addonxmlhandler.h"
#include "assuma_addonengine.h"
#include "assuma_subsciberimporterdialog.h"
#include "postalcodehandler.h"
#include "assuma_maildialog.h"
#include "federationhandler.h"
#include "assuma_treewidgetitemeditordelegate.h"
#include "assuma_effect.h"
#include "assuma_helpbrowser.h"
#include "assuma_customwidgets.h"
#include "customfielditem.h"

#ifdef ASSUMA_WITH_UPDATER
#include "assuma_updatedialog.h"
#endif

#ifndef ASSUMA_DATADIR
#define ASSUMA_DATADIR "/usr/share/AssociationSubscribersManager"
#endif

#ifndef ASSUMA_DOCDIR
#define ASSUMA_DOCDIR "/usr/share/AssociationSubscribersManager/doc"
#endif

#ifndef ASSUMA_IMGDIR
#define ASSUMA_IMGDIR "/usr/share/AssociationSubscribersManager/images"
#endif

#ifndef ASSUMA_THEMESDIR
#define ASSUMA_THEMESDIR "/usr/share/AssociationSubscribersManager/themes"
#endif

#ifndef ASSUMA_I18N
#define ASSUMA_I18N "/usr/share/AssociationSubscribersManager/i18n"
#endif

#ifndef ASSUMA_CORE_VERSION
#define ASSUMA_CORE_VERSION "3.2.0"
#endif

AssumaMainWindow *AssumaMainWindow::m_singleInstance = NULL;
bool AssumaMainWindow::m_canInstanciate = true;
bool AssumaMainWindow::m_isInitializing = false;

AssumaMainWindow::AssumaMainWindow(QWidget *parent) : QMainWindow(parent)
{
// 	Private variables
	m_isInitializing = true;
	m_currentDatabase = "";
	m_previousLanguage = "";
	m_shouldCloseEditor = false;
	m_shouldSavePostalCodes=false;
	m_dontUpdateSubscribersList = false;
	m_translator=0;
	m_customWidgets.clear();
	// This protect the application from segfaulting during early calls of getDirectory();
	configWidget=0;
	
// 	QPixmap pixmap(getDirectory("themes")+"/default/m_splash3X.png");
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> themes directory is: " << getDirectory("themes");
#endif
	QPixmap pixmap(":/images/images/assuma-splashscreen.png");
// 	QPixmap pixmap( getImageFromTheme( "default", "assuma-splashscreen" ) );
	m_splash = new QSplashScreen( pixmap,  Qt::WindowStaysOnTopHint );
	m_splash->show();
	m_splash->repaint();
	
	setSplashMessage("Creating user interface");
	
	setupUi(this);
#ifdef ASSUMA_WITH_UPDATER
	actionUpdate->setVisible( true );
#else
	actionUpdate->setVisible( false );
#endif
	// Loading Qt's defaultf system translations
// 	QTranslator qtTranslator;
// 	qtTranslator.load("qt_" + QLocale::system().name(),QLibraryInfo::location(QLibraryInfo::TranslationsPath));
// 	QCoreApplication::installTranslator(&qtTranslator);
	
	setWindowTitle(tr("no document [*]"));
	treeWidget->setIconSize( QSize(32,32) );
	imagesListWidget->setUniformItemSizes(true);
	imagesListWidget->setLayoutMode( QListView::Batched );
	// 	Connect actions
	connect(actionAbout_Qt, SIGNAL(activated()), this, SLOT(aboutQt()) );
	connect(configWidget, SIGNAL( configMessage( const QString & , AssumaConfigManager::Severity ) ), this, SLOT( showPassiveMessage( const QString & , AssumaConfigManager::Severity) ) );
	connect(configWidget, SIGNAL( languageChanged(const QString &) ), this, SLOT( changeLanguage(const QString &) ) );
	connect( configWidget, SIGNAL( themeChanged(const QString &) ), this, SLOT( loadTheme( const QString & ) ) );
	connect( associationSettingsWidget, SIGNAL( customFieldsChanged() ), this, SLOT( reloadCustomFields() ) );
	
	setSplashMessage("Restoring previous session");
	
	readSettings();
	editArea->hide();
	pictureEdit->hide();
	searchWidget->hide();
	imagesListWidget->hide();
	addImagesButton->hide();
	configWidget->hide();
	associationSettingsWidget->hide();
	QDir cfDir( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/CustomFields/" );
	if( ! cfDir.exists() ){
		if( ! cfDir.mkpath( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/CustomFields/" ) ){
			showPassiveMessage( tr("Unable to create the <b>custom fields</b> directory at: %1<br/>Please check if you have the right to write in the data directory (%2)<br/>We will not be able to save custom fields !").arg(cfDir.absolutePath() ).arg( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() ), AssumaConfigManager::Warning );
		}
	}
	associationSettingsWidget->setCustomFieldsFileName( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/CustomFields/" + AssumaCM->getConfigToken( AssumaConfigManager::AssociationName ).toString().replace(" ","_") + ".customfields.xml" );
	loadCustomFields();
	
	// Set the item delegate
	m_treeWidgetDelegate = new AssumaTreeWidgetItemEditorDelegate(treeWidget);
	m_treeWidgetDelegate->setCoursesList( associationSettingsWidget->getCoursesList() );
	m_treeWidgetDelegate->setCoursesLessonsNumberData( associationSettingsWidget->getCoursesLessonsNumberData() );
	treeWidget->setItemDelegate( m_treeWidgetDelegate );
	
	// Restoring columns' visibility
	updateColumnVisibility();
	
	// set the tree widget after loading custom fields to be able to search in them and after restoring columns visibility to avoid searching in hidden columns
	searchWidget->setTreeWidget(treeWidget);
	
	setSplashMessage("Creating menus");
	
	m_contextMenu = new QMenu(treeWidget);
	m_contextMenu->addAction(action_Copy);
	m_contextMenu->addAction(actionPaste);
	m_contextMenu->addAction(actionAdd_a_subscriber);
	m_contextMenu->addAction(actionEdit_subscriber);
	m_contextMenu->addAction(actionRemove_subscriber);
	m_contextMenu->addSeparator();
	
	// Getting QTreeWidget header's name and map them with the column number
	QTreeWidgetItem *header = treeWidget->headerItem();
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		m_columnMap.insert( header->text(i) , i );
	}
	searchWidget->setColumnMap( m_columnMap );
	m_postal_codes_filename = AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/PostalCodes/" + AssumaCM->getConfigToken(AssumaConfigManager::Country).toString() + ".cities.xml" ;
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> loading postal codes files :" << m_postal_codes_filename;
#endif
	setSplashMessage( "Loading postal codes/cities: " + AssumaCM->getConfigToken(AssumaConfigManager::Country).toString() );
	if ( QFile::exists( m_postal_codes_filename+".qz" ) )
			m_postal_codes_filename = m_postal_codes_filename+".qz";
	loadPostalCodes( m_postal_codes_filename );
	
	m_addonEngine = new AssumaAddonEngine( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/Addons" );
	loadAddons();
	
	setSplashMessage( "Loading federation: " + AssumaCM->getConfigToken(AssumaConfigManager::Federation).toString() );
	loadFederation( AssumaCM->getConfigToken(AssumaConfigManager::Federation).toString() );
	
	setSplashMessage( "Loading subscribers database: " + m_currentDatabase );
	loadDatabase( m_currentDatabase );
	
	setSplashMessage( "Loading theme..." );
	loadTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() );

#ifdef ASSUMA_DEBUG
        qDebug() << "AssumaMainWindow -> About to destroy the splashscreen";
#endif
	m_splash->finish(this);
	
	// Test effects
	qApp->setEffectEnabled(Qt::UI_AnimateMenu, true);
	qApp->setEffectEnabled(Qt::UI_AnimateCombo, true);
	qApp->setEffectEnabled(Qt::UI_AnimateTooltip, true);
	qApp->setEffectEnabled(Qt::UI_AnimateToolBox, true);
	
	m_isInitializing=false;
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> constructor finish";
#endif
}

void AssumaMainWindow::reloadCustomFields(){
	if( ! m_isInitializing )
		showPassiveMessage( tr("You made some modifications to the custom fields (user defined fields), <br>you have to restart Association Subscribers Manager."), AssumaConfigManager::Warning );
}

void AssumaMainWindow::loadCustomFields() {
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::loadCustomFields() : start";
#endif
	QString fileName = associationSettingsWidget->getCustomFieldFileName();
	if( ! QFile::exists( fileName ) )
		return;
	// Clean all existing custom widgets
	foreach( AssumaCustomWidgetBase *widget, m_customWidgets ){
		if( widget ){
			customFieldsVLayout->removeWidget(widget);
			widget->setVisible(false);
			delete(widget);
		}
	}
	m_customWidgets.clear();
	QFile file(fileName);
	QXmlInputSource inputSource(&file);
	QXmlSimpleReader reader;
	CustomFieldsXmlHandler handler;
	reader.setContentHandler(&handler);
	reader.setErrorHandler(&handler);
	reader.parse(inputSource);
	m_customFields = handler.getData();
	foreach (CustomFieldItem *item, m_customFields){
		if( item ){
			QStringList extra_arguments = item->extraString().split(";");
			bool shouldAddColumn = false;
			if( item->type() == "One line text" ){
				AssumaCustomWidgetOneLine *w = new AssumaCustomWidgetOneLine(editArea);
				w->setCustomFieldLabelString( item->label() );
				w->setId( item->id() );
				w->show();
				customFieldsVLayout->addWidget(w);
				m_customWidgets << w;
                                connect( w, SIGNAL(valueChanged(QString)), this, SLOT(onCustomWidgetValueChanged(QString)) );
				shouldAddColumn = true;
			}
			else if( item->type() == "Multiple line text" ){
				AssumaCustomWidgetMultipleLine *w = new AssumaCustomWidgetMultipleLine(editArea);
				w->setCustomFieldLabelString( item->label() );
				w->setId( item->id() );
				w->show();
				customFieldsVLayout->addWidget(w);
				m_customWidgets << w;
				connect( w, SIGNAL(valueChanged(QString)), this, SLOT(onCustomWidgetValueChanged(QString)) );
				shouldAddColumn = true;
			}
			else if( item->type() == "Date selector" ){
				AssumaCustomWidgetDateEdit *w = new AssumaCustomWidgetDateEdit(editArea);
				w->setCustomFieldLabelString( item->label() );
				w->setId( item->id() );
				w->show();
				customFieldsVLayout->addWidget(w);
				m_customWidgets << w;
				connect( w, SIGNAL(valueChanged(QString)), this, SLOT(onCustomWidgetValueChanged(QString)) );
				shouldAddColumn = true;
			}
			else if( item->type() == "Ok/Not ok combobox" ){
				AssumaCustomWidgetComboBox *w = new AssumaCustomWidgetComboBox(editArea);
				w->setCustomFieldLabelString( item->label() );
				w->addItem( extra_arguments.at(0), QIcon( getImageFromTheme("button-ok") ) );
				w->addItem( extra_arguments.at(1), QIcon( getImageFromTheme("button-nok") ) );
				w->setId( item->id() );
				w->show();
				customFieldsVLayout->addWidget(w);
				m_customWidgets << w;
				connect( w, SIGNAL(valueChanged(QString)), this, SLOT(onCustomWidgetValueChanged(QString)) );
				shouldAddColumn = true;
			}
			else if( item->type() == "Multiple value selector" ){
				AssumaCustomWidgetComboBox *w = new AssumaCustomWidgetComboBox(editArea);
				w->setCustomFieldLabelString( item->label() );
				foreach( QString str, extra_arguments ){
					if( ! str.isEmpty() )
						w->addItem(str);
				}
				w->setId( item->id() );
				w->show();
				customFieldsVLayout->addWidget(w);
				m_customWidgets << w;
				connect( w, SIGNAL(valueChanged(QString)), this, SLOT(onCustomWidgetValueChanged(QString)) );
				shouldAddColumn = true;
			}
			else if( item->type() == tr("Fee field") ){
				AssumaCustomWidgetFeeEditor *w = new AssumaCustomWidgetFeeEditor(editArea);
				w->setCustomFieldLabelString( item->label() );
				w->setId( item->id() );
				w->show();
				customFieldsVLayout->addWidget(w);
				m_customWidgets << w;
				connect( w, SIGNAL(valueChanged(QString)), this, SLOT(onCustomWidgetValueChanged(QString)) );
				shouldAddColumn = true;
			}
			if( shouldAddColumn ){
				QTreeWidgetItem *header_item = treeWidget->headerItem();
				header_item->setText( header_item->columnCount(), item->label() );
				QAction *tmpAction = new QAction(this);
				tmpAction->setObjectName("actionShow_"+item->id()+"_column");
				tmpAction->setCheckable(true);
				tmpAction->setChecked(true);
				tmpAction->setText( tr("Show the") + " \"" + item->label() + "\" " + tr("column") );
				tmpAction->setData( item->label() );
				menuEdit->addAction(tmpAction);
				connect( tmpAction, SIGNAL( toggled(bool) ), this, SLOT( setColumnVisibility(bool) ) );
			}
		}
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::loadCustomFields() : done";
#endif
}

QString AssumaMainWindow::assumaVersion(){
	return QString(ASSUMA_CORE_VERSION);
}

void AssumaMainWindow::processPostInitTasks(){
	QString themeName = AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString();
	// Process tasks that are needed to be done after all initialization.
	if( ! m_isInitializing ){
		configWidget->loadTheme(themeName);
		searchWidget->loadTheme(themeName);
		associationSettingsWidget->loadTheme(themeName);
	}
}

AssumaMainWindow* AssumaMainWindow::instance(){
	if (m_canInstanciate && NULL == m_singleInstance && !m_isInitializing){
#ifdef ASSUMA_DEBUG
		qDebug() << "\n\nWARNING: New instance of AssumaMainWindow created !!\n";
#endif
		m_canInstanciate = false;
		m_singleInstance = new AssumaMainWindow ;
	}
	return m_singleInstance;
}

QString AssumaMainWindow::getDirectory( const QString & subDir ){
	if( AssumaCM ){
		QString path( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() );
		if( ! path.isEmpty() ){
			if( QFile::exists( path + "/" + subDir ) ){
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaMainWindow::getDirectory() => returning AssumaConfigManager::DataDir/"<< subDir;
#endif
				return path + "/" + subDir;
			}
		}
	}
	if( QFile::exists(QCoreApplication::applicationDirPath() + "/" + subDir) ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::getDirectory() => returning QCoreApplication::applicationDirPath()/"<< subDir;
#endif
		return QCoreApplication::applicationDirPath() + "/" + subDir;
	}
	else if( subDir == "data" ){
		if( AssumaCM && QFile::exists( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() ) )
			return AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString();
		else
			return QString( ASSUMA_DATADIR );
	}
	else if( subDir == "doc" )
		return QString( ASSUMA_DOCDIR );
	else if( subDir == "images" )
		return QString( ASSUMA_IMGDIR );
	else if( subDir == "themes" )
		return QString( ASSUMA_THEMESDIR );
	else if( subDir == "i18n" )
		return QString( ASSUMA_I18N );
	return QString();
}

QString AssumaMainWindow::loadTextFile( const QString & fileName ){
	if( QFile::exists( fileName ) ){
		QFile file( fileName );
		if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
			return QString();
		return file.readAll();
	}
	return QString();
}

QString AssumaMainWindow::getImageFromTheme(const QString & imageName){
	QString themeName("default") ;
	if( AssumaCM )
		themeName = AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString();
	return getImageFromTheme( themeName , imageName );
}

QString AssumaMainWindow::getImageFromTheme(const QString & themeName, const QString & imageName){
	QString imgFile;
	QString themeDirectory = getDirectory("themes");
	if( QFile::exists( themeDirectory+"/"+themeName+"/images/"+imageName+".svgz" ) )
		imgFile = themeDirectory+"/"+themeName+"/images/"+imageName+".svgz";
	else if( QFile::exists( themeDirectory+"/"+themeName+"/images/"+imageName+".svg" ) )
		imgFile = themeDirectory+"/"+themeName+"/images/"+imageName+".svg";
	else if( QFile::exists( themeDirectory+"/"+themeName+"/images/"+imageName+".png" ) )
		imgFile = themeDirectory+"/"+themeName+"/images/"+imageName+".png";
	else if( themeName != "default" ) // a little protection against infinite recursion...
		imgFile = getImageFromTheme("default",imageName);
	else
		imgFile = themeDirectory+"/default/images/"+imageName+".png";
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::getImageFromTheme(const QString & themeName, const QString & imageName) : returning image path: " << imgFile;
#endif
	return imgFile;
}

void AssumaMainWindow::loadTheme( const QString & themeName ){
	QIcon icon;
	QString themeDirectory = getDirectory("themes");
	icon.addPixmap(QPixmap( getImageFromTheme( themeName, "assuma-logo" ) ), QIcon::Normal, QIcon::Off);
	setWindowIcon(icon);
	if( QFile::exists( themeDirectory+"/"+themeName+"/styles/assuma-mainwindow.qtcss" ) ){
		setStyleSheet("");
		setStyleSheet( loadTextFile(themeDirectory+"/"+themeName+"/styles/assuma-mainwindow.qtcss") ) ;
	}
	if( QFile::exists( themeDirectory+"/"+themeName+"/styles/assuma-previewer-title-field.qtcss" ) ){
		nameLabel->setStyleSheet("");
		nameLabel->setStyleSheet( loadTextFile(themeDirectory+"/"+themeName+"/styles/assuma-previewer-title-field.qtcss") );
	}
	if( QFile::exists( themeDirectory+"/"+themeName+"/styles/assuma-previewer-image-field.qtcss" ) ){
		pictureLabel->setStyleSheet("");
		pictureLabel->setStyleSheet( loadTextFile(themeDirectory+"/"+themeName+"/styles/assuma-previewer-image-field.qtcss") );
	}
	if( QFile::exists( themeDirectory+"/"+themeName+"/styles/assuma-previewer-browser.qtcss" ) ){
		textBrowser->setStyleSheet("");
		textBrowser->setStyleSheet( loadTextFile(themeDirectory+"/"+themeName+"/styles/assuma-previewer-browser.qtcss") );
	}
	if( QFile::exists( themeDirectory+"/"+themeName+"/styles/assuma-editor-title-field.qtcss" ) ){
		label_16->setStyleSheet("");
		label_16->setStyleSheet( loadTextFile(themeDirectory+"/"+themeName+"/styles/assuma-editor-title-field.qtcss") );
	}
// 	setStyleSheet(QString::fromUtf8("QToolTip {\n"
// 	"	border: 2px solid darkkhaki;\n"
// 	"	padding: 2px;\n"
// 	"	border-radius: 3px;\n"
// 	"	font-size: 12px;\n"
// 	"}\n"
// 	"\n"
// 	"QTreeWidget::item QToolTip {\n"
// 	"	padding: 5px;\n"
// 	"	min-width: 160px;\n"
// 	"}"));
// 	treeWidget->setStyleSheet(QString::fromUtf8(""));
// 	nameLabel->setStyleSheet(QString::fromUtf8("border: 2px solid darkkhaki;\n"
// "border-radius: 3px;\n"
// "font: bold 2em \"Sans Serif\";\n"
// "background-color: rgb(164, 164, 164);\n"
// "text-align: middle;"));
// 	pictureLabel->setStyleSheet(QString::fromUtf8("border: 2px solid darkkhaki;\n"
// "border-radius: 3px;"));
// 	textBrowser->setStyleSheet(QString::fromUtf8("border: 2px solid darkkhaki;\n"
// "padding: 5px;\n"
// "border-radius: 3px;\n"
// "background-color: rgb(195,195,195);"));
// 	label_16->setStyleSheet(QString::fromUtf8("border: 1px solid black;\n"
// "background-color: rgb(255, 99, 99);"));
	QIcon icon1;
	icon1.addPixmap(QPixmap(getImageFromTheme(themeName,"file-new")), QIcon::Normal, QIcon::Off);
	action_New->setIcon(icon1);
	QIcon icon2;
	icon2.addPixmap(QPixmap(getImageFromTheme(themeName,"file-open")), QIcon::Normal, QIcon::Off);
	action_Open->setIcon(icon2);
	addImagesButton->setIcon(icon2);
	addImagesButton->setIconSize(QSize(22, 22));
	QIcon icon3;
	icon3.addPixmap(QPixmap(getImageFromTheme(themeName,"file-save")), QIcon::Normal, QIcon::Off);
	action_Save->setIcon(icon3);
	QIcon icon4;
	icon4.addPixmap(QPixmap(getImageFromTheme(themeName,"file-save-as")), QIcon::Normal, QIcon::Off);
	actionSave_as->setIcon(icon4);
	QIcon icon5;
	icon5.addPixmap(QPixmap(getImageFromTheme(themeName,"exit")), QIcon::Normal, QIcon::Off);
	actionQuit->setIcon(icon5);
	QIcon icon6;
	icon6.addPixmap(QPixmap(getImageFromTheme(themeName,"search-user")), QIcon::Normal, QIcon::Off);
	action_Find->setIcon(icon6);
	QIcon icon7;
	icon7.addPixmap(QPixmap(getImageFromTheme(themeName,"settings")), QIcon::Normal, QIcon::Off);
	actionSettings->setIcon(icon7);
	QIcon icon7bis;
	icon7bis.addPixmap(QPixmap(getImageFromTheme(themeName,"association-settings")), QIcon::Normal, QIcon::Off);
	actionAssociation_settings->setIcon(icon7bis);
	QIcon icon8;
	icon8.addPixmap(QPixmap(getImageFromTheme(themeName,"add-user")), QIcon::Normal, QIcon::Off);
	actionAdd_a_subscriber->setIcon(icon8);
	QIcon icon9;
	icon9.addPixmap(QPixmap(getImageFromTheme(themeName,"edit-user")), QIcon::Normal, QIcon::Off);
	actionEdit_subscriber->setIcon(icon9);
	editSubscriberButton->setIcon(icon9);
	editSubscriberButton->setIconSize(QSize(22, 22));
	QIcon icon10;
	icon10.addPixmap(QPixmap(getImageFromTheme(themeName,"delete-user")), QIcon::Normal, QIcon::Off);
	actionRemove_subscriber->setIcon(icon10);
	deleteSubscriberButton->setIcon(icon10);
	deleteSubscriberButton->setIconSize(QSize(22, 22));
	QIcon icon11;
	icon11.addPixmap(QPixmap(getImageFromTheme(themeName,"database-import")), QIcon::Normal, QIcon::Off);
	actionImport_subscribers_from_another_database->setIcon(icon11);
	QIcon icon12;
	icon12.addPixmap(QPixmap(getImageFromTheme(themeName,"document-calc")), QIcon::Normal, QIcon::Off);
	actionExport_to_CSV->setIcon(icon12);
	QIcon icon13;
	icon13.addPixmap(QPixmap(getImageFromTheme(themeName,"document-html")), QIcon::Normal, QIcon::Off);
	actionExport_to_HTML->setIcon(icon13);
	QIcon icon14;
	icon14.addPixmap(QPixmap(getImageFromTheme(themeName,"mail-generic")), QIcon::Normal, QIcon::Off);
	actionSend_a_newsletter->setIcon(icon14);
	actionSend_fee_reminder->setIcon(icon14);
	QIcon icon15;
	icon15.addPixmap(QPixmap(getImageFromTheme(themeName,"update")), QIcon::Normal, QIcon::Off);
	actionUpdate->setIcon(icon15);
	QIcon icon16;
	icon16.addPixmap(QPixmap(getImageFromTheme(themeName,"assuma-logo")), QIcon::Normal, QIcon::Off);
	actionAbout_Association_Subscribers_Manager->setIcon(icon);
	QIcon icon17;
	icon17.addPixmap(QPixmap(getImageFromTheme(themeName,"edit-copy")), QIcon::Normal, QIcon::Off);
	action_Copy->setIcon(icon17);
	QIcon icon18;
	icon18.addPixmap(QPixmap(getImageFromTheme(themeName,"document-excel")), QIcon::Normal, QIcon::Off);
	actionExport_to_Excel->setIcon(icon18);
	QIcon icon19;
	icon19.addPixmap(QPixmap(getImageFromTheme(themeName,"edit-paste")), QIcon::Normal, QIcon::Off);
	actionPaste->setIcon(icon19);
	actionSend_database_by_email->setIcon(icon14);
	QIcon icon20;
	icon20.addPixmap(QPixmap(getImageFromTheme(themeName,"file-print")), QIcon::Normal, QIcon::Off);
	action_Print->setIcon(icon20);
	QIcon icon21;
        icon21.addPixmap(QPixmap(getImageFromTheme(themeName,"assuma-help-icon")), QIcon::Normal, QIcon::Off);
        actionAssociation_Subscribers_Manager_manual->setIcon(icon21);
	QIcon icon22;
	icon22.addFile( getImageFromTheme(themeName,"mail-send") , QSize(), QIcon::Normal, QIcon::Off);
	sendSummaryButton->setIcon(icon22);
	sendSummaryButton->setIconSize(QSize(22, 22));
	QIcon icon23;
	icon23.addPixmap(QPixmap(getImageFromTheme(themeName,"youtube-icon")), QIcon::Normal, QIcon::Off);
	actionGo_to_YouTube_video_tutorials_channel->setIcon(icon23);
	// Updating default user picture in the image QListWidget
	QPixmap user_default_pixmap( getImageFromTheme(themeName,"user-default") );
	foreach( QListWidgetItem * tmp_item, imagesListWidget->findItems("user_default.png",Qt::MatchFixedString) ){
		imagesListWidget->takeItem( imagesListWidget->row( tmp_item ) );
		QListWidgetItem *__qlistwidgetitem = new QListWidgetItem(imagesListWidget);
		__qlistwidgetitem->setText(QString::fromUtf8("user_default.png"));
		__qlistwidgetitem->setIcon(user_default_pixmap);
	}
	imagesListWidget->setIconSize(QSize(48, 48));
	// Updating combo boxes
	QIcon icon_ok;
	icon_ok.addFile(getImageFromTheme(themeName,"button-ok"), QSize(), QIcon::Normal, QIcon::Off);
	QIcon icon_nok;
	icon_nok.addFile(getImageFromTheme(themeName,"button-nok"), QSize(), QIcon::Normal, QIcon::Off);
	duesStatusEdit->removeItem(0);
	duesStatusEdit->removeItem(0);
	medicalStatuEdit->removeItem(0);
	medicalStatuEdit->removeItem(0);
	duesStatusEdit->addItem(icon_ok, QApplication::translate("AssumaMainWindow", "Ok", 0, QApplication::UnicodeUTF8));
	duesStatusEdit->addItem(icon_nok, QApplication::translate("AssumaMainWindow", "Not ok", 0, QApplication::UnicodeUTF8));
	medicalStatuEdit->addItem(icon_ok, QApplication::translate("AssumaMainWindow", "Ok", 0, QApplication::UnicodeUTF8));
	medicalStatuEdit->addItem(icon_nok, QApplication::translate("AssumaMainWindow", "Not ok", 0, QApplication::UnicodeUTF8));;
	// Setting action bat tooltips
	setActionBarToolTips();
	// If initialization is finished, loading theme for other widgets
	if( ! m_isInitializing ){
		configWidget->loadTheme(themeName);
		searchWidget->loadTheme(themeName);
		associationSettingsWidget->loadTheme(themeName);
	}
}

void AssumaMainWindow::setActionBarToolTips(){
	// Get the current theme name from configuration
	QString themeName = AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString();
	// Set tooltip for actions in the action bar.
	action_New->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"file-new")+"\" /></td><td valign=\"middle\">" + tr("Create a new empty database.") + "</td></tr></table>");
	action_Open->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"file-open")+"\" /></td><td valign=\"middle\">" + tr("Open an existing database and load it.") + "</td></tr></table>");
	action_Save->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"file-save")+"\" /></td><td valign=\"middle\">" + tr("Save the current database.") + "</td></tr></table>");
	actionSave_as->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"file-save-as")+"\" /></td><td valign=\"middle\">" + tr("Save the current database and asks you for the place to save the database to.") + "</td></tr></table>");
	actionQuit->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"exit")+"\" /></td><td valign=\"middle\">" + tr("Quit Association Subscribers Manager.") + "</td></tr></table>");
	action_Find->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"search-user")+"\" /></td><td valign=\"middle\">" + tr("Display a window which allow you to search your database for a subscriber.") + "</td></tr></table>");
	actionSettings->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"settings")+"\" /></td><td valign=\"middle\">" + tr("Display the application settings window. This window allows you to modify the parameters of the <strong><u>application</u></strong> (language, country, internet settings, etc.).") + "</td></tr></table>");
	actionAdd_a_subscriber->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"add-user")+"\" /></td><td valign=\"middle\">" + tr("Add an empty subscriber to the current database.") + "</td></tr></table>");
	actionEdit_subscriber->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"edit-user")+"\" /></td><td valign=\"middle\">" + tr("Turn the right part of Association Subscribers Manager to the edit mode, allowing you to modify data about the currently selected subscriber.") + "</td></tr></table>");
	actionRemove_subscriber->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"delete-user")+"\" /></td><td valign=\"middle\">" + tr("Remove selected subscriber(s) from the current database. once the database is saved, this operation cannot be undone.") + "</td></tr></table>");
	actionImport_subscribers_from_another_database->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"database-import-big")+"\" /></td><td valign=\"middle\">" + tr("Display a new window which allow you to import subscribers from another database into the current one.") + "</td></tr></table>");
	actionUpdate->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"update")+"\" /></td><td valign=\"middle\">" + tr("Display the auto-update window.") + "</td></tr></table>");
	actionAssociation_settings->setToolTip("<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme(themeName,"association-settings")+"\" /></td><td valign=\"middle\">" + tr("Display the association settings window. This window allows you to modify the parameters of your <strong><u>association</u></strong> (dues, courses, etc.).") + "</td></tr></table>");
}

void AssumaMainWindow::on_treeWidget_itemSelectionChanged(){
	if( treeWidget->topLevelItemCount() > 0 && treeWidget->selectedItems().size() > 0 ){
		editSubscriberButton->setEnabled(true);
		deleteSubscriberButton->setEnabled(true);
		sendSummaryButton->setEnabled(true);
		actionEdit_subscriber->setEnabled(true);
		actionRemove_subscriber->setEnabled(true);
// 		action_Find->setEnabled(true);
		action_Copy->setEnabled(true);
	}
	else{
		if( actionEdit_subscriber->isChecked() )
			actionEdit_subscriber->setChecked(false);
		editSubscriberButton->setEnabled(false);
		deleteSubscriberButton->setEnabled(false);
		sendSummaryButton->setEnabled(false);
		actionEdit_subscriber->setEnabled(false);
		actionRemove_subscriber->setEnabled(false);
// 		action_Find->setEnabled(false);
		action_Copy->setEnabled(false);
	}
}

void AssumaMainWindow::setSplashMessage( const QString & mesg ){
	if(m_splash){
		m_splash->showMessage(mesg, Qt::AlignLeft|Qt::AlignBottom, Qt::white);
		m_splash->repaint();
	}
}

void AssumaMainWindow::checkUpdatesOnStart(){
#ifdef ASSUMA_WITH_UPDATER
	if( AssumaCM->getConfigToken(AssumaConfigManager::UpdatesOnStart).toInt() == 1 ){
		AssumaUpdateDialog *dial = new AssumaUpdateDialog(this);
		dial->loadTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() );
		connect( dial, SIGNAL(updateFinished(bool)), this, SLOT( finishUpdate(bool) ) );
		this->setVisible(false);
                dial->show();
                dial->checkForUpdates();
	}
#endif
}

void AssumaMainWindow::loadPostalCodes(const QString & pc_filename){
	if( QFile::exists(pc_filename) ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::loadPostalCodes(const QString & pc_filename) => will load " << pc_filename;
#endif
		QFile file(pc_filename);
		QIODevice *in;
		QBuffer buf ;
		if( pc_filename.endsWith(".qz") ){
			if( file.open(QIODevice::ReadOnly) ){
				buf.setData( qUncompress(file.readAll()) );
			}
			in = &buf;
		}
		else
			in = &file;
		QXmlInputSource inputSource(in);
		QXmlSimpleReader reader;
		PostalCodeHandler *handler = new PostalCodeHandler();
		reader.setContentHandler(handler);
		reader.setErrorHandler(handler);
		reader.parse(inputSource);
		m_pc_map = handler->getPostalCodes();
// 		postalCodeEdit->clear();
// 		postalCodeEdit->addItems( m_pc_map.values() );
// 		cityEdit->clear();
// 		cityEdit->addItems( m_pc_map.keys() );
		if( m_treeWidgetDelegate )
			m_treeWidgetDelegate->setPostCodesMap( m_pc_map );
		QCompleter *completer = new QCompleter(m_pc_map.keys(), this);
		completer->setCaseSensitivity(Qt::CaseInsensitive);
		cityEdit->setCompleter(completer);
		QCompleter *completer_pc = new QCompleter(m_pc_map.values(), this);
		completer_pc->setCaseSensitivity(Qt::CaseInsensitive);
		postalCodeEdit->setCompleter(completer_pc);
	}
	else{
		showPassiveMessage( tr("Postal codes/cities file <strong>%1</strong> could not be load.<br>File not found.").arg(pc_filename), AssumaConfigManager::Critical );
	}
}

void AssumaMainWindow::loadLanguageList( const QString & lp_list_file ){
	QFile file( lp_list_file );
	QString serializedLanguageList;
	if (!file.open(QIODevice::ReadOnly | QIODevice::Text)){
		showPassiveMessage( tr("The languages list could not be load.<br>It is possibly caused by a bad configuration of the data directory (see: Settings -> General -> Data directory)"), AssumaConfigManager::Warning );
		return ;
	}
	QString imgDir( getDirectory("images") );
	QString i18nDir( getDirectory("i18n") );
	QTextStream in(&file);
	in.setCodec("UTF-8");
	while (!in.atEnd()) {
		QString line = in.readLine();
		if( ! line.startsWith("#") ){
			// First we process the line to replace variables with values :
			line.replace(QString("%IMGDIR%"), imgDir);
			line.replace(QString("%TRDIR%"), i18nDir);
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaMainWindow::loadLanguageList() : line after variables replacement and before processing :\n\t" << line;
#endif
			// A line in the file is based on this template :
			// Language name;Language icon;Translation file
			QStringList tmp = line.split(";");
			m_translation_equiv.insert( tmp.at(0) ,  tmp.at(2) );
			serializedLanguageList.append( getImageFromTheme( "flags/"+tmp.at(1) ) );
			serializedLanguageList.append( "," );
			serializedLanguageList.append( tmp.at(0) );
			serializedLanguageList.append( ";" );
		}
	}
	AssumaCM->setConfigToken( AssumaConfigManager::LanguageList, serializedLanguageList );
}

void AssumaMainWindow::loadAddons(){
#ifdef ASSUMA_DEBUG
	QString pluginsDir = AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/Addons" ;
	qDebug() << "AssumaMainWindow::loadAddons() -> pluginDir =" << pluginsDir ;
#endif
	foreach (QString addonName, AssumaCM->getConfigToken(AssumaConfigManager::ActivatedAddonList).toString().split(";") )
	{
		if(addonName != ""){
// 			qDebug() << "AssumaMainWindow::loadAddons() -> instantiating addon:" << addonName;
			m_addonEngine->instanciateAddon(addonName);
		}
	}
	configWidget->setLoadedAddonsList( m_addonEngine->instanciatedAddons() );
}

void AssumaMainWindow::writePostalCodes( const QString & fileName){
//	qDebug() << citiesToXml();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::writePostalCodes(" << fileName << ") : saving postal codes list.";
#endif
	QFileInfo fi(fileName);
	qDebug() << "AssumaMainWindow::writePostalCodes => suffix : " << fi.suffix();
	if( fi.isWritable() ){
		if(fi.suffix() == "qz"){
			if( ! fileName.isEmpty() && QFile::exists(fileName) ){
				QFile file(fileName);
				QByteArray cba = qCompress( citiesToXml().toUtf8(), 9 );
				if( file.open( QIODevice::WriteOnly ) ){
					/*qint64 ret = */file.write(cba);
					file.flush();
					file.close();
				}
				else
					return;
			}
		}
		else{
			QFile file(fileName);
			if (!file.open(QIODevice::WriteOnly | QIODevice::Text))
				return;
			QTextStream out(&file);
			out.setCodec("UTF-8");
			out << citiesToXml() << "\n";
			file.flush();
			file.close();
			showPassiveMessage( tr("cities database saved as <strong>%1</strong>").arg(fileName) );
		}
	}
	else
		showPassiveMessage( tr("cities database could not be save as <strong>%1</strong><br>You don't have the permission to write this file.").arg(fileName), AssumaConfigManager::Critical );
}

QString AssumaMainWindow::citiesToXml() {
	QString xml;
	xml.append("<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?>\n<Cities version=\"2.0\">\n");
	foreach(QString city, m_pc_map.keys()){
		xml.append("\t<City name=\"") ; xml.append(city) ; xml.append("\" pc=\""); xml.append( m_pc_map.value(city) ) ; xml.append("\" />\n");
	}
	xml.append( "</Cities>" );
	return xml;
}

double AssumaMainWindow::licenseFeeByLabel(const QString & label){
	foreach ( FederationLicenseFee * fee, m_licences_fees ){
		if( fee->getName() == label )
			return fee->getFee();
	}
	return 0.0;
}

double AssumaMainWindow::licenseFeeById(const QString & id){
	foreach ( FederationLicenseFee * fee, m_licences_fees ){
		if( fee->getId() == id )
			return fee->getFee();
	}
	return 0.0;
}

QString AssumaMainWindow::licenseIdByName(const QString & name){
	foreach ( FederationLicenseFee * fee, m_licences_fees ){
		if( fee->getName() == name )
			return fee->getId();
	}
	return QString("");
}

QString AssumaMainWindow::licenseNameById(const QString & id){
	foreach ( FederationLicenseFee * fee, m_licences_fees ){
		if( fee->getId() == id )
			return fee->getName();
	}
	return QString("");
}

// int AssumaMainWindow::calculateDue( const QListWidgetItem *item ){
double AssumaMainWindow::calculateDue( const QTreeWidgetItem *item ){
#ifdef ASSUMA_DEBUG
	qDebug() << " **** ENTERING calculateDue()";
#endif
	// Contribution fee = ( (courses fee) x (number of trimesters in a year) ) + (licence cost) + (static cost)
	//! \todo extensively test the calculateDue() method (check that calculation are done the right way).
	int course_fee = associationSettingsWidget->getCourseFee( activityComboBox->currentText(), lessonsNumberEdit->value() ).toInt();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> course_fee is " << course_fee;
#endif
	
	int trimesters_count = AssumaCM->getConfigToken(AssumaConfigManager::TrimesterCount).toInt();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> trimesters_count is " << trimesters_count;
#endif
	
	int static_cost = AssumaCM->getConfigToken(AssumaConfigManager::StaticCost).toInt();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> dues = " << course_fee << " * " << trimesters_count << " + " << static_cost;
#endif
	double _license_fee = licenseFeeByLabel(item->text( elementColumn(tr("LicenseLabel")) ));
	//! \todo : see how to integrate the use of item's data instead of the current mechanism (it is also possible to embed the licenseLabelComboBox's index instead of duplicate informations)
#ifdef ASSUMA_DEBUG
	qDebug() << "   /!\\ _license_fee is " << _license_fee << " and data contained fee is " << item->data( elementColumn( tr("LicenseLabel") ), AssumaMainWindow::FeeRole );
#endif
// 	double due = course_fee * trimesters_count + static_cost + m_licences_fees.value( item->text( elementColumn(tr("LicenseLabel")) ) , 0 );
	double due = course_fee * trimesters_count + static_cost + _license_fee;
	
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> LicenseLabel column is " << elementColumn(tr("LicenseLabel")) << " and fees for" << item->text( elementColumn(tr("LicenseLabel")) ) << "is " << _license_fee ;
#endif
	
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> dues is " << course_fee << " * " << trimesters_count << " + " << static_cost << " + " << _license_fee << " = " << due;
#endif
	return due;
}

void AssumaMainWindow::changeLanguage(const QString & lang){
	if( lang != m_previousLanguage){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow -> we just received a languageChanged(" << lang << ") signal.";
#endif
		if( m_translator ){
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaMainWindow -> Removing previous translator.";
#endif
			QCoreApplication::removeTranslator(m_translator);
		}
		m_translator = new QTranslator() ;
		QString tr_file = m_translation_equiv.value( lang );
		if( tr_file.isEmpty() )
			tr_file = "i18n/assuma_en-en.qm";
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow -> Loading translation file : " << tr_file ;
#endif
		if( ! QFile::exists(tr_file) && ! m_isInitializing ){
			showPassiveMessage( QString("Unable to load the locale file (%1) for language <strong>%2</strong>").arg(tr_file).arg(lang), AssumaConfigManager::Warning );
			return;
		}
		m_translator->load( tr_file );
		QCoreApplication::installTranslator(m_translator);
		m_previousLanguage = lang;
		retranslateUi(this);
// 		QString currencies = AssumaCM->getConfigToken( AssumaConfigManager::CurrenciesList ).toString();
// 		QString currency = AssumaCM->getConfigToken( AssumaConfigManager::Currency ).toString();
		configWidget->retranslateUi(configWidget);
		searchWidget->retranslateUi(searchWidget);
		associationSettingsWidget->updateTranslations();
// 		associationSettingsWidget->retranslateUi(associationSettingsWidget);
// 		AssumaCM->setConfigToken( AssumaConfigManager::CurrenciesList, currencies );
// 		AssumaCM->setConfigToken( AssumaConfigManager::Currency, currency );
		setActionBarToolTips();
	}
}

QString AssumaMainWindow::processTemplate( const QString & original){
	// We don't want to modify the original string (it's a const anyway), so we works on a copy
	QString template_string = original;
	// Process non-item related variables
	template_string.replace( QString("%ASSO_NAME%"), AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() );
	template_string.replace( QString("%CURRENCY%"), AssumaCM->getConfigToken(AssumaConfigManager::Currency).toString() );
	QTreeWidgetItem *item;
	item = treeWidget->currentItem();
	// If there is a current item, we now process all item's related variables
	if( item ){
		template_string.replace( QString("%SUBSCRIBER_FIRSTNAME%"), item->text( elementColumn( tr("FirstName") ) ) );
		template_string.replace( QString("%SUBSCRIBER_LASTNAME%"), item->text( elementColumn( tr("LastName") ) ) );
		template_string.replace( QString("%SUBSCRIBER_MAIDENNAME%"), item->text( elementColumn( tr("MaidenName") ) ) );
		template_string.replace( QString("%SUBSCRIBER_BIRTHDATE%"), item->text( elementColumn( tr("BirthDate") ) ) );
		template_string.replace( QString("%SUBSCRIBER_ADDRESS%"), item->text( elementColumn( tr("Address") ) ) );
		template_string.replace( QString("%SUBSCRIBER_CITY%"), item->text( elementColumn( tr("City") ) ) );
		template_string.replace( QString("%SUBSCRIBER_POSTALCODE%"), item->text( elementColumn( tr("PostalCode") ) ) );
		template_string.replace( QString("%SUBSCRIBER_PHONENUMBER%"), item->text( elementColumn( tr("PhoneNumber") ) ) );
		template_string.replace( QString("%SUBSCRIBER_EMAIL%"), item->text( elementColumn( tr("Email") ) ) );
		template_string.replace( QString("%SUBSCRIBER_ACTIVITY%"), item->text( elementColumn( tr("Activity") ) ) );
		template_string.replace( QString("%SUBSCRIBER_LESSONSNUMBER%"), item->text( elementColumn( tr("LessonsNumber") ) ) );
		template_string.replace( QString("%SUBSCRIBER_DUES%"), item->text( elementColumn( tr("Dues") ) ) );
		template_string.replace( QString("%SUBSCRIBER_DUESSTATUS%"), item->text( elementColumn( tr("DuesStatus") ) ) );
		template_string.replace( QString("%SUBSCRIBER_SUBSCRIPTIONDATE%"), item->text( elementColumn( tr("SubscriptionDate") ) ) );
		template_string.replace( QString("%SUBSCRIBER_MEDICALCERTIFICATESTATUS%"), item->text( elementColumn( tr("MedicalCertificateStatus") ) ) );
		template_string.replace( QString("%SUBSCRIBER_LICENSENUMBER%"), item->text( elementColumn( tr("LicenseNumber") ) ) );
		template_string.replace( QString("%SUBSCRIBER_COMMENT%"), item->text( elementColumn( tr("Comment") ) ) );
		template_string.replace( QString("%SUBSCRIBER_PICTURE%"), item->text( elementColumn( tr("Picture") ) ) );
		
	}
	return template_string;
}

void AssumaMainWindow::showPassiveMessage( const QString & str ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str ) called";
#endif
	AssumaPassivePopup *pop = new AssumaPassivePopup(this);
	pop->setText(str);
	if( m_isInitializing ){
		pop->setCloseButtonPixmap( QPixmap(getImageFromTheme("button-close")) );
		pop->setPixmap( QPixmap(getImageFromTheme("messagebox-info")) );
	}
	AssumaEffect *effect = new AssumaEffect(0,pop, AssumaEffect::SLIDE_FROM_BOTTOM);
	connect( pop, SIGNAL( closeButtonClicked() ), effect, SLOT( reverseEffect() ) );
	connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( hide() ) );
	if( isVisible() ){
		connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( close() ) );
		effect->setTimeBeforeReversing( pop->timeToShow() );
	}
	else
		effect->setTimeBeforeReversing( pop->timeToShow() * 5 );
	effect->doEffect();
}

void AssumaMainWindow::showPassiveMessage( const QString & str , AssumaConfigManager::Severity sev ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaConfigManager::Severity sev ) called";
#endif
	AssumaPassivePopup *pop = new AssumaPassivePopup(this);
	pop->setText(str);
	if( m_isInitializing ){
		pop->setCloseButtonPixmap( QPixmap(getImageFromTheme("button-close")) );
		if( sev == AssumaConfigManager::Info)
			pop->setPixmap( QPixmap(getImageFromTheme("messagebox-info")) );
		if( sev == AssumaConfigManager::Warning)
			pop->setPixmap( QPixmap(getImageFromTheme("messagebox-warning")) );
		if( sev == AssumaConfigManager::Critical)
			pop->setPixmap( QPixmap(getImageFromTheme("messagebox-critical")) );
	}
	else{
		if( sev == AssumaConfigManager::Info)
			pop->setPixmap( AssumaPassivePopup::Info );
		if( sev == AssumaConfigManager::Warning)
			pop->setPixmap( AssumaPassivePopup::Warning );
		if( sev == AssumaConfigManager::Critical)
			pop->setPixmap( AssumaPassivePopup::Critical );
	}
	AssumaEffect *effect = new AssumaEffect(0,pop, AssumaEffect::SLIDE_FROM_BOTTOM);
	connect( pop, SIGNAL( closeButtonClicked() ), effect, SLOT( reverseEffect() ) );
	connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( hide() ) );
	if( isVisible() ){
		connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( close() ) );
		effect->setTimeBeforeReversing( pop->timeToShow() );
	}
	else
		effect->setTimeBeforeReversing( pop->timeToShow() * 5 );
	effect->doEffect();
}

void AssumaMainWindow::showPassiveMessage( const QString & str , const QPixmap & pix ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , const QPixmap & pix ) called";
#endif
	AssumaPassivePopup *pop = new AssumaPassivePopup(this);
	pop->setText(str);
	if( m_isInitializing ){
		pop->setCloseButtonPixmap( QPixmap(getImageFromTheme("button-close")) );
	}
	pop->setPixmap( pix.scaled(32,32) );
	AssumaEffect *effect = new AssumaEffect(0,pop, AssumaEffect::SLIDE_FROM_BOTTOM);
	connect( pop, SIGNAL( closeButtonClicked() ), effect, SLOT( reverseEffect() ) );
	connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( hide() ) );
	if( isVisible() ){
		connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( close() ) );
		effect->setTimeBeforeReversing( pop->timeToShow() );
	}
	else
		effect->setTimeBeforeReversing( pop->timeToShow() * 5 );
	effect->doEffect();
}

void AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) called";
#endif
	AssumaPassivePopup *pop = new AssumaPassivePopup(this);
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) => 0.1";
	pop->setText(str);
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) => 0.2";
	pop->setPixmap( ico );
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) => 1";
	AssumaEffect *effect = new AssumaEffect(0,pop, AssumaEffect::SLIDE_FROM_BOTTOM);
	connect( pop, SIGNAL( closeButtonClicked() ), effect, SLOT( reverseEffect() ) );
	connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( hide() ) );
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) => 2";
	if( isVisible() ){
		connect( effect, SIGNAL( AssumaEffectFinished() ), pop, SLOT( close() ) );
		effect->setTimeBeforeReversing( pop->timeToShow() );
		qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) => 3";
	}
	else
		effect->setTimeBeforeReversing( pop->timeToShow() * 5 );
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) => 4";
	effect->doEffect();
	qDebug() << "AssumaMainWindow::showPassiveMessage( const QString & str , AssumaPassivePopup::Icon ico ) done";
}

void AssumaMainWindow::on_treeWidget_customContextMenuRequested ( const QPoint & pos ){
	int currentColumn = treeWidget->columnAt( pos.x() );
	bool showAllColumnsActions = false;
	QTreeWidgetItem *it_b = treeWidget->itemAt(pos);
	if( ! it_b ){
		showAllColumnsActions = true;
	}
	QTreeWidgetItem *header = treeWidget->headerItem();
#ifdef ASSUMA_DEBUG
	qDebug() << "We are in the column: " << header->text(currentColumn);
#endif
	while ( m_contextMenu->actions().last()->text() != "" )
		m_contextMenu->removeAction(m_contextMenu->actions().last());
	if( header->text(currentColumn) == tr("FirstName") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_FirstName_column ) ;
	}
	if( header->text(currentColumn) == tr("MaidenName") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_MaidenName_column ) ;
	}
	if( header->text(currentColumn) == tr("LastName") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_LastName_column ) ;
	}
	if( header->text(currentColumn) == tr("BirthDate") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_BirthDate_column ) ;
	}
	if( header->text(currentColumn) == tr("Address") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_Address_column ) ;
	}
	if( header->text(currentColumn) == tr("City") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_City_column ) ;
	}
	if( header->text(currentColumn) == tr("PostalCode") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_PostalCode_column ) ;
	}
	if( header->text(currentColumn) == tr("PhoneNumber") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_PhoneNumber_column ) ;
	}
	if( header->text(currentColumn) == tr("Email") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_Email_column ) ;
	}
	if( header->text(currentColumn) == tr("Activity") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_Activity_column ) ;
	}
	if( header->text(currentColumn) == tr("LessonsNumber") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_LessonsNumber_column ) ;
	}
	if( header->text(currentColumn) == tr("LicenseLabel") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_LicenseLabel_column ) ;
	}
	if( header->text(currentColumn) == tr("Dues") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_Dues_column ) ;
	}
	if( header->text(currentColumn) == tr("DuesStatus") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_DuesStatus_column ) ;
	}
	if( header->text(currentColumn) == tr("SubscriptionDate") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_SubscriptionDate_column ) ;
	}
	if( header->text(currentColumn) == tr("MedicalCertificateStatus") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_MedicalCertificateStatus_column ) ;
	}
	if( header->text(currentColumn) == tr("LicenseNumber") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_LicenseNumber_column ) ;
	}
	if( header->text(currentColumn) == tr("Comment") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_Comment_column ) ;
	}
	if( header->text(currentColumn) == tr("Picture") || showAllColumnsActions ){
		m_contextMenu->addAction( actionShow_Picture_column ) ;
	}
	foreach( CustomFieldItem *cfItem, m_customFields ){
		if( header->text(currentColumn) == cfItem->label() || showAllColumnsActions ){
			QAction *tmpAction = findChild<QAction *>( "actionShow_"+cfItem->id()+"_column" );
			m_contextMenu->addAction( tmpAction ) ;
		}
	}
	m_contextMenu->popup( treeWidget->mapToGlobal(pos) );
}

void AssumaMainWindow::on_actionQuit_activated(){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> about to quit";
#endif
	close();
}

void AssumaMainWindow::on_action_New_activated(){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> Clearing m_currentDatabase";
#endif
	m_currentDatabase = "";
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> Clearing treeWidget";
#endif
	treeWidget->clear();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> New document is ready !";
#endif
	setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
	if( ! m_isInitializing )
		setWindowModified(true);
}

void AssumaMainWindow::aboutQt()
{
	QMessageBox::aboutQt(this);
}

void AssumaMainWindow::on_actionImport_subscribers_from_another_database_activated(){
	AssumaSubscriberImporterDialog *importer = new AssumaSubscriberImporterDialog(this);
	importer->loadTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() );
	connect(importer,SIGNAL(importSubscribers( QList<QTreeWidgetItem*>* )),this,SLOT(subscribersImported( QList<QTreeWidgetItem*>* )));
	importer->show();
}

void AssumaMainWindow::on_action_Open_activated(){
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),".",tr("ADB database (*.adb);;Old XML database (*.xml)"));
	if( fileName.size() > 0 ){
		update();
		// If the QTreeWidget is not empty clear it
		if( treeWidget->topLevelItemCount() > 0 )
			treeWidget->clear();
		// Loading XML database
		loadDatabase(fileName);
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "Database file (open) : " << m_currentDatabase ;
#endif
}

AssumaMailDialog* AssumaMainWindow::createMailDialog(){
	AssumaMailDialog *dial = new AssumaMailDialog(this);
	dial->loadTheme( AssumaCM->getConfigToken(AssumaConfigManager::Theme).toString() );
	QStringList list;
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	for(int k=0;k<rootitem->childCount(); k++){
		QTreeWidgetItem *it = rootitem->child(k);
		if( it->text( elementColumn( tr("Email") ) ).size() > 5 )
			list << it->text( elementColumn( tr("Email") ) );
	}
	list << AssumaCM->getConfigToken( AssumaConfigManager::ContactsList ).toString().split(";");
	dial->setEmailList( list );
	dial->setSender( AssumaCM->getConfigToken( AssumaConfigManager::Email ).toString() );
	dial->setSmtpServer( AssumaCM->getConfigToken( AssumaConfigManager::SmtpServer ).toString() );
	if( ! AssumaCM->getConfigToken( AssumaConfigManager::SmtpLogin ).toString().isEmpty() ){
		dial->setAuth( AssumaCM->getConfigToken( AssumaConfigManager::SmtpLogin ).toString(), AssumaCM->getConfigToken( AssumaConfigManager::SmtpPassword ).toString() );
	}
	dial->setMailEncoding( AssumaCM->getConfigToken( AssumaConfigManager::Encoding ).toString() );
	return dial ;
}

void AssumaMainWindow::on_actionSend_a_newsletter_activated(){
	AssumaMailDialog *dial = createMailDialog();
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	for(int k=0;k<rootitem->childCount(); k++){
		QTreeWidgetItem *it = rootitem->child(k);
		if( it->text( elementColumn( tr("Email") ) ).size() > 5 )
			dial->addEmail( it->text( elementColumn( tr("Email") ) ) );
	}
	QString body(tr("Hello,\n\n\n\nBest regards,\n%ASSO_NAME%"));
	dial->setSubject( processTemplate( tr("[%ASSO_NAME%] Newsletter") ) );
	dial->setBody( processTemplate(body) );
	dial->show();
}

void AssumaMainWindow::on_actionSend_fee_reminder_activated(){
	AssumaMailDialog *dial = createMailDialog();
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	for(int k=0;k<rootitem->childCount(); k++){
		QTreeWidgetItem *it = rootitem->child(k);
		if( it->text( elementColumn( tr("DuesStatus") ) ) == tr("Not ok") && it->text( elementColumn( tr("Email") ) ).size() > 5 ){
			dial->addEmail( it->text( elementColumn(tr("Email")) ) );
		}
	}
	dial->setSubject( processTemplate( tr("[%ASSO_NAME%] Fee reminder") ) );
	dial->setBody( processTemplate( tr("Hello,\n\nwe would like to remind you that you still need to pay your fee to our association.\n\nBest regards,\n%ASSO_NAME%") ) );
	dial->show();
}

void AssumaMainWindow::on_actionSend_database_by_email_activated(){
	on_action_Save_activated();
	AssumaMailDialog *dial = createMailDialog();
	foreach (QString to, AssumaCM->getConfigToken( AssumaConfigManager::ContactsList ).toString().split(";") ){
		dial->addEmail(to);
	}
	dial->setSubject( processTemplate( tr("[%ASSO_NAME%] Subscribers database") ) );
	dial->setBody( processTemplate( tr("Hello,\n\nplease find attached the subscribers database.\n\nBest regards,\n%ASSO_NAME%") ) );
	dial->addAttachement( m_currentDatabase );
	dial->show();
}

void AssumaMainWindow::on_action_Print_activated(){
#ifndef QT_NO_PRINTER
	QPrinter printer;
	printer.setOrientation(QPrinter::Landscape);
	printer.setPageMargins( 0.25, 0.5, 0.25, 0.5, QPrinter::Inch );
	QPrintDialog *dialog = new QPrintDialog(&printer, this);
	dialog->setWindowTitle(tr("Print Database"));
	if (dialog->exec() != QDialog::Accepted)
		return;
	QTextDocument doc;
	QString html = subscribersListToPrintableHtml();
	doc.setHtml( html );
	doc.print(&printer);
#endif
}

bool AssumaMainWindow::areFeesEquivalent( QString base, QString comp){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) => base : " << base << " comp : " << comp;
#endif
	// Trivial tests here (to speed up the tests) : if the 2 strings are exactly the same => true and if they have not the same size => false
	if ( base == comp ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : base == comp -> return true";
#endif
		return true;
	}
// 	else if( base.size() != comp.size() ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : have different sizes -> return false";
#endif
// 		return false;
// 	}
	
	// Now we have to take care of the non trivial part : if the strings have the same size but are not the same (they can be the same if the order is not the same).
	QStringList blist = base.split(";");
	QStringList clist = comp.split(";");
	/** "Tous les cours:1=80,2=150,3=200;Kendo:1=150,2=200" **/
	 for (int i = 0 ; i < blist.count(); ++i){
		QStringList blist_fee = blist.at(i).split(":");
		QString tmp = blist_fee.at(0) ;
		tmp.append(":.*");
		QRegExp rx( tmp );
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : about to search for " << tmp << "in " << clist;
#endif
		int pos = clist.indexOf( rx );
		if( pos != -1 ){
// 			qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : activity " << blist_fee.at(0) << " found in the clist at position " << pos;
			// Woohoo ! We have found an activity ! now we have to check the fees.
			foreach( QString tmp_cfee , clist.at(pos).split(":").at(1).split(",") ){
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : tmp_cfee ->" << tmp_cfee;
#endif
				bool tmpOkToContinue = false;
				foreach( QString tmp_bfee , blist_fee.at(1).split(",") ){
					if( tmp_bfee == tmp_cfee ){
						tmpOkToContinue = true;
						break;
					}
				}
				if( ! tmpOkToContinue ){
#ifdef ASSUMA_DEBUG
					qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : One ore more fees were not found in the list -> return false";
#endif
					return false;
				}
			}
		}
		else{
// 			qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : activity " << blist_fee.at(0) << " not found in the clist (pos=" << pos << ") -> return false";
			return false;
		}
	}
	// If we get here it means we have passed all test and the fees are equivalents, so...
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::areFeesEquivalent(QString, QString) : fees look equivalent to me...";
#endif
	return true;
}

// This method loads all federation related taxes (if any) and update the GUI.
void AssumaMainWindow::loadFederation(const QString& federationName){
	QString fileName( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/Federations/" + federationName + ".federation.xml" );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::loadFederation(" << federationName << ") : loading the following federation file : " << fileName;
#endif
	if( QFile::exists(fileName) ){
		// Load federation description file
		QFile file(fileName);
		QXmlInputSource inputSource(&file);
		QXmlSimpleReader reader;
		FederationHandler handler ;
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		reader.parse(inputSource);
		// TODO: We have to adapt code to the new FederationHandler !! (use FederationLicenseFee instead of the current QString in a QMap).
		// NOTE: this is done, but before everything is tested and functionnal I will not remove this todo.
		m_licences_fees.empty();
		m_licences_fees = handler.getLicenceFees();
// 		foreach( FederationLicenseFee *key, tmp_fees.keys() ){
// 			qDebug() << "AssumaMainWindow::loadFederation(" << federationName << ") : inserting into m_licences_fees -> " << key << "=" << tmp_fees.value(key);
// 			m_licences_fees.insert( key , tmp_fees.value(key) );
// 		}
		// Updating GUI.
		for( int i=0; i < licenseLabelComboBox->count(); ++i ){
			licenseLabelComboBox->removeItem(i);
		}
		QStringList labels_list;
		foreach( FederationLicenseFee *key, m_licences_fees ){
			licenseLabelComboBox->addItem( key->getName() );
			licenseLabelComboBox->setItemData( licenseLabelComboBox->count() - 1, key->getId(), AssumaMainWindow::IdRole  );
			licenseLabelComboBox->setItemData( licenseLabelComboBox->count() - 1, key->getName(), AssumaMainWindow::NameRole  );
			licenseLabelComboBox->setItemData( licenseLabelComboBox->count() - 1, key->getFee(), AssumaMainWindow::FeeRole  );
			labels_list << key->getName();
		}
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::loadFederation(" << federationName << ") : adding follwings items to the licenseLabelComboBox : " << labels_list;
#endif
		if( m_treeWidgetDelegate )
			m_treeWidgetDelegate->setLicenseLabelsList( labels_list );
		m_currentFederation = federationName;
	}
	else
		showPassiveMessage( tr("Federation %1 taxes file could not be load.<br>It means that fee calculation cannot be done correctly.").arg(federationName), AssumaConfigManager::Critical );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::loadFederation(" << federationName << ") : finished";
#endif
}

void AssumaMainWindow::loadDatabase(const QString& fileName){
	if( ! QFile::exists(fileName) ){
		// If we are still initializing the main window, the AssumaMainWindow::instance() will return an invalid pointer so, AssumaPassivePopup class won't 
		// be able to load images from the theme. So we are giving it the right image.
		if( m_isInitializing )
			showPassiveMessage( tr("Database <strong>%1</strong> could not be load.<br>File not found.").arg(fileName), QPixmap( getImageFromTheme("messagebox-critical") ) );
		else
			showPassiveMessage( tr("Database <strong>%1</strong> could not be load.<br>File not found.").arg(fileName), AssumaConfigManager::Critical );
		return;
	}
	QProgressDialog progress(tr("Please wait while loading subscibers database..."), "Abort loading", 0, 4, this);
	progress.setWindowModality(Qt::WindowModal);
	treeWidget->clear();
	progress.setWindowTitle(tr("Loading database..."));
	progress.setMinimumDuration(0);
	progress.setCancelButton( 0 );
	progress.setValue(0);
	this->update();
	QCoreApplication::processEvents(QEventLoop::AllEvents);
	m_dontUpdateSubscribersList=true;
	if(fileName.endsWith(".adb")){
		m_currentDatabase = fileName ;
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> Openning " << fileName;
#endif
		progress.setValue(1);
		progress.setLabelText(tr("Opening database: %1").arg(fileName));
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		QFile file(fileName);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> Creating QXmlInputSource";
#endif
		QXmlInputSource inputSource(&file);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> Creating QXmlSimpleReader";
#endif
		// Get the custom fields id/label equivalence
		QMap<QString,QString> cfEquivalence;
		foreach(AssumaCustomWidgetBase *wdg, m_customWidgets ){
			cfEquivalence.insert( wdg->id(), wdg->labelAsString() );
		}
		
		QXmlSimpleReader reader;
		AdbHandler handler(treeWidget);
		handler.setCustomFieldsMap( cfEquivalence );
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		progress.setLabelText(tr("Processing database..."));
		progress.setValue(2);
		progress.update();
		QCoreApplication::processEvents(QEventLoop::AllEvents);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> parsing inputSource";
#endif
		reader.parse(inputSource);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> inputSource parsed";
#endif
		progress.setLabelText(tr("Now exploiting database's data..."));
		progress.setValue(3);
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		exploitData();
		progress.setValue(4);
		progress.setLabelText(tr("Now exploiting database's data...done !"));
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow -> This file is saved with the following courses list : \n" << handler.getCoursesList() << "\nAnd config courses list is :\n" << AssumaCM->getConfigToken( AssumaConfigManager::CoursesList );
#endif
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow -> are fees equivalent ? " << areFeesEquivalent(handler.getCoursesList(), AssumaCM->getConfigToken( AssumaConfigManager::CoursesList ).toString() );
#endif
		if( (handler.getCoursesList().size() > 0 && ! areFeesEquivalent(handler.getCoursesList(), AssumaCM->getConfigToken( AssumaConfigManager::CoursesList ).toString() )) || handler.getStaticCost() != AssumaCM->getConfigToken( AssumaConfigManager::StaticCost ).toString() || handler.getTrimesterCount() != AssumaCM->getConfigToken( AssumaConfigManager::TrimesterCount ).toString() ){
			int ret = QMessageBox::question(this,tr("Association Subscribers Manager"),tr("The database you just load was written with a different set of settings (fees, static cost, etc.).\nDo you want to load this database's settings ?"),QMessageBox::Yes|QMessageBox::No);
			if( ret == QMessageBox::Yes){
				AssumaCM->setConfigToken(AssumaConfigManager::CoursesList, handler.getCoursesList() );
				activityComboBox->clear();
				activityComboBox->addItems( associationSettingsWidget->getCoursesList() );
				AssumaCM->setConfigToken(AssumaConfigManager::StaticCost, handler.getStaticCost() );
				AssumaCM->setConfigToken(AssumaConfigManager::TrimesterCount, handler.getTrimesterCount() );
			}
			
		}
		if( handler.getFederationName() != AssumaCM->getConfigToken( AssumaConfigManager::Federation ).toString() ){
			QString federationFileName( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/Federations/" + handler.getFederationName() + ".federation.xml" );
			if( QFile::exists( federationFileName )  ){
				int ret = QMessageBox::question(this,tr("Association Subscribers Manager"),tr("The database you just load was written with a different federation file (%1 instead of your current %2).\nDo you want to load the %1 federation file ?\n(This will not change your settings, only load the federation file for this session)").arg( handler.getFederationName() ).arg( AssumaCM->getConfigToken( AssumaConfigManager::Federation ).toString() ),QMessageBox::Yes|QMessageBox::No);
				if( ret == QMessageBox::Yes){
					loadFederation( handler.getFederationName() );
				}
			}
			else if( ! handler.getFederationName().isEmpty() ){
				// See comment about this trick in upper code.
				if( m_isInitializing )
					showPassiveMessage( tr("<b>WARNING !</b> The database you just load was written using the <u><b>%1</b></u> federation conditions,<br><u>you do not have this federation file on this computer</u> !<br>Continue the edition of this file will most likely leads to data compromission.").arg(handler.getFederationName()), QPixmap( getImageFromTheme("messagebox-critical") ) );
				else
					showPassiveMessage( tr("<b>WARNING !</b> The database you just load was written using the <u><b>%1</b></u> federation conditions,<br><u>you do not have this federation file on this computer</u> !<br>Continue the edition of this file will most likely leads to data compromission.").arg(handler.getFederationName()), AssumaConfigManager::Critical );
			}
		}
	}
	else if(fileName.endsWith(".xml")){
		m_currentDatabase = fileName;
		QMessageBox::warning(this,tr("Association Subscribers Manager"),tr("<html>You are loading an old version of the Association Subscriber Manager database,\nyour subscribers will be imported in the current view\nand you will have to save this database in the new format.\nThe new database files ends with \".adb\" (stands for <b>A</b>ssociation subscribers manager <b>D</b>ata<b>B</b>ase).</html>"));
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> Openning " << fileName;
#endif
		progress.setLabelText(tr("Opening database: %1").arg(fileName));
		QFile file(fileName);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> Creating QXmlInputSource";
#endif
		QXmlInputSource inputSource(&file);
		progress.setValue(1);
		QCoreApplication::processEvents(QEventLoop::AllEvents);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> Creating QXmlSimpleReader";
#endif
		QXmlSimpleReader reader;
		OldXmlHandler handler(treeWidget);
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		progress.setValue(2);
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		progress.setLabelText(tr("Processing database..."));
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> parsing inputSource";
#endif
		reader.parse(inputSource);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> inputSource parsed";
#endif
		progress.setLabelText(tr("Now exploiting database's data..."));
		progress.setValue(3);
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		exploitData();
		progress.setValue(4);
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
		m_currentDatabase = "";
	}
	m_dontUpdateSubscribersList=false;
	treeWidget->sortItems( 0, Qt::AscendingOrder );
	for( int i = 0; i < treeWidget->headerItem()->columnCount(); i++){
		treeWidget->resizeColumnToContents(i);
	}
	setWindowModified(false);
	if( m_isInitializing )
		showPassiveMessage( tr( "Database <strong>%1</strong> <br/>successfully loaded." ).arg(m_currentDatabase), QPixmap( getImageFromTheme("messagebox-info") ) );
	else
		showPassiveMessage( tr( "Database <strong>%1</strong> <br/>successfully loaded." ).arg(m_currentDatabase) );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> Application directory path:" << ASSUMA_DATADIR;
#endif
}

void AssumaMainWindow::exploitData(){
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	// storing those variables here will considerably speed up the application and reduce memory usage
	int dsc = elementColumn(tr("DuesStatus"));
	int mcsc = elementColumn(tr("MedicalCertificateStatus"));
	//int pc = elementColumn("Picture");
	QIcon ok_icon = QIcon( getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "button-ok" ) );
	QIcon nok_icon = QIcon(getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "button-nok" ));
	QPixmap defaultPixmap( getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "user-default" ) );
	for(int k=0;k<rootitem->childCount(); k++){
		QTreeWidgetItem *it = rootitem->child(k);
		// Setting all items editable
		it->setFlags(it->flags() | Qt::ItemIsEditable);
		QPixmap pix = QPixmap( it->text(elementColumn(tr("Picture"))) );
		if(!pix)
			pix = defaultPixmap;
		it->setIcon( 0, QIcon(pix) );
		// Setting text and icon for DuesStatus column
		if( it->text(dsc) == "1" ){
			it->setIcon(dsc, ok_icon );
			it->setText(dsc, tr("Ok") );
		}
		else{
			it->setIcon(dsc, nok_icon );
			it->setText(dsc, tr("Not ok") );
		}
		// Setting text and icon for MedicalCertificateStatus column
		if( it->text(mcsc) == "1" ){
			it->setIcon(mcsc, ok_icon );
			it->setText(mcsc, tr("Ok") );
		}
		else{
			it->setIcon(mcsc, nok_icon );
			it->setText(mcsc, tr("Not ok") );
		}
		// Setting the license label if there is one (after parsing the .adb file there is only an id here, but it can also contains a license name from an old database)
		QString license_label = licenseNameById( it->text( elementColumn(tr("LicenseLabel")) ) );
#ifdef ASSUMA_DEBUG
		qDebug() << " **** AssumaMainWindow::exploitData -> try to match a license id for " << it->text( elementColumn(tr("LicenseLabel")) ) << " match is " << license_label;
#endif
		if( license_label != "" ){
			it->setData( elementColumn(tr("LicenseLabel")), AssumaMainWindow::IdRole, it->text( elementColumn(tr("LicenseLabel")) ) );
			it->setText( elementColumn(tr("LicenseLabel")), license_label );
		}
		bool uSL_previous = m_dontUpdateSubscribersList;
		m_dontUpdateSubscribersList=false;
		for( int i=0; i < it->columnCount(); i++ )
			on_treeWidget_itemChanged(it,i);
		m_dontUpdateSubscribersList = uSL_previous;
	}
	
	for ( int i = 0; i < treeWidget->columnCount(); i++ ){
		treeWidget->resizeColumnToContents(i);
	}
}

void AssumaMainWindow::exploitItemData( QTreeWidgetItem *it ){
	int dsc = elementColumn(tr("DuesStatus"));
	int mcsc = elementColumn(tr("MedicalCertificateStatus"));
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::exploitItemData() : got an Item to exploit with DuesStatus="<<it->text(dsc)<<" and MedicalCertificateStatus="<<it->text(mcsc);
#endif
	QIcon ok_icon = QIcon(getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "button-ok" ));
	QIcon nok_icon = QIcon(getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "button-nok" ));
	if( it->text(dsc) == "1" ||  it->text(dsc) == "Ok"){
		it->setIcon(dsc, ok_icon );
		it->setText(dsc, tr("Ok") );
	}
	else{
		it->setIcon(dsc, nok_icon );
		it->setText(dsc, tr("Not ok") );
	}
	// Setting text and icon for MedicalCertificateStatus column
	if( it->text(mcsc) == "1" ||  it->text(mcsc) == "Ok" ){
		it->setIcon(mcsc, ok_icon );
		it->setText(mcsc, tr("Ok") );
	}
	else{
		it->setIcon(mcsc, nok_icon );
		it->setText(mcsc, tr("Not ok") );
	}
	// Setting the license label if there is one (after parsing the .adb file there is only an id here, but it can also contains a license name from an old database)
	QString license_label = licenseNameById( it->text( elementColumn(tr("LicenseLabel")) ) );
	if( license_label != "" ){
		it->setData( elementColumn(tr("LicenseLabel")), AssumaMainWindow::IdRole, it->text( elementColumn(tr("LicenseLabel")) ) );
		it->setText( elementColumn(tr("LicenseLabel")), license_label );
	}
}

void AssumaMainWindow::on_action_Save_activated(){
	if( m_currentDatabase.isEmpty() ){
		on_actionSave_as_activated();
	}
	else{
		if(writeDatabase(m_currentDatabase)){
// 			setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
// 			treeWidget->setWindowModified(false);
			setWindowModified(false);
			showPassiveMessage( tr("database saved as <strong>%1</strong>").arg(m_currentDatabase) );
		}
		else{
			QFileInfo fi(m_currentDatabase);
			if( ! fi.isWritable() )
				showPassiveMessage( tr("database <strong>%1</strong> could not be saved !<br>You don't have the permission to write this file.").arg(m_currentDatabase), AssumaConfigManager::Critical );
			else
				showPassiveMessage( tr("database <strong>%1</strong> could not be saved !").arg(m_currentDatabase), AssumaConfigManager::Critical );
		}
	}
}

bool AssumaMainWindow::writeDatabase(const QString & fileName){
	QFile file(fileName);
	if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
		return false;
	}
	QTextStream out(&file);
	out.setCodec("UTF-8");
	out << subscribersListToXml() << "\n";
	
// 	QFile filec( fileName + ".gz" );
// 	if (!filec.open(QIODevice::WriteOnly | QIODevice::Text))
// 		return;
// 	QDataStream outc(&filec);
// 	outc << qCompress( subscribersListToXml().toAscii(), 9 );
	
	return true;
}

int AssumaMainWindow::priceToInt(QString & price){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->priceToInt(const QString & price) : price=" << price;
#endif
	int pos = price.indexOf(" ");
	if(pos != -1){
		price.remove(pos,100);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow->priceToInt(const QString & price) : returning=>\""<<price<<"\"";
#endif
		return price.toInt();
	}
	else
		return -1;
}


QString AssumaMainWindow::subscribersListToHtml(){
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	QStringList html;
	html << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
	html << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
	html << "<head>\n" ;
	html << "<title>" << AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() << ": " << tr("Subscribers list") << "</title>\n"; 
	html << "<meta http-equiv=\"Content-type\" content=\"text/html; charset=utf-8\" />";
	html << "<style type=\"text/css\"><!--\n";
	html << "table.table { border-top: 1px solid black; border-bottom: 1px solid black; border-left: 1px solid black;}\n";
	html << ".table_cell { border-top: 1px solid black; border-right: 1px solid black; text-align : center; } \n";
	html << "th { border-right: 1px solid black;}\n";
	html << ".table_row_header { background: #80acff;}\n";
	html << ".table_row_odd { background: #a2d7ff;}\n";
	html << ".table_row:hover, .table_row_odd:hover { background: #a7bfd2;}\n";
	html << "#info_box { border: 1px solid black;}\n";
	html << ".box_title { position: relative; width: 100%;  background: #80acff; }\n";
	html << " --></style>";
	html << "</head>\n<body>\n";
	html << "<div id=\"info_box\">\n";
	html << "<div class=\"box_title\"><strong>" << tr("Association: ") << "</strong>" << AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() << "</div>\n";
	html << "<strong>" << tr("Number of subscribers") << ": </strong>" << QString::number(rootitem->childCount()) << "<br/>\n";
	html << "</div><br/>\n";
	html << "<table class=\"table\" cellspacing=\"0\">\n";
	QTreeWidgetItem *header = treeWidget->headerItem();
	html << "<tr class=\"table_row_header\">\n";
	for ( int j=0 ; j < header->columnCount() ; ++j ){
		if(header->text(j) != tr("Picture"))
			html << "<th>" << header->text(j) << "</th>";
	}
	html << "</th>\n";
	bool isOdd = false;
	for(int k=0;k<rootitem->childCount(); k++)
	{
		QTreeWidgetItem *it = rootitem->child(k);
		if(isOdd){
			html << "<tr class=\"table_row_odd\">" ;
			isOdd = false;
		}
		else{
			html << "<tr class=\"table_row\">" ;
			isOdd = true;
		}
		for ( int j=0 ; j < it->columnCount() ; ++j ){
			if(header->text(j) != tr("Picture"))
				html << "<td class=\"table_cell\">" << it->text(j) << "</td>";
		}
		html << "</tr>\n";
	}
	html << "</table>\n</body>\n</html>\n";
	return html.join("");
}

QString AssumaMainWindow::subscribersListToPrintableHtml(){
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	QStringList html;
	html << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
	html << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
	html << "<head>\n" ;
	html << "<title>" << AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() << ": " << tr("Subscribers list") << "</title>\n"; 
	html << "<meta http-equiv=\"Content-type\" content=\"text/html; charset=utf-8\" />";
	html << "</head>\n<body>\n";
	html << "<div>\n";
	html << "<strong>" << tr("Association: ") << "</strong>" << AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() << "<br/>\n";
	html << "<strong>" << tr("Number of subscribers") << ": </strong>" << QString::number(rootitem->childCount()) << "<br/>\n";
	html << "</div><br/><font size=\"10px\">\n";
	html << "<table cellspacing=\"0\" border=\"1\">\n";
	QTreeWidgetItem *header = treeWidget->headerItem();
	html << "<tr bgcolor=\"#80acff\">\n";
	for ( int j=0 ; j < header->columnCount() ; ++j ){
		if(header->text(j) != tr("Picture"))
			html << "<th>" << header->text(j) << "</th>";
	}
	html << "</th>\n";
	bool isOdd = false;
	for(int k=0;k<rootitem->childCount(); k++)
	{
		QTreeWidgetItem *it = rootitem->child(k);
		if(isOdd){
			html << "<tr bgcolor=\"#a2d7ff\">" ;
			isOdd = false;
		}
		else{
			html << "<tr>" ;
			isOdd = true;
		}
		for ( int j=0 ; j < it->columnCount() ; ++j ){
			if(header->text(j) != tr("Picture"))
				html << "<td align=\"center\" valign=\"middle\">" << it->text(j) << "</td>";
		}
		html << "</tr>\n";
	}
	html << "</table>\n</font></body>\n</html>\n";
	return html.join("");
}

QString AssumaMainWindow::subscribersListToXml(){
// 	int embedImages = QMessageBox::question( this, tr("Embed images ?"), 
// 				tr("Association Subscribers Manager can embed images (like subscribers pictures),<br>This will produce a bigger file but save you the trouble to manage pictures separatly.<br><br><strong>Do you want to embed all subscribers pictures ?</strong>"),
// 				QMessageBox::Yes|QMessageBox::No
// 			);
	QProgressDialog progress(tr("Please wait while saving subscibers database..."), "Abort", 0, treeWidget->topLevelItemCount(), this);
	progress.setWindowModality(Qt::WindowModal);
	progress.setWindowTitle(tr("Writing database..."));
	progress.setMinimumDuration(0);
	progress.setCancelButton( 0 );
	progress.setValue(0);
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	QString xml = "<?xml version=\"1.0\" encoding=\"utf-8\" standalone=\"yes\"?>\n<Assuma adbversion=\""+ASSUMA_DB_VERSION+"\" itemcount=\"";
	xml.append( QString::number(rootitem->childCount()) ); xml.append("\" ");
	xml.append("courseslist=\"");xml.append( AssumaCM->getConfigToken(AssumaConfigManager::CoursesList).toString() );xml.append("\" ");
	xml.append("staticcost=\"");xml.append( AssumaCM->getConfigToken(AssumaConfigManager::StaticCost).toString() );xml.append("\" ");
	xml.append("trimestercount=\"");xml.append( AssumaCM->getConfigToken(AssumaConfigManager::TrimesterCount).toString() );xml.append("\" ");
	xml.append("federation=\"");xml.append( AssumaCM->getConfigToken(AssumaConfigManager::Federation).toString() );xml.append("\" ");
	xml.append(">\n");
	int progression = 1;
	for(int k=0;k<rootitem->childCount(); k++)
	{
		QTreeWidgetItem *it = rootitem->child(k);
		calculateDue(it);
		xml.append("\t<Subscriber>\n");
		xml.append("\t\t<LastName><![CDATA["); xml.append( it->text( elementColumn(tr("LastName")) ) ); xml.append("]]></LastName>\n");
		xml.append("\t\t<MaidenName><![CDATA["); xml.append( it->text( elementColumn(tr("MaidenName")) ) ); xml.append("]]></MaidenName>\n");
		xml.append("\t\t<FirstName><![CDATA["); xml.append( it->text( elementColumn(tr("FirstName")) ) ); xml.append("]]></FirstName>\n");
		xml.append("\t\t<BirthDate><![CDATA["); xml.append( it->text( elementColumn(tr("BirthDate")) ) ); xml.append("]]></BirthDate>\n");
		xml.append("\t\t<Address><![CDATA["); xml.append( it->text( elementColumn(tr("Address")) ) ); xml.append("]]></Address>\n");
		xml.append("\t\t<City><![CDATA["); xml.append( it->text( elementColumn(tr("City")) ) ); xml.append("]]></City>\n");
		xml.append("\t\t<PostalCode><![CDATA["); xml.append( it->text( elementColumn(tr("PostalCode")) ) ); xml.append("]]></PostalCode>\n");
		xml.append("\t\t<PhoneNumber><![CDATA["); xml.append( it->text( elementColumn(tr("PhoneNumber")) ) ); xml.append("]]></PhoneNumber>\n");
		xml.append("\t\t<Email><![CDATA["); xml.append( it->text( elementColumn(tr("Email")) ) ); xml.append("]]></Email>\n");
		xml.append("\t\t<Activity><![CDATA["); xml.append( it->text( elementColumn(tr("Activity")) ) ); xml.append("]]></Activity>\n");
		xml.append("\t\t<LessonsNumber><![CDATA["); xml.append( it->text( elementColumn(tr("LessonsNumber")) ) ); xml.append("]]></LessonsNumber>\n");
		if( ! it->data( elementColumn( tr("LicenseLabel") ), AssumaMainWindow::IdRole ).toString().isEmpty() ){
			xml.append("\t\t<LicenseLabel><![CDATA["); xml.append( it->data( elementColumn( tr("LicenseLabel") ), AssumaMainWindow::IdRole ).toString() ); xml.append("]]></LicenseLabel>\n");
		}
		else{
			xml.append("\t\t<LicenseLabel><![CDATA["); xml.append( licenseIdByName( it->text( elementColumn( tr("LicenseLabel") ) )) ); xml.append("]]></LicenseLabel>\n");
		}
#ifdef ASSUMA_DEBUG
		QString dues_str = it->text( elementColumn( tr("Dues") ) );
		qDebug() << "AssumaMainWindow::subscribersListToXml() : dues_str=" << dues_str;
// 		int dues = priceToInt( dues_str );
		int dues = dues_str.toInt();
		qDebug() << "AssumaMainWindow::subscribersListToXml() : int dues=\""<<dues<<"\"";
#endif
		xml.append("\t\t<Dues><![CDATA["); xml.append( it->text( elementColumn( tr("Dues") ) ) ); xml.append("]]></Dues>\n");
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::subscribersListToXml() : xml=" << xml;
#endif
		if( it->text( elementColumn(tr("DuesStatus")) ) == tr("Ok") ){
			xml.append("\t\t<DuesStatus><![CDATA[1]]></DuesStatus>\n");
		}
		else{
			xml.append("\t\t<DuesStatus><![CDATA[0]]></DuesStatus>\n");
		}
		xml.append("\t\t<SubscriptionDate><![CDATA["); xml.append( it->text( elementColumn(tr("SubscriptionDate")) ) ); xml.append("]]></SubscriptionDate>\n");
		
		if( it->text( elementColumn(tr("MedicalCertificateStatus")) ) == tr("Ok") ){
			xml.append("\t\t<MedicalCertificateStatus><![CDATA[1]]></MedicalCertificateStatus>\n");
		}
		else{
			xml.append("\t\t<MedicalCertificateStatus><![CDATA[0]]></MedicalCertificateStatus>\n");
		}
		
		xml.append("\t\t<LicenseNumber><![CDATA["); xml.append( it->text( elementColumn(tr("LicenseNumber")) ) ); xml.append("]]></LicenseNumber>\n");
		xml.append("\t\t<Comment><![CDATA["); xml.append( it->text( elementColumn(tr("Comment")) ) ); xml.append("]]></Comment>\n");
		
		xml.append("\t\t<Picture embed=\"0\"><![CDATA["); xml.append( it->text( elementColumn(tr("Picture")) ) ); xml.append("]]></Picture>\n");
// 		if(embedImages == QMessageBox::No){
// 			xml.append("\t\t<Picture embed=\"0\"><![CDATA["); xml.append( it->text( elementColumn(tr("Picture")) ) ); xml.append("]]></Picture>\n");
// 		}
// 		else{
// 			QPixmap pixmap( it->text( elementColumn(tr("Picture")) ) );
// 			QByteArray bytes;
// 			QBuffer buffer(&bytes);
// 			buffer.open(QIODevice::WriteOnly);
// 			pixmap.scaled(QSize(64,64), Qt::KeepAspectRatio).save(&buffer, "PNG"); 
// 			xml.append("\t\t<Picture embed=\"1\"><![CDATA[\n");
// 			xml.append( bytes.toBase64() );
// 			xml.append("\n\t\t]]></Picture>\n");
// 		}
		foreach( AssumaCustomWidgetBase *wdg, m_customWidgets ){
			xml.append( "\t\t<CustomField id=\"" );
			xml.append( wdg->id() );
			xml.append( "\"><![CDATA[" );
			xml.append( it->text( elementColumn( wdg->labelAsString() ) ) );
			xml.append( "]]></CustomField>\n" );
		}
		xml.append("\t</Subscriber>\n");
		progress.setValue(progression);
		progression++;
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		
	}
	xml.append("</Assuma>");
	progress.setValue(treeWidget->topLevelItemCount());
	return xml;
}

bool AssumaMainWindow::maybeSave(){
#ifdef ASSUMA_DEBUG
	qDebug() << "treeWidget->isWindowModified() = " << treeWidget->isWindowModified() << " and isWindowModified() = " << isWindowModified();
#endif
	if( isWindowModified() ) {
		QMessageBox::StandardButton ret;
		ret = QMessageBox::warning(this, tr("Association Subscribers Manager"),
			tr("The current database has been modified.\n"
			"Do you want to save your changes?"),
			QMessageBox::Save | QMessageBox::Discard | QMessageBox::Cancel);
			if (ret == QMessageBox::Save){
				on_action_Save_activated();
				return true;
			}
			else if (ret == QMessageBox::Cancel)
				return false;
	}
	if( m_shouldSavePostalCodes )
		writePostalCodes(m_postal_codes_filename);
	return true;
}

void AssumaMainWindow::closeEvent(QCloseEvent *event)
{
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> Got a close event.";
#endif
	if (maybeSave()) {
		writeSettings();
		m_addonEngine->destroyAllAddons();
		event->accept();
	} else {
		event->ignore();
	}
}

int AssumaMainWindow::elementColumn(const QString& element){
	QTreeWidgetItem *header = treeWidget->headerItem();
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		if( element == header->text(i) )
			return i;
	}
	return -1;
}

int AssumaMainWindow::elementColumn(const QTreeWidgetItem *header, const QString& element){
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		if( element == header->text(i) )
			return i;
	}
	return -1;
}

void AssumaMainWindow::on_editSubscriberButton_toggled(bool b){
	if(b){
		editArea->show();
		textBrowser->hide();
		pictureEdit->show();
		pictureLabel->hide();
		editSubscriberButton->setStyleSheet("background-color: rgb(255, 99, 99);border: 2px solid darkkhaki;border-radius: 3px;");
	}
	else{
		on_treeWidget_currentItemChanged( treeWidget->currentItem(), 0 ); // I often use this to refresh the textBrowser content.
		editArea->hide();
		textBrowser->show();
		pictureEdit->hide();
		pictureLabel->show();
		editSubscriberButton->setStyleSheet("");
		on_pictureEdit_toggled(false);
	}
// 	setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
// 	treeWidget->setWindowModified(true);
	if( ! m_isInitializing )
		setWindowModified(true);
}

void AssumaMainWindow::on_actionSave_as_activated(){
	QString path = m_currentDatabase;
	if( path.isEmpty() ){
		path =".";
	}
	 QString fileName = QFileDialog::getSaveFileName(this, tr("Save File"),path,tr("ADB database (*.adb)"));
	 if(fileName.size() > 0){
	 	m_currentDatabase = fileName;
		if( !m_currentDatabase.endsWith(".adb") )
		    m_currentDatabase.append(".adb");
		on_action_Save_activated();
// 		setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
	 }
// 	 qDebug() << "AssumaMainWindow-> Database file (save as) : " << m_currentDatabase;
}

void AssumaMainWindow::updateSplitterSize(){
	int tww = treeWidget->width();
	int fw = frame->width();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::updateSplitterSize() -> splitter sizes : " << splitter->sizes() << " treeWidget width : " << tww << " frame width : " << fw;
#endif
	splitter->setSizes( QList<int>()<< tww << fw  );
}

void AssumaMainWindow::on_actionAssociation_settings_toggled(bool isChecked){
	if( isChecked ){
		if( actionEdit_subscriber->isChecked() )
			actionEdit_subscriber->setChecked(false);
		if( actionSettings->isChecked() )
			actionSettings->setChecked(false);
		splitter->hide();
		configWidget->hide();
		associationSettingsWidget->show();
	}
	else{
		configWidget->hide();
		associationSettingsWidget->hide();
		splitter->show();
	}
}

void AssumaMainWindow::on_action_Find_toggled(bool visible){
	if(visible){
		
		
		AssumaEffect *tw_effect = new AssumaEffect(0,treeWidget,AssumaEffect::SHRINK_VERTICALLY);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::on_action_Find_toggled(bool visible) -> searchWidget size : " << searchWidget->size() << " searchWidget sizeHint : " << searchWidget->sizeHint() << " treeWidget size : " << treeWidget->size();
#endif
		tw_effect->setVSizeDelta( searchWidget->size().height() + 2 );
		tw_effect->setReversing(false);
		
		searchWidget->move(0, ( treeWidget->pos().y()+treeWidget->height() ) );
		AssumaEffect *effect = new AssumaEffect(0,searchWidget,AssumaEffect::GROW_VERTICALLY);
		effect->setReversing(false);
// 		connect( tw_effect, SIGNAL( AssumaEffectFinished() ), effect, SLOT( doEffect() ) );
		
		
		int tw_effect_size = 750;
		if( searchWidget->size().width() >= 750 )
			tw_effect_size = searchWidget->size().width();
		if( tw_effect_size > treeWidget->size().width() ){
			AssumaEffect *expand_effect = new AssumaEffect(0,treeWidget,AssumaEffect::EXPAND_HORIZONTALLY);
			expand_effect->setVSizeDelta( tw_effect_size - treeWidget->size().width() );
			expand_effect->setReversing(false);
			connect( expand_effect, SIGNAL( AssumaEffectFinished() ), effect, SLOT( doEffect() ) );
			connect( expand_effect, SIGNAL( AssumaEffectFinished() ), tw_effect, SLOT( doEffect() ) );
			connect( expand_effect, SIGNAL( AssumaEffectWasUpdated() ), this, SLOT( updateSplitterSize() ) );
			expand_effect->doEffect();
		}
		else{
// 			qDebug() << "AssumaMainWindow::on_action_Find_toggled(bool visible) -> starting GROW_VERTICALLY effect on searchWidget";
			effect->doEffect();
// 			qDebug() << "AssumaMainWindow::on_action_Find_toggled(bool visible) -> starting SHRINK_VERTICALLY effect on treeWidget";
			tw_effect->doEffect();
		}
		searchWidget->setFocus();
	}
	else{
		AssumaEffect *tw_effect = new AssumaEffect(0,treeWidget,AssumaEffect::EXPAND_VERTICALLY);
		tw_effect->setVSizeDelta( searchWidget->size().height() + 2 );
		tw_effect->setReversing(false);
		AssumaEffect *effect = new AssumaEffect(0,searchWidget,AssumaEffect::SLIM_VERTICALLY);
		effect->setReversing(false);
		connect( effect, SIGNAL( AssumaEffectFinished() ), searchWidget, SLOT( hide() ) );
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::on_action_Find_toggled(bool visible) -> starting EXPAND_VERTICALLY effect on treeWidget";
#endif
		tw_effect->doEffect();
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::on_action_Find_toggled(bool visible) -> starting SLIM_VERTICALLY effect on searchWidget";
#endif
		effect->doEffect();
// 		searchWidget->hide();
	}
}

void AssumaMainWindow::readSettingsInConfigurationManager(){
	QSettings settings("Infinity Perl","assuma");
	// Loading the whole configuration into the configuration manager
	AssumaCM->setConfigToken( AssumaConfigManager::MainWindowGeometry, settings.value("mw_geometry") );
	AssumaCM->setConfigToken( AssumaConfigManager::MainWindowSplitterGeometry, settings.value("splitter_geometry") );
	AssumaCM->setConfigToken( AssumaConfigManager::TrimesterCount, settings.value("configTrimesterCount", QString::number(3)) );
	AssumaCM->setConfigToken( AssumaConfigManager::StaticCost, settings.value("configStaticCost") );
	AssumaCM->setConfigToken( AssumaConfigManager::DataDir, settings.value("configDataDir", getDirectory("data") ) );
	// DataDir is set, this is all we need to load the list of available languages.
	loadLanguageList( AssumaCM->getConfigToken(AssumaConfigManager::DataDir).toString().append("/langpack.lst") );
	// Getting the list of federation files
	QDir dataDir( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString()+"/Federations" );
	QStringList filters;
	QStringList federation_list;
	filters << "*.federation.xml";
	dataDir.setNameFilters(filters);
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> federation files : " << dataDir.entryList();
#endif
	foreach ( QString it, dataDir.entryList() ){
		it.remove(".federation.xml");
		federation_list << it;
	}
	AssumaCM->setConfigToken( AssumaConfigManager::FederationList , federation_list.join(";") );
	AssumaCM->setConfigToken(AssumaConfigManager::Federation, settings.value("configFederation","DefaultNull"));
	AssumaCM->setConfigToken( AssumaConfigManager::UpdateServer, settings.value("configUpdateServer","http://assuma.sourceforge.net") );
	AssumaCM->setConfigToken( AssumaConfigManager::UpdatesOnStart, settings.value("configUpdatesOnStart",QString::number(1)) );
	AssumaCM->setConfigToken( AssumaConfigManager::ContactsList, settings.value("configContactsList","") );
	AssumaCM->setConfigToken( AssumaConfigManager::CoursesList, settings.value("configCoursesList","") );
	AssumaCM->setConfigToken( AssumaConfigManager::Language, settings.value("configLanguage","English") );
	AssumaCM->setConfigToken( AssumaConfigManager::Encoding, settings.value("configEncoding","UTF-8") );
	AssumaCM->setConfigToken( AssumaConfigManager::AssociationName, settings.value("configAssociationName","MyAssociation") );
	AssumaCM->setConfigToken( AssumaConfigManager::Email, settings.value("configEmail","Owner@MyAssociation.org") );
	AssumaCM->setConfigToken( AssumaConfigManager::SmtpServer, settings.value("configSmtpServer","localhost") );
	AssumaCM->setConfigToken( AssumaConfigManager::SmtpLogin, settings.value("configSmtpLogin","") );
	AssumaCM->setConfigToken( AssumaConfigManager::SmtpPassword, settings.value("configSmtpPassword","") );
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyServer, settings.value("configProxyServer","") );
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyLogin, settings.value("configProxyLogin","") );
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyPassword, settings.value("configProxyPassword","") );
	AssumaCM->setConfigToken( AssumaConfigManager::ProxyEnabled, settings.value("configProxyEnabled","0") );
	AssumaCM->setConfigToken( AssumaConfigManager::Country, settings.value("configCountry","SmallTest") );
	AssumaCM->setConfigToken( AssumaConfigManager::ActivatedAddonList, settings.value("configActivatedAddonList","") );
	AssumaCM->setConfigToken( AssumaConfigManager::CurrenciesList, settings.value("configCurrenciesList",QString::fromUtf8("Baht (฿);Dollar ($);Dong (₫);Euro (\xe2\x82\xac);Hryvnia (₴);Real (R$);Won (₩);Pound (\xc2\xa3);svensk krona (SEK);Yen (¥);Yuan (元)")) );
	AssumaCM->setConfigToken( AssumaConfigManager::Currency, settings.value("configCurrency", QString::fromUtf8("Euro (\xe2\x82\xac)")) );
	AssumaCM->setConfigToken( AssumaConfigManager::MainWindowLastDatabase, settings.value("last_database","") );
	AssumaCM->setConfigToken( AssumaConfigManager::UserDefinedAvatars, settings.value("userDefinedAvatars",QStringList()) );
	AssumaCM->setConfigToken( AssumaConfigManager::Theme, settings.value("configTheme", "default") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );
// 	AssumaCM->setConfigToken( AssumaConfigManager::, settings.value("") );

	// Restoring column status
	QRegExp rx("show.+Column");
	QStringList tmpColumnList;
	foreach( QString key, settings.allKeys() ){
		if( key.contains( rx ) ){
			QString key_replaced = key;
			key_replaced.replace("show","");
			key_replaced.replace("Column","");
			tmpColumnList << key_replaced;
			AssumaCM->setTmpConfigToken(key_replaced,settings.value(key,true).toBool());
		}
	}
	AssumaCM->setTmpConfigToken("mainTreeWidgetColumnList",tmpColumnList);
	// Settings loading end
}

void AssumaMainWindow::updateColumnVisibility(){
	// Restoring column status
	QRegExp rx("show.+Column");
	QStringList tmpColumnList = AssumaCM->getTmpConfigToken("mainTreeWidgetColumnList").toStringList() ;
	foreach( QString key, tmpColumnList ){
		QAction *action = findChild<QAction *>("actionShow_"+key+"_column");
		if(action){
			action->setChecked( AssumaCM->getTmpConfigToken( key ).toBool() );
		}
// 		if( key.contains( rx ) ){
// 			QString key_replaced = key;
// 			key_replaced.replace("show","");
// 			key_replaced.replace("Column","");
// 			 QAction *action = findChild<QAction *>("actionShow_"+key_replaced+"_column");
// 			 if(action){
// 				 action->setChecked( settings.value(key,true).toBool() );
// 			 }
// 		}
	}
	// Settings loading end
}

void AssumaMainWindow::readSettings(){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> Reading settings";
#endif
	readSettingsInConfigurationManager();
// 	QSettings settings("Infinity Perl","assuma");

// 	restoreGeometry(settings.value("mw_geometry").toByteArray());
	restoreGeometry(AssumaCM->getConfigToken(AssumaConfigManager::MainWindowGeometry).toByteArray());
	splitter->restoreState(AssumaCM->getConfigToken(AssumaConfigManager::MainWindowSplitterGeometry).toByteArray());
	
	// Restoring settings from the configuration widget
// 	configWidget->setConfigToken(AssumaConfigManager::TrimesterCount, AssumaCM->getConfigToken(AssumaConfigManager::TrimesterCount).toString());
	
// 	configWidget->setConfigToken(AssumaConfigManager::StaticCost, settings.value("configStaticCost",QString::number(0)).toString());
// 	configWidget->setConfigToken(AssumaConfigManager::DataDir, settings.value("configDataDir",QString(ASSUMA_DATADIR) + "/data").toString() );

	// DataDir is set, this is all we need to load the list of available languages.
// 	loadLanguageList( AssumaCM->getConfigToken(AssumaConfigManager::DataDir).toString().append("/langpack.lst") );

	
	// Getting the list of Federation files
// 	QDir dataDir( AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString()+"/Federations" );
// 	QStringList filters;
// 	QStringList federation_list;
// 	filters << "*.federation.xml";
// 	dataDir.setNameFilters(filters);
// #ifdef ASSUMA_DEBUG
// 	qDebug() << "AssumaMainWindow -> federation files : " << dataDir.entryList();
// #endif
// 	foreach ( QString it, dataDir.entryList() ){
// 		it.remove(".federation.xml");
// 		federation_list << it;
// 	}
// 	configWidget->setConfigToken( AssumaConfigManager::FederationList , federation_list.join(";") );
// 	configWidget->setConfigToken(AssumaConfigManager::Federation, settings.value("configFederation","DefaultNull").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::UpdateServer, settings.value("configUpdateServer","http://assuma.sourceforge.net").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::UpdatesOnStart, settings.value("configUpdatesOnStart",QString::number(1)).toString());
// 	configWidget->setConfigToken(AssumaConfigManager::ContactsList, settings.value("configContactsList","").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::CoursesList, settings.value("configCoursesList","").toString());
	if( AssumaCM->getConfigToken( AssumaConfigManager::CoursesList ).toString() == "" )
		associationSettingsWidget->addActivity( tr("All courses") );
// 	configWidget->setConfigToken(AssumaConfigManager::Language, settings.value("configLanguage","English").toString());
	changeLanguage( AssumaCM->getConfigToken( AssumaConfigManager::Language ).toString() );
// 	configWidget->setConfigToken(AssumaConfigManager::Encoding, settings.value("configEncoding","UTF-8").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::AssociationName, settings.value("configAssociationName","MyAssociation").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::Email, settings.value("configEmail","Owner@MyAssociation.org").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::SmtpServer, settings.value("configSmtpServer","localhost").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::SmtpLogin, settings.value("configSmtpLogin","").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::SmtpPassword, settings.value("configSmtpPassword","").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::ProxyServer, settings.value("configProxyServer","").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::ProxyLogin, settings.value("configProxyLogin","").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::ProxyPassword, settings.value("configProxyPassword","").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::ProxyEnabled, settings.value("configProxyEnabled","0").toString());
// 	configWidget->setConfigToken(AssumaConfigManager::Country, settings.value("configCountry","SmallTest").toString());
	// We have to update the addons list before setting the list of activated ones or else the list being empty we lose settings
	configWidget->updateAddonList();
// 	configWidget->setConfigToken(AssumaConfigManager::ActivatedAddonList, settings.value("configActivatedAddonList","").toString());
// 	configWidget->setConfigToken( AssumaConfigManager::CurrenciesList, settings.value("configCurrenciesList",QString::fromUtf8("Baht (฿);Dollar ($);Dong (₫);Euro (\xe2\x82\xac);Hryvnia (₴);Real (R$);Won (₩);Pound (\xc2\xa3);svensk krona (SEK);Yen (¥);Yuan (元)")).toString() );
// 	configWidget->setConfigToken( AssumaConfigManager::Currency, settings.value( "configCurrency", QString::fromUtf8("Euro (€)") ).toString() );
	
	QString arg_db;
	foreach ( QString str, QCoreApplication::arguments() ){
		if( QFile::exists(str) && str.endsWith(".adb") )
			arg_db = str;
	}
	if(arg_db.isEmpty())
		m_currentDatabase = AssumaCM->getConfigToken( AssumaConfigManager::MainWindowLastDatabase ).toString() /*settings.value("last_database","").toString()*/ ;
	else
		m_currentDatabase = arg_db;
// 	QStringList images = settings.value("userDefinedAvatars",QStringList()).toStringList();
	QStringList images = AssumaCM->getConfigToken(AssumaConfigManager::UserDefinedAvatars).toStringList();
	for (int k = 0; k < images.count(); ++k){
		addAvatar( images.at(k) );
	}
	
	// Restoring column status
// 	actionShow_FirstName_column->setChecked( settings.value("showFirstNameColumn",true).toBool() );
// 	actionShow_MaidenName_column->setChecked( settings.value("showMaidenNameColumn",true).toBool() );
// 	actionShow_LastName_column->setChecked( settings.value("showLastNameColumn",true).toBool() );
// 	actionShow_BirthDate_column->setChecked( settings.value("showBirthDateColumn",true).toBool() );
// 	actionShow_City_column->setChecked( settings.value("showCityColumn",true).toBool() );
// 	actionShow_PostalCode_column->setChecked( settings.value("showPostalCodeColumn",true).toBool() );
// 	actionShow_PhoneNumber_column->setChecked( settings.value("showPhoneNumberColumn",true).toBool() );
// 	actionShow_Email_column->setChecked( settings.value("showEmailColumn",true).toBool() );
// 	actionShow_Activity_column->setChecked( settings.value("showActivityColumn",true).toBool() );
// 	actionShow_LessonsNumber_column->setChecked( settings.value("showLessonsNumberColumn",true).toBool() );
// 	actionShow_LicenseLabel_column->setChecked( settings.value("showLicenseLabelColumn",true).toBool() );
// 	actionShow_Dues_column->setChecked( settings.value("showDuesColumn",true).toBool() );
// 	actionShow_DuesStatus_column->setChecked( settings.value("showDuesStatusColumn",true).toBool() );
// 	actionShow_SubscriptionDate_column->setChecked( settings.value("showSubscriptionDateColumn",true).toBool() );
// 	actionShow_MedicalCertificateStatus_column->setChecked( settings.value("showMedicalCertificateStatusColumn",true).toBool() );
// 	actionShow_LicenseNumber_column->setChecked( settings.value("showLicenseNumberColumn",true).toBool() );
// 	actionShow_Comment_column->setChecked( settings.value("showCommentColumn",true).toBool() );
// 	actionShow_Picture_column->setChecked( settings.value("showPictureColumn",true).toBool() );
	
	activityComboBox->clear();
	activityComboBox->addItems( associationSettingsWidget->getCoursesList() );
	duesEdit->setSuffix( " "+AssumaCM->getConfigToken( AssumaConfigManager::Currency ).toString() );
}

void AssumaMainWindow::writeSettings(){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> Writting settings";
#endif
	QSettings settings("Infinity Perl","assuma");
	settings.setValue("mw_geometry", saveGeometry());
	settings.setValue("last_database",m_currentDatabase);
	settings.setValue("splitter_geometry", splitter->saveState() );
	QStringList images;
	for( int k=0; k < imagesListWidget->count() ; ++k ){
		QListWidgetItem *it = imagesListWidget->item(k);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::writeSettings() : item -> " << it->text() ;
#endif
		if( QFile::exists( it->text() ) ){
			images << it->text();
		}
	}
	settings.setValue("userDefinedAvatars",images);
	
	// Saving settings from the configuration widget
	settings.setValue("configTrimesterCount", AssumaCM->getConfigToken(AssumaConfigManager::TrimesterCount) );
	settings.setValue("configStaticCost", AssumaCM->getConfigToken(AssumaConfigManager::StaticCost) );
	settings.setValue("configFederation", AssumaCM->getConfigToken(AssumaConfigManager::Federation) );
	settings.setValue("configUpdateServer", AssumaCM->getConfigToken(AssumaConfigManager::UpdateServer) );
	settings.setValue("configUpdatesOnStart", AssumaCM->getConfigToken(AssumaConfigManager::UpdatesOnStart) );
	settings.setValue("configContactsList", AssumaCM->getConfigToken(AssumaConfigManager::ContactsList) );
	settings.setValue("configCoursesList", AssumaCM->getConfigToken(AssumaConfigManager::CoursesList) );
	settings.setValue("configLanguage", AssumaCM->getConfigToken(AssumaConfigManager::Language) );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::writeSettings() -> saved language is :" << AssumaCM->getConfigToken(AssumaConfigManager::Language) ;
#endif
	settings.setValue("configEncoding", AssumaCM->getConfigToken(AssumaConfigManager::Encoding) );
	settings.setValue("configAssociationName", AssumaCM->getConfigToken(AssumaConfigManager::AssociationName) );
	settings.setValue("configEmail", AssumaCM->getConfigToken(AssumaConfigManager::Email) );
	settings.setValue("configSmtpServer", AssumaCM->getConfigToken(AssumaConfigManager::SmtpServer) );
	settings.setValue("configSmtpLogin", AssumaCM->getConfigToken(AssumaConfigManager::SmtpLogin) );
	settings.setValue("configSmtpPassword", AssumaCM->getConfigToken(AssumaConfigManager::SmtpPassword) );
	settings.setValue("configProxyServer", AssumaCM->getConfigToken(AssumaConfigManager::ProxyServer) );
	settings.setValue("configProxyLogin", AssumaCM->getConfigToken(AssumaConfigManager::ProxyLogin) );
	settings.setValue("configProxyPassword", AssumaCM->getConfigToken(AssumaConfigManager::ProxyPassword) );
	settings.setValue("configProxyEnabled", AssumaCM->getConfigToken(AssumaConfigManager::ProxyEnabled) );
	settings.setValue("configDataDir", AssumaCM->getConfigToken( AssumaConfigManager::DataDir ) );
	settings.setValue("configCountry", AssumaCM->getConfigToken(AssumaConfigManager::Country) );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::writeSettings() -> saved country is :" << AssumaCM->getConfigToken(AssumaConfigManager::Country) ;
#endif
	settings.setValue("configActivatedAddonList", AssumaCM->getConfigToken(AssumaConfigManager::ActivatedAddonList) );
	settings.setValue("configCurrenciesList", AssumaCM->getConfigToken(AssumaConfigManager::CurrenciesList) );
	settings.setValue("configCurrency", AssumaCM->getConfigToken(AssumaConfigManager::Currency) );
	settings.setValue("configTheme", AssumaCM->getConfigToken(AssumaConfigManager::Theme) );
	
	// Saving column status (showed or not)
// 	settings.setValue("showFirstNameColumn", actionShow_FirstName_column->isChecked() );
// 	settings.setValue("showMaidenNameColumn", actionShow_MaidenName_column->isChecked() );
// 	settings.setValue("showLastNameColumn", actionShow_LastName_column->isChecked() );
// 	settings.setValue("showBirthDateColumn", actionShow_BirthDate_column->isChecked() );
// 	settings.setValue("showCityColumn", actionShow_City_column->isChecked() );
// 	settings.setValue("showPostalCodeColumn", actionShow_PostalCode_column->isChecked() );
// 	settings.setValue("showPhoneNumberColumn", actionShow_PhoneNumber_column->isChecked() );
// 	settings.setValue("showEmailColumn", actionShow_Email_column->isChecked() );
// 	settings.setValue("showActivityColumn", actionShow_Activity_column->isChecked() );
// 	settings.setValue("showLessonsNumberColumn", actionShow_LessonsNumber_column->isChecked() );
// 	settings.setValue("showLicenseLabelColumn", actionShow_LicenseLabel_column->isChecked() );
// 	settings.setValue("showDuesColumn", actionShow_Dues_column->isChecked() );
// 	settings.setValue("showDuesStatusColumn", actionShow_DuesStatus_column->isChecked() );
// 	settings.setValue("showSubscriptionDateColumn", actionShow_SubscriptionDate_column->isChecked() );
// 	settings.setValue("showMedicalCertificateStatusColumn", actionShow_MedicalCertificateStatus_column->isChecked() );
// 	settings.setValue("showLicenseNumberColumn", actionShow_LicenseNumber_column->isChecked() );
// 	settings.setValue("showCommentColumn", actionShow_Comment_column->isChecked() );
// 	settings.setValue("showPictureColumn", actionShow_Picture_column->isChecked() );
	
	// Save custom fields columns' states (TEST)
	QRegExp rx("actionShow_.+_column");
	foreach( QAction *action, findChildren<QAction *>(rx) ){
		QString key_replaced = action->objectName();
		key_replaced.replace("actionShow_","show");
		key_replaced.replace("_column","Column");
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow -> Setting column " << key_replaced << " visibility to " << action->isChecked();
#endif
		settings.setValue( key_replaced , action->isChecked() );
	}
	
}

void AssumaMainWindow::addAvatar( const QString& fileName){
	QListWidgetItem *it = new QListWidgetItem(imagesListWidget);
	it->setText( fileName );
	it->setIcon( QPixmap(fileName) );
}

void AssumaMainWindow::on_treeWidget_itemChanged( QTreeWidgetItem * item, int column ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::on_treeWidget_itemChanged() : begin (text="<< item->text(column) <<",column="<<column<<",column name="<< treeWidget->headerItem()->text(column) <<")";
#endif
	if(m_dontUpdateSubscribersList)
		return;
	QTreeWidgetItem *head = treeWidget->headerItem();
	if( head->text(column) == tr("LastName") ){
		if( item->text(elementColumn(tr("MaidenName"))).isEmpty() )
			nameLabel->setText( item->text(column) + " " + item->text(elementColumn(tr("FirstName"))) );
		else
			nameLabel->setText( item->text(column) + " " + "(" + item->text(elementColumn(tr("MaidenName"))) + ")" + " " + item->text(elementColumn(tr("FirstName"))) );
		if( lastNameEdit->text() != item->text(column) )
			lastNameEdit->setText(item->text(column));
	}
	else if( head->text(column) == tr("FirstName") ){
		if( item->text(elementColumn(tr("MaidenName"))).isEmpty() )
			nameLabel->setText( item->text(elementColumn(tr("LastName"))) + " " + item->text(column) );
		else
			nameLabel->setText( item->text(elementColumn(tr("LastName"))) + " " + "(" + item->text(elementColumn(tr("MaidenName"))) + ")" + " " + item->text(column) );
		if( firstNameEdit->text() != item->text(column) )
			firstNameEdit->setText(item->text(column));
	}
	else if( head->text(column) == tr("MaidenName") ){
		nameLabel->setText( item->text(elementColumn(tr("LastName"))) + " " + "(" + item->text(column) + ")" + " " + item->text(elementColumn(tr("FirstName"))) );
		if( maidenNameEdit->text() != item->text(column) )
			maidenNameEdit->setText(item->text(column));
	}
	else if( head->text(column) == tr("BirthDate") ){
		if( birthDateEdit->text() != item->text(column) )
			birthDateEdit->setDate( QDate::fromString( item->text( column ) , birthDateEdit->displayFormat() ) );
	}
	else if( head->text(column) == tr("Address") ){
		if( addressEdit->text() != item->text(column) )
			addressEdit->setText( item->text( column ) );
	}
	else if( head->text(column) == tr("City") ){
		if( cityEdit->text().toUpper() != item->text(column).toUpper() )
			cityEdit->setText( item->text( column ).toUpper() );
// 		if( city_idx == -1 ){
			//! \warning this code adds each letter typed in the editor in the list of cities.
// 			qDebug() << "AssumaMainWindow-> inserting new city in cityEdit : " << item->text( elementColumn( tr("City") ));
// 			cityEdit->insertItem(0,item->text( elementColumn( tr("City") )));
// 			cityEdit->setCurrentIndex(0);
// 		}
// 		else
// 			cityEdit->setCurrentIndex(city_idx);
	}
	else if( head->text(column) == tr("PostalCode") ){
		if( postalCodeEdit->text() != item->text(column) )
			postalCodeEdit->setText( item->text( column ).toUpper() );
// 		int pc_idx = postalCodeEdit->findText ( item->text( elementColumn( tr("PostalCode") ) ), Qt::MatchFixedString );
// 		if( pc_idx == -1 ){
// 				postalCodeEdit->insertItem(0,item->text( elementColumn( tr("PostalCode") )));
// 				postalCodeEdit->setCurrentIndex(0);
// 		}
// 		else
// 			postalCodeEdit->setCurrentIndex(pc_idx);
	}
	else if( head->text(column) == tr("Email") ){
		if( emailEdit->text() != item->text(column) )
			emailEdit->setText( item->text( elementColumn( tr("Email") ) ) );
	}
	else if( head->text(column) == tr("PhoneNumber") ){
		if( phoneNumberEdit->text() != item->text(column) )
			phoneNumberEdit->setText( item->text( column ) );
	}
	else if( head->text(column) == tr("Activity") ){
		if( activityComboBox->currentText() != item->text(column) ){
			int activity_index = activityComboBox->findText( item->text( column ), Qt::MatchFixedString );
			if( activity_index > -1 ){
				activityComboBox->setCurrentIndex(activity_index);
				m_treeWidgetDelegate->setCurrentActivity(item->text( column ));
			}
		}
	}
	else if( head->text(column) == tr("LessonsNumber") ){
		if( lessonsNumberEdit->value() != item->text( column ).toInt() ){
			lessonsNumberEdit->setValue( item->text( column ).toInt() );
			lessonsNumberEdit->setMaximum( associationSettingsWidget->getMaxCoursesPerWeek( activityComboBox->currentText() ) );
		}
	}
	else if( head->text(column) == tr("LicenseLabel") ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::on_treeWidget_itemChanged -> item content changed at column" << column << "name:" << head->text(column);
#endif
		if( licenseLabelComboBox->currentText() != item->text( column ) ){
			int ll_idx = licenseLabelComboBox->findText ( item->text( column ), Qt::MatchFixedString );
			if( ll_idx == -1 ){
				licenseLabelComboBox->setCurrentIndex(0);
				item->setText( column, licenseLabelComboBox->currentText() );
			}
			else
				licenseLabelComboBox->setCurrentIndex(ll_idx);
		}
	}
	else if( head->text(column) == tr("Dues") ){
		if( duesEdit->value() != item->text( column ).toDouble() )
			duesEdit->setValue( calculateDue(item) );
	}
	else if( head->text(column) == tr("DuesStatus") ){
		if( duesStatusEdit->currentText() != item->text( column ) ){
			int ds_index = duesStatusEdit->findText( item->text( column ), Qt::MatchFixedString );
			if( ds_index > -1 && ds_index != duesStatusEdit->currentIndex() )
				duesStatusEdit->setCurrentIndex(ds_index);
		}
	}
	else if( head->text(column) == tr("SubscriptionDate") ){
		if( subscriptionDateEdit->text() != item->text(column) )
			subscriptionDateEdit->setDate( QDate::fromString( item->text( column ) , subscriptionDateEdit->displayFormat() ) );
	}
	else if( head->text(column) == tr("MedicalCertificateStatus") ){
		if( medicalStatuEdit->currentText() != item->text( column ) ){
			int mcs_index = medicalStatuEdit->findText( item->text( column ), Qt::MatchFixedString );
			if( mcs_index > -1 && mcs_index != medicalStatuEdit->currentIndex() )
				medicalStatuEdit->setCurrentIndex(mcs_index);
		}
	}
	else if( head->text(column) == tr("LicenseNumber") ){
		if( licenseNumberEdit->text() != item->text( column ) )
			licenseNumberEdit->setText( item->text( column ) );
	}
	else if( head->text(column) == tr("Comment") ){
		if( commentEdit->text() != item->text( column ) )
			commentEdit->setText( item->text( column ) );
	}
	else {
		foreach( AssumaCustomWidgetBase *wdg, m_customWidgets ){
			if( head->text(column) == wdg->labelAsString() && wdg->valueAsString() != item->text( column ) ){
				wdg->setCustomFieldValue( item->text( column ) );
			}
		}
	}
	setItemTooltip(item);
// 	setWindowTitle( QString("Association Subscribers Manager - [%1][*]").arg(m_currentDatabase) );
// 	treeWidget->setWindowModified(true);
	if( ! m_isInitializing ){
		setWindowModified(true);
		textBrowser->setHtml(buildHtmlPreview(item));
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::on_treeWidget_itemChanged() : end (text="<< item->text(column) <<",column="<<column<<",column name="<< treeWidget->headerItem()->text(column) <<")";
#endif
}

void AssumaMainWindow::setItemTooltip(QTreeWidgetItem *item){
	bool temp_m_dontUpdateSubscribersList = m_dontUpdateSubscribersList;
	m_dontUpdateSubscribersList = true;
	QString toolTip;
	QString picture = item->text( elementColumn(tr("Picture")) );
	if( !picture.startsWith(":/") && !QFile::exists(picture) )
		picture = getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "user-default" );
	toolTip.append("<div align=\"center\"><img src=\"" + picture + "\" height=\"128\" width=\"128\" /></div><b>" + item->text( elementColumn(tr("LastName")) ) + " ");
	if( ! item->text( elementColumn(tr("MaidenName")) ).isEmpty() )
		toolTip.append( "(" + item->text( elementColumn(tr("MaidenName")) ) + ") " );
	toolTip.append( item->text( elementColumn(tr("FirstName")) ) + "</b>" );
	
	if( ! item->text( elementColumn(tr("BirthDate")) ).isEmpty() ){
		QDate bday = QDate::fromString( item->text( elementColumn(tr("BirthDate")) ) , "dd/MM/yyyy" );
		QDate today = QDate::currentDate();
		QDate age = today.addYears( - bday.year() );
		age = age.addMonths( - bday.month() );
		age = age.addDays( - bday.day() );
		toolTip.append( "<br>" + QString::number(age.year()) + " " + tr("years old") );
	}
	
	if( !treeWidget->isColumnHidden( elementColumn(tr("DuesStatus")) ) && item->text( elementColumn(tr("DuesStatus")) ) == tr("Not ok") )
		toolTip.append( "<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "messagebox-warning" )+"\"/></td><td valign=\"middle\">" + tr("<b>Dues to be paid</b>") + "</td></tr></table>" ) ;
	if( ! treeWidget->isColumnHidden( elementColumn(tr("MedicalCertificateStatus")) )&& item->text( elementColumn(tr("MedicalCertificateStatus")) ) == tr("Not ok") )
		toolTip.append( "<table><tr><td valign=\"middle\"><img src=\""+getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "messagebox-warning" )+"\"/></td><td valign=\"middle\">" + tr("<b>Medical certificate to provide</b>") + "</td></tr></table>" ) ;
	
	toolTip.append( "<br>" + item->text( elementColumn(tr("Comment")) ) );
	
	// Now set the tooltip for all column
	for (int i = 0; i < treeWidget->columnCount(); i++ )
		item->setToolTip( i, toolTip );
	m_dontUpdateSubscribersList = temp_m_dontUpdateSubscribersList;
}

QString AssumaMainWindow::buildHtmlPreview(QTreeWidgetItem *it){
	QTreeWidgetItem *head = treeWidget->headerItem();
	// Build HTML for the text browser
	//! \todo We should support themable html for the display of a subscriber and produce this ugly html only as a default value
	QString html;
	for( int i=0; i < head->columnCount(); i++ ){
		if( ! treeWidget->isColumnHidden(i) ){
			html.append("<strong>"); html.append(head->text(i)); html.append(":</strong> ");
			html.append(it->text(i));
			if(head->text(i) == tr("Dues"))
				html.append(" "+AssumaCM->getConfigToken(AssumaConfigManager::Currency).toString() );
			html.append("<br/>");
		}
	}
	return html;
}

void AssumaMainWindow::on_treeWidget_currentItemChanged ( QTreeWidgetItem * current, QTreeWidgetItem * previous ){
	if(m_dontUpdateSubscribersList)
		return;
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> entering in on_treeWidget_currentItemChanged ( QTreeWidgetItem * current, QTreeWidgetItem * previous )";
#endif
	if(previous && current && current != previous ){
		if( ! previous->text( elementColumn( tr("City") ) ).isEmpty() && ! previous->text( elementColumn( tr("PostalCode") ) ).isEmpty() ){
			if( !m_pc_map.contains( previous->text( elementColumn( tr("City") ) ) ) && m_pc_map.contains( previous->text( elementColumn( tr("City") ) ).toUpper() ) )
				previous->setText( elementColumn( tr("City") ), previous->text( elementColumn( tr("City") ) ).toUpper() );
			if( !m_pc_map.contains( previous->text( elementColumn( tr("City") ) ) ) ){
				int r = QMessageBox::question ( treeWidget, tr("Add postal code ?"), 
					tr("The city and postal code you entered (%1 - %2) are not in the current list,\nDo you want to add it ?").arg(previous->text( elementColumn( tr("City") ) )).arg(previous->text( elementColumn( tr("PostalCode") ) )), 
					QMessageBox::Yes, 
					QMessageBox::No 
				);
				if( r == QMessageBox::Yes ){
					m_pc_map.insert( previous->text( elementColumn( tr("City") ) ) , previous->text( elementColumn( tr("PostalCode") ) ) );
					m_shouldSavePostalCodes=true;
					if( m_treeWidgetDelegate )
						m_treeWidgetDelegate->setPostCodesMap( m_pc_map );
					QCompleter *completer = new QCompleter(m_pc_map.keys(), this);
					completer->setCaseSensitivity(Qt::CaseInsensitive);
					cityEdit->setCompleter(completer);
					QCompleter *completer_pc = new QCompleter(m_pc_map.values(), this);
					completer_pc->setCaseSensitivity(Qt::CaseInsensitive);
					postalCodeEdit->setCompleter(completer_pc);
				}
			}
			else if( m_pc_map.contains( previous->text( elementColumn( tr("City") ) ) ) && m_pc_map.value( previous->text( elementColumn( tr("City") ) ) ) != previous->text( elementColumn( tr("PostalCode") ) ) ){
				int r = QMessageBox::question ( treeWidget, tr("Modify postal code ?"), 
					tr("The postal code you entered (%2) for <b>%1</b> is not the one stored in our list,\nDo you want to replace <b>%3</b> by <b>%2</b> in the list ?").arg(previous->text( elementColumn( tr("City") ) )).arg(previous->text( elementColumn( tr("PostalCode") ) )).arg( m_pc_map.value( previous->text( elementColumn( tr("City") ) ) ) ), 
					QMessageBox::Yes, 
					QMessageBox::No 
				);
				if( r == QMessageBox::Yes ){
					m_pc_map.insert(previous->text( elementColumn( tr("City") ) ) , previous->text( elementColumn( tr("PostalCode") ) ) );
					m_shouldSavePostalCodes=true;
					if( m_treeWidgetDelegate )
						m_treeWidgetDelegate->setPostCodesMap( m_pc_map );
					QCompleter *completer = new QCompleter(m_pc_map.keys(), this);
					completer->setCaseSensitivity(Qt::CaseInsensitive);
					cityEdit->setCompleter(completer);
					QCompleter *completer_pc = new QCompleter(m_pc_map.values(), this);
					completer_pc->setCaseSensitivity(Qt::CaseInsensitive);
					postalCodeEdit->setCompleter(completer_pc);
				}
			}
		}
	}
	if( current && current != previous ){
// 		QTreeWidgetItem *head = treeWidget->headerItem();
		QString name = current->text( elementColumn(tr("FirstName")) );
		name.append(" ");
		if( ! current->text(elementColumn(tr("MaidenName"))).isEmpty() )
			name.append( "(" + current->text(elementColumn(tr("MaidenName"))) + ") " );
		name.append( current->text( elementColumn(tr("LastName")) ) );
		nameLabel->setText(name);
		// Set picture viewer and editor
		QPixmap pix;
		if( current->text( elementColumn(tr("Picture")) ).startsWith("embedImage:") ){
			QString img_str = current->text( elementColumn(tr("Picture")) );
			img_str.remove("embedImage:");
			QByteArray bytes = QByteArray::fromBase64( img_str.toAscii() );
			pix.loadFromData( bytes );
		}
		else{
			pix = QPixmap( current->text(elementColumn(tr("Picture"))) ).scaled(QSize(128,128), Qt::KeepAspectRatio);
		}
		if(!pix)
			pix = QPixmap(getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "user-default" ));
		pictureEdit->setIcon( QIcon(pix) );
		pictureEdit->setIconSize( pix.size() );
		pictureLabel->setPixmap( pix );
// 		// Build HTML for the text browser
// 		//! \todo We should support themable html for the display of a subscriber and produce this ugly html only as a default value
// 		QString html;
// 		for( int i=0; i < head->columnCount(); i++ ){
// 			if( ! treeWidget->isColumnHidden(i) ){
// 				html.append("<strong>"); html.append(head->text(i)); html.append(":</strong> ");
// 				html.append(current->text(i));
// 				if(head->text(i) == tr("Dues"))
// 					html.append(" "+AssumaCM->getConfigToken(AssumaConfigManager::Currency).toString() );
// 				html.append("<br/>");
// 			}
// 		}
// 		textBrowser->setHtml(html);
		textBrowser->setHtml(buildHtmlPreview(current));
		// Set editors' values
		firstNameEdit->setText( current->text( elementColumn( tr("FirstName") ) ) );
		maidenNameEdit->setText( current->text( elementColumn( tr("MaidenName") ) ) );
		lastNameEdit->setText( current->text( elementColumn( tr("LastName") ) ) );
		birthDateEdit->setDate( QDate::fromString( current->text( elementColumn( tr("BirthDate") ) ) , birthDateEdit->displayFormat() ) );
		subscriptionDateEdit->setDate( QDate::fromString( current->text( elementColumn( tr("SubscriptionDate") ) ) , subscriptionDateEdit->displayFormat() ) );
		
		int activity_index = activityComboBox->findText( current->text( elementColumn( tr("Activity") ) ), Qt::MatchFixedString );
		activityComboBox->setCurrentIndex(activity_index);
		m_treeWidgetDelegate->setCurrentActivity(current->text( elementColumn( tr("Activity") ) ));
		
		lessonsNumberEdit->setValue( current->text( elementColumn( tr("LessonsNumber") ) ).toInt() );
		lessonsNumberEdit->setMaximum( associationSettingsWidget->getMaxCoursesPerWeek( activityComboBox->currentText() ) );
		
		double due = calculateDue(current);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::on_treeWidget_currentItemChanged -> due =" << due;
#endif
		duesEdit->setValue( due );
		
		int ds_index = duesStatusEdit->findText( current->text( elementColumn( tr("DuesStatus") ) ), Qt::MatchFixedString );
		duesStatusEdit->setCurrentIndex(ds_index);
		
		int mcs_index = medicalStatuEdit->findText( current->text( elementColumn( tr("MedicalCertificateStatus") ) ), Qt::MatchFixedString );
		medicalStatuEdit->setCurrentIndex(mcs_index);
		
// 		int city_idx = cityEdit->findText ( current->text( elementColumn( tr("City") ) ), Qt::MatchFixedString );
// // 		qDebug() << "AssumaMainWindow-> cityEdit index found: " << city_idx;
// 		if( city_idx == -1 ){
// 			qDebug() << "AssumaMainWindow -> inserting into cityEdit city name : " << current->text( elementColumn( tr("City") ));
// 			cityEdit->insertItem(0,current->text( elementColumn( tr("City") )));
// 			cityEdit->setCurrentIndex(0);
// 		}
// 		else
// 			cityEdit->setCurrentIndex(city_idx);
		
		QString postal_code = m_pc_map.value( current->text( elementColumn( tr("City") ) ).toUpper() );
		cityEdit->setText( current->text( elementColumn( tr("City") ) ).toUpper() );
		// Set the completer updated with the freshly added new city. (hum... no : this is the role of the change detection code)
		if( ! postal_code.isEmpty() ){
			postalCodeEdit->setText( postal_code );
		}
		int ll_idx = licenseLabelComboBox->findText ( current->text( elementColumn( tr("LicenseLabel") ) ), Qt::MatchFixedString );
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> licenseLabelComboBox index found: " << ll_idx;
#endif
		if( ll_idx == -1 ){
// 				licenseLabelComboBox->insertItem(0,current->text( elementColumn( tr("LicenseLabel") )));
				licenseLabelComboBox->setCurrentIndex(0);
		}
		else
			licenseLabelComboBox->setCurrentIndex(ll_idx);
		
// 		int pc_idx = postalCodeEdit->findText ( current->text( elementColumn( tr("PostalCode") ) ), Qt::MatchFixedString );
// // 		qDebug() << "AssumaMainWindow-> postalCodeEdit index found: " << pc_idx;
// 		if( pc_idx == -1 ){
// 				postalCodeEdit->insertItem(0,current->text( elementColumn( tr("PostalCode") )));
// 				postalCodeEdit->setCurrentIndex(0);
// 		}
// 		else
// 			postalCodeEdit->setCurrentIndex(pc_idx);
		
		addressEdit->setText( current->text( elementColumn( tr("Address") ) ) );
		phoneNumberEdit->setText( current->text( elementColumn( tr("PhoneNumber") ) ) );
		emailEdit->setText( current->text( elementColumn( tr("Email") ) ) );
		licenseNumberEdit->setText( current->text( elementColumn( tr("LicenseNumber") ) ) );
		commentEdit->setText( current->text( elementColumn( tr("Comment") ) ) );
		
		// Updating custom fields
		foreach( AssumaCustomWidgetBase *wdg, m_customWidgets ){
			wdg->setCustomFieldValue( current->text( elementColumn( wdg->labelAsString() ) ) );
		}
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->on_treeWidget_currentItemChanged ( QTreeWidgetItem * current, QTreeWidgetItem * previous ) : work done !";
#endif
}

// Editors modification slots

void AssumaMainWindow::onCustomWidgetValueChanged( const QString & text ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::onCustomWidgetValueChanged => " << text;
#endif
	AssumaCustomWidgetBase *wdg = qobject_cast<AssumaCustomWidgetBase *>( sender() );
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item && wdg){
		item->setText( elementColumn( wdg->labelAsString() ), text );
	}
}

void AssumaMainWindow::on_firstNameEdit_textChanged( const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("FirstName") ), text );
	}
}

void AssumaMainWindow::on_maidenNameEdit_textChanged( const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("MaidenName") ), text );
	}
}

void AssumaMainWindow::on_lastNameEdit_textChanged( const QString & text){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::on_lastNameEdit_textChanged("<< text << ") : begin";
#endif
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("LastName") ), text );
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::on_lastNameEdit_textChanged("<< text << ") : end";
#endif
}

void AssumaMainWindow::on_emailEdit_textChanged( const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("Email") ), text );
	}
}

void AssumaMainWindow::on_phoneNumberEdit_textChanged( const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("PhoneNumber") ), text );
	}
}

void AssumaMainWindow::on_licenseNumberEdit_textChanged( const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("LicenseNumber") ), text );
	}
}


void AssumaMainWindow::on_commentEdit_textChanged( const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("Comment") ), text );
	}
}

void AssumaMainWindow::on_subscriptionDateEdit_dateChanged ( const QDate & date ){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("SubscriptionDate") ), date.toString( subscriptionDateEdit->displayFormat() ) );
	}
}

void AssumaMainWindow::on_birthDateEdit_dateChanged ( const QDate & date ){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("BirthDate") ), date.toString( birthDateEdit->displayFormat() ) );
	}
}

void AssumaMainWindow::on_lessonsNumberEdit_valueChanged(const QString & text){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("LessonsNumber") ), text );
		double debug_tmp = calculateDue(item);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow::on_lessonsNumberEdit_valueChanged -> debug_tmp from calculateDue() =" << debug_tmp;
#endif
		duesEdit->setValue( debug_tmp );
	}
}

void AssumaMainWindow::on_duesEdit_valueChanged(double i){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow-> on_duesEdit_valueChanged(const QString & text): text=" << i;
#endif
		item->setText( elementColumn( tr("Dues") ), QString::number(i) );
	}
}

void AssumaMainWindow::on_addressEdit_textChanged ( const QString & text ){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("Address") ), text );
	}
}

void AssumaMainWindow::on_licenseLabelComboBox_currentIndexChanged ( int index ) {
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> on_licenseLabelComboBox_currentIndexChanged(int): index changed !";
#endif
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow -> on_licenseLabelComboBox_currentIndexChanged(int): changing main view LicenseLabel text and recalculating dues.";
#endif
		item->setText( elementColumn( tr("LicenseLabel") ), licenseLabelComboBox->itemText(index) );
		item->setData(elementColumn( tr("LicenseLabel") ), AssumaMainWindow::IdRole, licenseLabelComboBox->itemData( index,AssumaMainWindow::IdRole ) );
		item->setData(elementColumn( tr("LicenseLabel") ), AssumaMainWindow::FeeRole, licenseLabelComboBox->itemData( index,AssumaMainWindow::FeeRole ) );
		double due = calculateDue(item);
		duesEdit->setValue(due);
	}
}

void AssumaMainWindow::on_cityEdit_textChanged ( const QString & text ) {
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("City") ), text );
	}
	if( m_pc_map.value( cityEdit->text() ) != "" ){
		QString postal_code = m_pc_map.value( cityEdit->text() ) ;
		if( ! postal_code.isEmpty() )
			postalCodeEdit->setText(postal_code);
	}
}

void AssumaMainWindow::on_postalCodeEdit_textChanged ( const QString & text ){
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("PostalCode") ), text );
	}
}

void AssumaMainWindow::on_duesStatusEdit_currentIndexChanged ( int index ) {
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		if( duesStatusEdit->itemText(index) == tr("Ok") )
			item->setText( elementColumn( tr("DuesStatus") ), "1" );
		else
			item->setText( elementColumn( tr("DuesStatus") ), "0" );
		exploitItemData(item);
	}
}

void AssumaMainWindow::on_medicalStatuEdit_currentIndexChanged ( int index ) {
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		if( medicalStatuEdit->itemText(index) == tr("Ok") )
			item->setText( elementColumn( tr("MedicalCertificateStatus") ), "1" );
		else
			item->setText( elementColumn( tr("MedicalCertificateStatus") ), "0" );
		exploitItemData(item);
	}
}

void AssumaMainWindow::on_activityComboBox_currentIndexChanged( int index ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> Entering on_activityComboBox_currenIndexChanged(" << index << ")";
#endif
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow -> Activity column is number : " << elementColumn( tr("Activity") );
#endif
	QTreeWidgetItem *item = treeWidget->currentItem() ;
	if(item){
		item->setText( elementColumn( tr("Activity") ), activityComboBox->itemText(index) );
		duesEdit->setValue( calculateDue(item) );
		lessonsNumberEdit->setMaximum( associationSettingsWidget->getMaxCoursesPerWeek( activityComboBox->currentText() ) );
	}
}

void AssumaMainWindow::on_pictureEdit_toggled(bool isChecked){
	if(isChecked){
		imagesListWidget->show();
		addImagesButton->show();
	}
	else{
		imagesListWidget->hide();
		addImagesButton->hide();
	}
}

void AssumaMainWindow::on_imagesListWidget_itemDoubleClicked ( QListWidgetItem * item ){
	QPixmap pix;
	QTreeWidgetItem *current = treeWidget->currentItem() ;
	if( QFile::exists( item->text() ) ){
		current->setText(elementColumn(tr("Picture")), item->text() );
		current->setIcon( 0, QIcon( item->text() ) );
	}
	else{
		current->setText(elementColumn(tr("Picture")), QString(":/images/images/%1").arg(item->text()) );
		current->setIcon( 0, QIcon( QString(":/images/images/%1").arg(item->text()) ) );
	}
	pix = QPixmap( current->text( elementColumn(tr("Picture")) ) ).scaled(QSize(128,128), Qt::KeepAspectRatio);
	pictureEdit->setIcon( QIcon( pix ) );
	pictureEdit->setIconSize( pix.size() );
}

void AssumaMainWindow::on_deleteSubscriberButton_clicked() {
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow-> deleteSubscriberButton clicked" ;
#endif
	QList<QTreeWidgetItem *> item_list =  treeWidget->selectedItems();
	if( item_list.size() > 0 ){
		int r = QMessageBox::question ( treeWidget, tr("Delete subscribers ?"), 
				tr("You are about to delete %1 subscriber(s),\nDo you want to continue ?").arg(item_list.size()), 
				QMessageBox::Yes, 
				QMessageBox::No 
			);
		if(r == QMessageBox::Yes){
			foreach( QTreeWidgetItem *it, item_list ){
				int index = treeWidget->indexOfTopLevelItem(it);
// 				qDebug() << "Index of element " << it->text( elementColumn( tr("FirstName") ) ) << " " << it->text( elementColumn( tr("LastName") ) ) << " is : " << index;
				treeWidget->takeTopLevelItem(index);
			}
			showPassiveMessage( tr("%1 subscriber(s) have been deleted from this database.<br>To commit this deletion(s) you have to save your database.").arg(item_list.size() ) );
// 			treeWidget->setWindowModified(true);
			if( ! m_isInitializing )
				setWindowModified(true);
			on_treeWidget_itemSelectionChanged();
		}
	}
}

void AssumaMainWindow::addSubscriber( QTreeWidgetItem *item ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->addSubscriber() adding new item : " << item ;
#endif
	treeWidget->addTopLevelItem(item);
// 	treeWidget->setWindowModified(true);
	if( ! m_isInitializing )
		setWindowModified(true);
}

void AssumaMainWindow::subscribersImported(QList<QTreeWidgetItem*>* imported_list){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->subscribersImported(): got a " << imported_list->size() << " long list.";
#endif
	int firstNameColumn = elementColumn(tr("FirstName")) ;
	int lastNameColumn = elementColumn(tr("LastName")) ;
	if( imported_list->isEmpty() )
		return;
	QTreeWidgetItem *importedHead = imported_list->takeFirst();
	for (int i = 0; i < imported_list->size(); ++i){
		QTreeWidgetItem *tmp_item = imported_list->at(i) ;
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow->subscribersImported(): got " << tmp_item->text(0) << " " << tmp_item->text(2);
#endif
		QList<QTreeWidgetItem *> tmp_list = treeWidget->findItems( tmp_item->text( elementColumn(importedHead, tr("FirstName") ) ) , Qt::MatchFixedString , firstNameColumn );
		if( tmp_list.size() > 0 ){
			bool shouldImport = true;
			for( int k = 0; k < tmp_list.size(); ++k ){
				QTreeWidgetItem *it = tmp_list.at(k) ;
				if( it->text( lastNameColumn ) == tmp_item->text( elementColumn(importedHead, tr("LastName") ) ) ) {
					int r = QMessageBox::question ( this, 
						tr("Subscriber already exists"), 
						tr("The subscriber you try to import (<b>%1 %2 born %3</b>)<br>is already in the database as:<br><b>%4 %5 born %6</b>.<br><br>Do you want to import it anyway ?<br><br><i>(Yes will import the new subscriber without removing the one already present, No will discard importation)</i>.").arg(tmp_item->text( elementColumn(importedHead, tr("FirstName") ) ))
							.arg(tmp_item->text( elementColumn(importedHead, tr("LastName") ) ))
							.arg(tmp_item->text( elementColumn(importedHead, tr("BirthDate") ) ))
							.arg(it->text( firstNameColumn ))
							.arg(it->text( lastNameColumn ))
							.arg(it->text( elementColumn(tr("BirthDate")) ))
						,QMessageBox::Yes|QMessageBox::No, QMessageBox::Yes 
					);
					if( r == QMessageBox::No)
						shouldImport=false;
				}
			}
			if( shouldImport ){
				QTreeWidgetItem *newItem = new QTreeWidgetItem ;
				QTreeWidgetItem *treeHead = treeWidget->headerItem();
				for (int f = 0 ; f <= treeHead->columnCount() ; f++ ){
					newItem->setText( f, tmp_item->text( elementColumn( importedHead, treeHead->text(f) ) ) );
				}
				newItem->setIcon(0, QIcon( newItem->text( elementColumn(tr("Picture") ) ) ) );
				newItem->setText( elementColumn(tr("DuesStatus")),"0" );
				newItem->setText( elementColumn(tr("MedicalCertificateStatus")),"0" );
				exploitItemData(newItem);
				setItemTooltip(newItem);
				addSubscriber(newItem);
			}
		}
		else{
			QTreeWidgetItem *newItem = new QTreeWidgetItem ;
			QTreeWidgetItem *treeHead = treeWidget->headerItem();
			for (int f = 0 ; f <= treeHead->columnCount() ; f++ ){
				newItem->setText( f, tmp_item->text( elementColumn( importedHead, treeHead->text(f) ) ) );
			}
			newItem->setIcon(0, QIcon( newItem->text( elementColumn(tr("Picture") ) ) ) );
			newItem->setText( elementColumn(tr("DuesStatus")),"0" );
			newItem->setText( elementColumn(tr("MedicalCertificateStatus")),"0" );
			exploitItemData(newItem);
			setItemTooltip(newItem);
			addSubscriber(newItem);
		}
	}
}

QString AssumaMainWindow::subscribersListToString(const QString& separator, bool addHeaders){
	QString text;
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	if( addHeaders ){
		QTreeWidgetItem *header = treeWidget->headerItem();
		QStringList clipline;
		for ( int j=0 ; j < header->columnCount() ; ++j ){
			clipline << header->text(j);
		}
		text.append( clipline.join(separator) );
		text.append("\n");
	}
	for(int k=0;k<rootitem->childCount(); k++)
	{
		QTreeWidgetItem *it = rootitem->child(k);
		QStringList clipline;
		for ( int j=0 ; j < it->columnCount() ; ++j ){
			clipline << it->text(j);
		}
		text.append( clipline.join(separator) );
		text.append("\n");
	}
	return text;
}

// Action toggle slots

void AssumaMainWindow::on_addImagesButton_clicked(){
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open image..."),m_currentDatabase,tr("Images files (*.png *.jpg *.jpeg *.bmp *.svg *.gif)"));
	if( ! fileName.isEmpty() ){
		addAvatar(fileName);
	}
}

void AssumaMainWindow::on_actionExport_to_Excel_activated(){
	QString file_name = m_currentDatabase ;
	file_name.replace(".adb",".xls");
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->on_actionExport_to_Excel_activated() XLS data should be exported to : " << file_name ;
#endif
	file_name = QFileDialog::getSaveFileName(this, tr("Save File"),file_name,tr("Microsoft Excel files (*.xls)"));
	if( !file_name.isEmpty() ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow->on_actionExport_to_Excel_activated() XLS data will be exported to : " << file_name ;
#endif
		QFile file(file_name);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
				QMessageBox::critical(this,tr("Write error"), tr("We are unable to write the XLS file.\nWe cannot open %1 for writting.").arg(file_name) );
				return;
		}
		QTextStream out(&file);
		//out.setCodec("UTF-8");
		// TODO: we have to set here the codec depending on the configured one.
		out.setCodec("ISO-8859-1");
		out << subscribersListToString("\t",true).toLatin1() << "\n";
		showPassiveMessage( tr("database: <strong>%1</strong><br>was exported to <br>Excel file: <strong>%2</strong>").arg(m_currentDatabase).arg(file_name) );
	}
// 	else
// 		qDebug() << "AssumaMainWindow->on_actionExport_to_Excel_activated() XLS exportation aborded !!" ;
}

void AssumaMainWindow::on_actionExport_to_CSV_activated() {
	QString file_name = m_currentDatabase ;
	file_name.replace(".adb",".csv");
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->on_actionExport_to_CSV_activated() CSV data should be exported to : " << file_name ;
#endif
	file_name = QFileDialog::getSaveFileName(this, tr("Save File"),file_name,tr("Comma Separated files (*.csv)"));
	if( !file_name.isEmpty() ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow->on_actionExport_to_CSV_activated() CSV data will be exported to : " << file_name ;
#endif
		QFile file(file_name);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
				QMessageBox::critical(this,tr("Write error"), tr("We are unable to write the CSV file.\nWe cannot open %1 for writting.").arg(file_name) );
				return;
		}
		QTextStream out(&file);
		out.setCodec("UTF-8");
		out << subscribersListToString(" ; ",true) << "\n";
		showPassiveMessage( tr("database: <strong>%1</strong><br>was exported to <br>CSV file: <strong>%2</strong>").arg(m_currentDatabase).arg(file_name) );
	}
// 	else
// 		qDebug() << "AssumaMainWindow->on_actionExport_to_CSV_activated() CSV exportation aborded !!" ;
}

void AssumaMainWindow::on_actionExport_to_HTML_activated() {
	QString file_name = m_currentDatabase ;
	file_name.replace(".adb",".html");
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow->on_actionExport_to_HTML_activated() HTML data should be exported to : " << file_name ;
#endif
	file_name = QFileDialog::getSaveFileName(this, tr("Save File"),file_name,tr("HTML files (*.htm *.html)"));
	if( !file_name.isEmpty() ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaMainWindow->on_actionExport_to_HTML_activated() HTML data will be exported to : " << file_name ;
#endif
		QFile file(file_name);
		if (!file.open(QIODevice::WriteOnly | QIODevice::Text)){
				QMessageBox::critical(this,tr("Write error"), tr("We are unable to write the HTML file.\nWe cannot open %1 for writting.").arg(file_name) );
				return;
		}
		QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
		QTextStream out(&file);
		out.setCodec("UTF-8");
		out << "<!DOCTYPE html PUBLIC \"-//W3C//DTD XHTML 1.0 Strict//EN\" \"http://www.w3.org/TR/xhtml1/DTD/xhtml1-strict.dtd\">\n";
		out << "<html xmlns=\"http://www.w3.org/1999/xhtml\" xml:lang=\"en\" lang=\"en\">\n";
		out << "<head>\n" ;
		out << "<title>" << AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() << ": " << tr("Subscribers list") << "</title>\n"; 
		out << "<meta http-equiv=\"Content-type\" content=\"text/html; charset=utf-8\" />";
		out << "<style type=\"text/css\"><!--\n";
		out << "table.table { border-top: 1px solid black; border-bottom: 1px solid black; border-left: 1px solid black;}\n";
		out << ".table_cell { border-top: 1px solid black; border-right: 1px solid black; text-align : center; } \n";
		out << "th { border-right: 1px solid black;}\n";
		out << ".table_row_header { background: #80acff;}\n";
		out << ".table_row_odd { background: #a2d7ff;}\n";
		out << ".table_row:hover, .table_row_odd:hover { background: #a7bfd2;}\n";
		out << "#info_box { border: 1px solid black;}\n";
		out << ".box_title { position: relative; width: 100%;  background: #80acff; }\n";
		out << " --></style>";
		out << "</head>\n<body>\n";
		out << "<div id=\"info_box\">\n";
		out << "<div class=\"box_title\"><strong>" << tr("Association: ") << "</strong>" << AssumaCM->getConfigToken(AssumaConfigManager::AssociationName).toString() << "</div>\n";
		out << "<strong>" << tr("Number of subscribers") << ": </strong>" << rootitem->childCount() << "<br/>\n";
		out << "</div><br/>\n";
		out << "<table class=\"table\" cellspacing=\"0\">\n";
		QTreeWidgetItem *header = treeWidget->headerItem();
		out << "<tr class=\"table_row_header\">\n";
		for ( int j=0 ; j < header->columnCount() ; ++j ){
			if(header->text(j) != tr("Picture"))
				out << "<th>" << header->text(j) << "</th>";
		}
		out << "</th>\n";
		bool isOdd = false;
		for(int k=0;k<rootitem->childCount(); k++)
		{
			QTreeWidgetItem *it = rootitem->child(k);
			if(isOdd){
				out << "<tr class=\"table_row_odd\">" ;
				isOdd = false;
			}
			else{
				out << "<tr class=\"table_row\">" ;
				isOdd = true;
			}
			for ( int j=0 ; j < it->columnCount() ; ++j ){
				if(header->text(j) != tr("Picture"))
					out << "<td class=\"table_cell\">" << it->text(j) << "</td>";
			}
			out << "</tr>\n";
		}
		out << "</table>\n</body>\n</html>\n";
		showPassiveMessage( tr("database: <strong>%1</strong><br>was exported to <br>HTML file: <strong>%2</strong>").arg(m_currentDatabase).arg(file_name) );
	}
// 	else
// 		qDebug() << "AssumaMainWindow->on_actionExport_to_HTML_activated() HTML exportation aborded !!" ;
}

void AssumaMainWindow::on_actionAdd_a_subscriber_activated(){
	QTreeWidgetItem *it = new QTreeWidgetItem(QStringList() << tr("New") << tr("") << tr("Subscriber"));
	it->setText( elementColumn( tr("Picture") ), getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "user-default" ) );
	it->setFlags(it->flags() | Qt::ItemIsEditable);
	it->setIcon(0,QIcon(getImageFromTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString(), "user-default" )) );
	addSubscriber(it);
	treeWidget->setCurrentItem(it,0,QItemSelectionModel::Select);
	actionEdit_subscriber->setChecked(true);
	// Quick & dirty hack to reset the form's date (but we'll need a real automatic mechanism)
	birthDateEdit->setDate( QDate(1970,1,1) );
	subscriptionDateEdit->setDate( QDate::currentDate() );
}

void AssumaMainWindow::on_actionAbout_Association_Subscribers_Manager_activated(){
	QString themeName = AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString();
	QWidget * w = new QWidget(this);
	w->resize(300,90);
	QPalette palette;
	QBrush brush(QColor(255, 255, 255, 255));
	brush.setStyle(Qt::SolidPattern);
	palette.setBrush(QPalette::Active, QPalette::Base, brush);
	QBrush brush1(QColor(145, 233, 150, 220));
	brush1.setStyle(Qt::SolidPattern);
	palette.setBrush(QPalette::Active, QPalette::Window, brush1);
	palette.setBrush(QPalette::Inactive, QPalette::Base, brush);
	palette.setBrush(QPalette::Inactive, QPalette::Window, brush1);
	palette.setBrush(QPalette::Disabled, QPalette::Base, brush1);
	palette.setBrush(QPalette::Disabled, QPalette::Window, brush1);
	w->setPalette(palette);
	w->setAutoFillBackground(true);
	
	QPushButton *pushButton = new QPushButton(w);
        pushButton->setMaximumSize(QSize(17, 17));
        QIcon icon;
        icon.addPixmap(QPixmap(getImageFromTheme(themeName,"button-close")), QIcon::Normal, QIcon::Off);
        pushButton->setIcon(icon);
        pushButton->setFlat(true);
	QVBoxLayout *l = new QVBoxLayout ;
	l->addWidget(pushButton);
	l->addStretch();
	
	w->setLayout( new QHBoxLayout );
	QLabel *pix_label = new QLabel(w);
	pix_label->setPixmap( QPixmap(getImageFromTheme(themeName,"assuma-logo")).scaled(80,80) );
	pix_label->setMaximumWidth(90);
	w->layout()->addWidget( pix_label );
	QLabel *text_label = new QLabel(w);
	text_label->setOpenExternalLinks(true);
	text_label->setText( tr("<html><body><strong>Association Subscribers Manager - v%1 - Arnaud Dupuis</strong><br>Brazilian Portuguese translation: Diego Ferreira & Márcio Moraes<br>Swedish translation: Daniel Persson<br>Romanian translation: Lucian Lupescu<br>Documentation French translation: Fabrice Eyraud<br><a href=\"https://sourceforge.net:443/tracker/?group_id=211536\">Bug report/feature request</a><br><a href=\"http://www.associationsubscribersmanager.org\">Official website</a><br>&copy; 2006 - 2009 Infinity Perl</body></html>").arg(ASSUMA_CORE_VERSION) );
	w->layout()->addWidget( text_label );
	((QBoxLayout *)w->layout())->addLayout( l );
	
	if(text_label->sizeHint() != text_label->size()){
		int delta_height = text_label->sizeHint().height() - text_label->size().height();
		int delta_width = text_label->sizeHint().width() - text_label->size().width();
		QSize s = w->size();
		s.setHeight( s.height() + delta_height );
		if( s.height() < 50 )
			s.setHeight(50);
		if( s.height() < (height() / 10) )
			s.setHeight(height() / 10);
		s.setWidth( s.width() + delta_width );
		w->resize(s);
	}
	
	AssumaEffect *effect = new AssumaEffect(0,w,AssumaEffect::SLIDE_FROM_BOTTOM);
	effect->setTimeBeforeReversing(30000);
	connect( pushButton, SIGNAL(clicked()), effect, SLOT( reverseEffect() ) );
	effect->doEffect();
	
// 	showPassiveMessage(tr("<strong>Association Subscribers Manager - v3.0rc1 - Arnaud Dupuis</strong><br>Brazilian Portuguese translation: Diego Ferreira<br>Swedish translation: Daniel Persson<br>Romanian translation: Lucian Lupescu<br>&copy; 2006 - 2009 Infinity Perl"), QPixmap(":/images/images/as.png") );
}

void AssumaMainWindow::on_actionAssociation_Subscribers_Manager_manual_activated(){
	AssumaHelpBrowser *helpBrowser = new AssumaHelpBrowser(this);
	helpBrowser->loadTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() );
	if( AssumaCM->getConfigToken(AssumaConfigManager::Language).toString().startsWith("Fra") )
		helpBrowser->setUrl( QUrl( QString(ASSUMA_DOCDIR).append("/user/assuma_subscribers_database_doc_fr.html") ) );
	else
		helpBrowser->setUrl( QUrl( QString(ASSUMA_DOCDIR).append("/user/assuma_subscribers_database_doc.html") ) );
	helpBrowser->show();
}

void AssumaMainWindow::on_action_Copy_activated(){
	QList<QTreeWidgetItem *> item_list =  treeWidget->selectedItems();
	QClipboard *clipboard = QApplication::clipboard();
	QStringList clip;
	for (int i = 0; i < item_list.size(); ++i){
		QTreeWidgetItem *it = item_list.at(i);
		QStringList clipline;
		for ( int j=0 ; j < it->columnCount() ; ++j ){
			clipline << it->text(j);
		}
		clip << clipline.join("\t");
	}
	clipboard->setText( clip.join("\n") );
}

void AssumaMainWindow::on_actionPaste_activated(){
	QClipboard *clipboard = QApplication::clipboard();
	QString originalText = clipboard->text();
	foreach (QString text, originalText.split("\n") ){
		QStringList strlst = text.split( QRegExp("(\t|,|;)") );
		QTreeWidgetItem *it = new QTreeWidgetItem(strlst);
		addSubscriber(it);
		treeWidget->setCurrentItem(it,0,QItemSelectionModel::Select);
		actionEdit_subscriber->setChecked(true);
		exploitItemData(it);
	}
}

void AssumaMainWindow::setColumnVisibility( bool isVisible ){
	QAction *tmpAction = qobject_cast<QAction *>( sender() );
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaMainWindow::setColumnVisibility => isVisible = " << isVisible << " tmpAction's text = " << tmpAction->text() << "tmpAction's data = " << tmpAction->data() ;
#endif
	if( isVisible )
		treeWidget->showColumn( elementColumn( tmpAction->data().toString() ) );
	else
		treeWidget->hideColumn( elementColumn( tmpAction->data().toString() ) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_FirstName_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("FirstName")) );
	else
		treeWidget->hideColumn( elementColumn(tr("FirstName")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_MaidenName_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("MaidenName")) );
	else
		treeWidget->hideColumn( elementColumn(tr("MaidenName")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_LastName_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("LastName")) );
	else
		treeWidget->hideColumn( elementColumn(tr("LastName")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_BirthDate_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("BirthDate")) );
	else
		treeWidget->hideColumn( elementColumn(tr("BirthDate")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_Address_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("Address")) );
	else
		treeWidget->hideColumn( elementColumn(tr("Address")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_City_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("City")) );
	else
		treeWidget->hideColumn( elementColumn(tr("City")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_PostalCode_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("PostalCode")) );
	else
		treeWidget->hideColumn( elementColumn(tr("PostalCode")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_PhoneNumber_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("PhoneNumber")) );
	else
		treeWidget->hideColumn( elementColumn(tr("PhoneNumber")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_MobilePhone_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("MobilePhone")) );
	else
		treeWidget->hideColumn( elementColumn(tr("MobilePhone")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_Email_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("Email")) );
	else
		treeWidget->hideColumn( elementColumn(tr("Email")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_LessonsNumber_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("LessonsNumber")) );
	else
		treeWidget->hideColumn( elementColumn(tr("LessonsNumber")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_LicenseLabel_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("LicenseLabel")) );
	else
		treeWidget->hideColumn( elementColumn(tr("LicenseLabel")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_Activity_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("Activity")) );
	else
		treeWidget->hideColumn( elementColumn(tr("Activity")) );
}

void AssumaMainWindow::on_actionShow_Dues_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("Dues")) );
	else
		treeWidget->hideColumn( elementColumn(tr("Dues")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_DuesStatus_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("DuesStatus")) );
	else
		treeWidget->hideColumn( elementColumn(tr("DuesStatus")) );
	updateAllItemsTooltips();
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_SubscriptionDate_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("SubscriptionDate")) );
	else
		treeWidget->hideColumn( elementColumn(tr("SubscriptionDate")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_MedicalCertificateStatus_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("MedicalCertificateStatus")) );
	else
		treeWidget->hideColumn( elementColumn(tr("MedicalCertificateStatus")) );
	updateAllItemsTooltips();
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_LicenseNumber_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("LicenseNumber")) );
	else
		treeWidget->hideColumn( elementColumn(tr("LicenseNumber")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_Comment_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("Comment")) );
	else
		treeWidget->hideColumn( elementColumn(tr("Comment")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionShow_Picture_column_toggled (bool isChecked) {
	if(isChecked)
		treeWidget->showColumn( elementColumn(tr("Picture")) );
	else
		treeWidget->hideColumn( elementColumn(tr("Picture")) );
	if(searchWidget)
		searchWidget->updateFieldSearchMenu();
}

void AssumaMainWindow::on_actionEdit_subscriber_toggled(bool isChecked){
	if( isChecked && configWidget->isVisible() ){
		actionSettings->setChecked(false);
	}
}

void AssumaMainWindow::on_actionSettings_toggled(bool isChecked){
	if( isChecked ){
		if( actionEdit_subscriber->isChecked() )
			actionEdit_subscriber->setChecked(false);
		if( actionAssociation_settings->isChecked() )
			actionAssociation_settings->setChecked(false);
		splitter->hide();
		associationSettingsWidget->hide();
		configWidget->show();
	}
	else{
		duesEdit->setSuffix( " "+AssumaCM->getConfigToken( AssumaConfigManager::Currency ).toString() );
		configWidget->hide();
		splitter->show();
		if( !action_Find->isChecked() )
			searchWidget->hide();
		treeWidget->setCurrentItem(0);
		activityComboBox->clear();
		activityComboBox->addItems( associationSettingsWidget->getCoursesList() );
		QString _pc_file = AssumaCM->getConfigToken( AssumaConfigManager::DataDir ).toString() + "/PostalCodes/" + AssumaCM->getConfigToken(AssumaConfigManager::Country).toString() + ".cities.xml" ;
		if ( QFile::exists( _pc_file+".qz" ) )
			_pc_file = _pc_file+".qz";
		if( _pc_file != m_postal_codes_filename ){
			m_postal_codes_filename = _pc_file;
			loadPostalCodes( m_postal_codes_filename );
		}
		
		if( AssumaCM->getConfigToken(AssumaConfigManager::Federation) != m_currentFederation && treeWidget->topLevelItemCount() > 0 ){
			loadFederation( AssumaCM->getConfigToken(AssumaConfigManager::Federation).toString() );
			int ret = QMessageBox::question(this,tr("Association Subscribers Manager"),tr("The federation have changed, it is recommended to reload the subscribers' database.\nDo you want to reload it now (the database will be saved before reload) ?"),QMessageBox::Yes|QMessageBox::No);
			if( ret == QMessageBox::Yes){
				if( QFile::exists(m_currentDatabase) ){
					on_action_Save_activated();
					loadDatabase(m_currentDatabase);
				}
				else{
					showPassiveMessage(tr("<html>Database not reloaded.<br>File <strong>%1</strong> does not exist.<html>").arg(m_currentDatabase));
				}
			}
		}
		
		// Test
// 		qDebug() << "TrimesterCount: " << AssumaCM->getConfigToken(AssumaConfigManager::TrimesterCount);
// 		qDebug() << "UpdatesOnStart: " << AssumaCM->getConfigToken(AssumaConfigManager::UpdatesOnStart);
// 		qDebug() << "ContactsList: " << AssumaCM->getConfigToken(AssumaConfigManager::ContactsList);
// 		qDebug() << "CoursesList: " << AssumaCM->getConfigToken(AssumaConfigManager::CoursesList);
// 		qDebug() << "All courses price for 1 course : " << associationSettingsWidget->getCourseFee("All courses", 1);
	}
}

void AssumaMainWindow::on_actionUpdate_activated(){
#ifdef ASSUMA_DEBUG
	qDebug() << "Trying to update !!";
#endif
#ifdef ASSUMA_WITH_UPDATER
	AssumaUpdateDialog *dial = new AssumaUpdateDialog(this);
	dial->loadTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() );
	connect( dial, SIGNAL(updateFinished(bool)), this, SLOT( finishUpdate(bool) ) );
	this->setVisible(false);
	dial->show();
#endif
}

void AssumaMainWindow::finishUpdate(bool haveBeenUpdated){
	this->setVisible(true);
	if(haveBeenUpdated)
		showPassiveMessage( tr("Association Subscribers Manager have been updated, you should restart it now.") );
}

void AssumaMainWindow::on_sendSummaryButton_clicked(){
	AssumaMailDialog *dial = createMailDialog();
	QTreeWidgetItem *item = treeWidget->currentItem();
	if( item->text( elementColumn( tr("Email") ) ).size() > 5 )
			dial->addEmail( item->text( elementColumn( tr("Email") ) ) );
	QString body(tr("Hello %SUBSCRIBER_FIRSTNAME%,\n\nPlease find bellow your dues details:\nActivity: %SUBSCRIBER_ACTIVITY%\nLessons' number: %SUBSCRIBER_LESSONSNUMBER%\nDues: %SUBSCRIBER_DUES% %CURRENCY%\nDues status: %SUBSCRIBER_DUESSTATUS%\nMedical certificate status: %SUBSCRIBER_MEDICALCERTIFICATESTATUS%\n\nBest regards,\n%ASSO_NAME%"));
	dial->setSubject( processTemplate( tr("[%ASSO_NAME%] Dues details") ) );
	dial->setBody( processTemplate(body) );
	dial->show();
}

void AssumaMainWindow::on_actionGo_to_YouTube_video_tutorials_channel_activated(){
	AssumaHelpBrowser *helpBrowser = new AssumaHelpBrowser(this);
	helpBrowser->loadTheme( AssumaCM->getConfigToken( AssumaConfigManager::Theme ).toString() );
// 	helpBrowser->setUrl( QUrl( "http://www.youtube.com/groups_new?name=associationmanager&hl=en&gl=EN&gl=US" ) );
	helpBrowser->setUrl( QUrl( "http://www.youtube.com/group/associationmanager" ) );
	helpBrowser->show();
}

void AssumaMainWindow::updateAllItemsTooltips(){
	QTreeWidgetItemIterator it(treeWidget);
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
#ifdef ASSUMA_DEBUG
	qDebug() << "rootitem->childCount() : " << rootitem->childCount() << " treeWidget->topLevelItemCount() : " << treeWidget->topLevelItemCount();
#endif
	QProgressDialog prg( tr("Please wait during subscribers list update..."), tr("Cancel"), 0, rootitem->childCount(), this );
	
	for(int i = 0; i < treeWidget->topLevelItemCount(); ++i){
		if(prg.wasCanceled())
			break;
		QTreeWidgetItem *item = treeWidget->topLevelItem(i);
		if( item ){
			qDebug() << "AssumaMainWindow::updateAllItemsTooltips() : Setting tooltip for " << item->text( elementColumn( tr("FirstName") ) ) << " " << item->text( elementColumn( tr("LastName") ) );
			setItemTooltip(item);
		}
		prg.setValue( prg.value() + 1 );
	}
	
// 	while (*it) {
// 		if(prg.wasCanceled())
// 			break;
// 		setItemTooltip(*it);
// 		++it;
// 		prg.setValue( prg.value() + 1 );
// 	}
}

CustomFieldItem * AssumaMainWindow::getCustomFieldItemByName(const QString &p_cfName){
	foreach(CustomFieldItem *item, m_customFields){
		if( item->label() == p_cfName )
			return item;
	}
	return 0;
}

// eof
