
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_passivepopup.h"

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QMouseEvent>
#include "assuma_mainwindow.h"

AssumaPassivePopup::AssumaPassivePopup(QWidget *parent) : QWidget(parent){
	setupUi(this);
	connect( pushButton, SIGNAL( clicked() ) , this, SIGNAL( closeButtonClicked() ) );
	text_label->setScaledContents( true );
#ifdef ASSUMA_DEBUG
	qDebug() << "++++++ sizeHint() : " << sizeHint() << " and size()" << size();
#endif
	if( AssumaMW ){
		pushButton->setMaximumSize(QSize(17, 17));
		QIcon icon;
		icon.addFile( AssumaMW->getImageFromTheme("button-close"), QSize(), QIcon::Normal, QIcon::Off);
		pushButton->setIcon(icon);
		pushButton->setFlat(true);
		setPixmap( Info );
	}
	setVisible(false);
	if(parent)
		this->setMaximumWidth( parent->width() );
}

QString AssumaPassivePopup::text(){
	return text_label->text();
}

int AssumaPassivePopup::timeToShow(){
	int minShownDuration = text().size()*150;
	if( minShownDuration < 3000 )
		minShownDuration = 3000;
	return minShownDuration;
}

void AssumaPassivePopup::setPixmap( const QPixmap & pix ){
	pix_label->setPixmap(pix);
}

void AssumaPassivePopup::setCloseButtonPixmap( const QPixmap & pix ){
	QIcon icon;
	icon.addPixmap(pix, QIcon::Normal, QIcon::Off);;
	pushButton->setIcon(icon);
	pushButton->setFlat(true);
}

void AssumaPassivePopup::setPixmap( Icon ico){
	QPixmap pix;
	if(ico == AssumaPassivePopup::Info){
		pix = QPixmap( AssumaMW->getImageFromTheme("messagebox-info") );
	}
	if(ico == AssumaPassivePopup::Critical){
		pix = QPixmap( AssumaMW->getImageFromTheme("messagebox-critical") );
	}
	if(ico == AssumaPassivePopup::Warning){
		pix = QPixmap( AssumaMW->getImageFromTheme("messagebox-warning") );
	}
	pix_label->setPixmap(pix);
}

void AssumaPassivePopup::setText( const QString & str){
	text_label->setText(str);
	updateDimensions();
}


void AssumaPassivePopup::updateDimensions(){
	if(text_label->sizeHint() != text_label->size()){
		int delta_height = text_label->sizeHint().height() - text_label->size().height();
		int delta_width = text_label->sizeHint().width() - text_label->size().width();
		QSize s = size();
		s.setHeight( s.height() + delta_height );
		if( s.height() < 50 )
			s.setHeight(50);
		if( s.height() < (this->parentWidget()->height() / 10) )
			s.setHeight(this->parentWidget()->height() / 10);
		s.setWidth( s.width() + delta_width );
		resize(s);
	}
}

void AssumaPassivePopup::on_pushButton_clicked(){
	pushButton->setEnabled(false);
}

