
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_searchwidget.h"
#include "assuma_mainwindow.h"

AssumaSearchWidget::AssumaSearchWidget(QWidget *parent, QTreeWidget *tw) : QWidget(parent){
	setupUi(this);
	_treeWidget = tw;
	setFocusProxy(searchQuery);
	updateFieldSearchMenu();
}

void AssumaSearchWidget::loadTheme( const QString & themeName ){
	if( ! AssumaMW )
		return;
	QIcon icon;
	icon.addFile(AssumaMW->getImageFromTheme(themeName,"button-erase"), QSize(), QIcon::Normal, QIcon::Off);
	clearSearchButton->setIcon(icon);
	QIcon icon1;
	icon1.addFile(AssumaMW->getImageFromTheme(themeName,"search-user"), QSize(), QIcon::Normal, QIcon::Off);
	doSearchButton->setIcon(icon1);
	rightCornerPixmap->setPixmap(QPixmap(AssumaMW->getImageFromTheme(themeName,"search-corner")));
}

void AssumaSearchWidget::populateMenu( QMenu *menu ){
	if(_treeWidget){
		QTreeWidgetItem *header = _treeWidget->headerItem();
		for (int i = 0 ; i <= header->columnCount() ; i++ ){
			if( ! header->text(i).isEmpty() ){
				QAction *action = new QAction( header->text(i) , this);
				action->setCheckable(true);
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaSearchWidget::populateMenu() : Column " << i << " (" << header->text(i) << ") isHidden: " << _treeWidget->isColumnHidden(i);
#endif
				if( _treeWidget->isColumnHidden(i) ){
					action->setChecked(false);
					if( hideFieldsCheckBox->isChecked() )
						action->setVisible(false);
					else
						action->setVisible(true);
				}
				else
					action->setChecked(true);
				action->setToolTip( tr("Check to search in the \"%1\" field").arg( header->text(i) ) );
				// Connect all actions to updates slots
                                connect( action, SIGNAL( toggled(bool) ), this, SLOT( updateSearch() ) );
				menu->addAction(action);
			}
		}
	}
}

void AssumaSearchWidget::updateFieldSearchMenu(){
	QMenu *menu = new QMenu(fieldsToolButton);
	populateMenu(menu);
        fieldsToolButton->setMenu( menu );
}

QSize AssumaSearchWidget::sizeHint () const{
        return QSize(430,170);
}

void AssumaSearchWidget::setTreeWidget(QTreeWidget *tw){
	_treeWidget = tw;
	updateFieldSearchMenu();
}

QTreeWidget* AssumaSearchWidget::treeWidget(){
	return _treeWidget;
}

void AssumaSearchWidget::setCornerPixmap(const QPixmap & pix){
	rightCornerPixmap->setPixmap(pix);
}

QPixmap AssumaSearchWidget::cornerPixmap() {
	return rightCornerPixmap->pixmap()->copy();
}

void AssumaSearchWidget::on_liveSearchBox_stateChanged( int state ){
	if( state == Qt::Checked){
		doSearchButton->setEnabled(false);
		on_searchQuery_textChanged( searchQuery->text() );
	}
	else
		doSearchButton->setEnabled(true);
}

void AssumaSearchWidget::on_searchQuery_textChanged(const QString & query){
	if( liveSearchBox->isChecked() && query.length() >= 3 ){
		performSearch(query);
	}
	else if(query == ""){
		QTreeWidgetItem *rootitem = _treeWidget->invisibleRootItem();
		for(int k=0;k<rootitem->childCount(); k++)
		{
			rootitem->child(k)->setHidden(false);
		}
	}
}

void AssumaSearchWidget::performSearch(const QString & query){
	if( _treeWidget ){
		bool shouldSearch = false;
		Qt::CaseSensitivity caseSensitivity;
		if( caseSensitiveBox->isChecked() )
			caseSensitivity = Qt::CaseSensitive;
		else
			caseSensitivity = Qt::CaseInsensitive;
		QList<int> colList;
		foreach( QAction *act, fieldsToolButton->menu()->actions() ){
			if( act->isChecked() ){
				colList << elementColumn( act->text() );
			}
		}
		shouldSearch = !colList.isEmpty();
		if(shouldSearch){
			QTreeWidgetItem *rootitem = _treeWidget->invisibleRootItem();
			for(int k=0;k<rootitem->childCount(); k++)
			{
				QTreeWidgetItem *it = rootitem->child(k);
				bool shouldHide=true;
				foreach (int i, colList ){
					if( it->text(i).contains( query, caseSensitivity ) ){
						shouldHide=false;
					}
				}
				it->setHidden( shouldHide );
			}
		}
	}
}

void AssumaSearchWidget::updateSearch(){
	on_searchQuery_textChanged( searchQuery->text() );
}

void AssumaSearchWidget::setColumnMap(const QMap<QString, int> map){
	columnMap = map;
}

int AssumaSearchWidget::elementColumn(const QString & t){
	if( _treeWidget ){
		QTreeWidgetItem *header = _treeWidget->headerItem();
		for (int i = 0 ; i <= header->columnCount() ; i++ ){
			if( t == header->text(i) )
				return i;
		}
	}
	return -1;
}

void AssumaSearchWidget::on_caseSensitiveBox_stateChanged ( int /*state*/ ){
	on_searchQuery_textChanged( searchQuery->text() );
}

void AssumaSearchWidget::on_clearSearchButton_clicked(bool){
	searchQuery->setText("");
}

void AssumaSearchWidget::on_doSearchButton_clicked(bool){
	performSearch(searchQuery->text());
}

void AssumaSearchWidget::on_hideFieldsCheckBox_stateChanged ( int ){
	foreach( QAction *act, fieldsToolButton->menu()->actions() ){
		if( hideFieldsCheckBox->isChecked() ){
			if( act->isChecked() )
				act->setVisible(true);
			else
				act->setVisible(false);
		}
		else
			act->setVisible(true);
	}
}
