
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_subsciberimporterdialog.h"
#include <QFileDialog>
#include <QMessageBox>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QProgressDialog>
#include <QDate>
#include "adbhandler.h"
#include "oldxmlhandler.h"
#include "assuma_mainwindow.h"

AssumaSubscriberImporterDialog::AssumaSubscriberImporterDialog(QWidget *parent) : QDialog(parent){
	setupUi(this);
	QTreeWidgetItem *header = treeWidget->headerItem();
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		columnMap.insert( header->text(i) , i );
	}
	assumasearchwidget->setColumnMap( columnMap );
	assumasearchwidget->setTreeWidget(treeWidget);
}

void AssumaSubscriberImporterDialog::loadTheme(const QString & themeName){
	if( ! AssumaMW )
		return;
	QIcon icon;
	icon.addFile( AssumaMW->getImageFromTheme(themeName,"database-import-big") , QSize(), QIcon::Normal, QIcon::Off);
	setWindowIcon(icon);
	QIcon icon1;
	icon1.addFile( AssumaMW->getImageFromTheme(themeName,"database-import") , QSize(), QIcon::Normal, QIcon::Off);
	openDatabaseButton->setIcon(icon1);
	openDatabaseButton->setIconSize(QSize(22, 22));
	QIcon icon2;
	icon2.addFile( AssumaMW->getImageFromTheme(themeName,"add-user") , QSize(), QIcon::Normal, QIcon::Off);
	importButton->setIcon(icon2);
	importButton->setIconSize(QSize(22, 22));
	QIcon icon3;
	icon3.addFile( AssumaMW->getImageFromTheme(themeName,"button-close") , QSize(), QIcon::Normal, QIcon::Off);
	closeButton->setIcon(icon3);
	closeButton->setIconSize(QSize(22, 22));
	assumasearchwidget->loadTheme(themeName);
}

void AssumaSubscriberImporterDialog::setItemTooltip(QTreeWidgetItem *item){
	QString toolTip;
	toolTip.append("<div align=\"center\"><img src=\"" + item->text( elementColumn(tr("Picture")) ) + "\" height=\"128\" width=\"128\" /></div><b>" + item->text( elementColumn(tr("LastName")) ) + " ");
	if( ! item->text( elementColumn(tr("MaidenName")) ).isEmpty() )
		toolTip.append( "(" + item->text( elementColumn(tr("MaidenName")) ) + ") " );
	toolTip.append( item->text( elementColumn(tr("FirstName")) ) + "</b>" );
	
	if( ! item->text( elementColumn(tr("BirthDate")) ).isEmpty() ){
		QDate bday = QDate::fromString( item->text( elementColumn(tr("BirthDate")) ) , "dd/MM/yyyy" );
		QDate today = QDate::currentDate();
		QDate age = today.addYears( - bday.year() );
		age = age.addMonths( - bday.month() );
		age = age.addDays( - bday.day() );
		toolTip.append( "<br>" + QString::number(age.year()) + " " + tr("years old") );
	}
	
	if( item->text( elementColumn(tr("DuesStatus")) ) == tr("Not ok") )
		toolTip.append( "<table><tr><td valign=\"middle\"><img src=\":/images/images/messagebox_warning.png\"/></td><td valign=\"middle\">" + tr("<b>Dues to be paid</b>") + "</td></tr></table>" ) ;
	if( item->text( elementColumn(tr("MedicalCertificateStatus")) ) == tr("Not ok") )
		toolTip.append( "<table><tr><td valign=\"middle\"><img src=\":/images/images/messagebox_warning.png\"/></td><td valign=\"middle\">" + tr("<b>Medical certificate to provide</b>") + "</td></tr></table>" ) ;
	
	toolTip.append( "<br>" + item->text( elementColumn(tr("Comment")) ) );
	
	// Now set the tooltip for all column
	for (int i = 0; i < treeWidget->columnCount(); i++ )
		item->setToolTip( i, toolTip );
}

void AssumaSubscriberImporterDialog::on_openDatabaseButton_clicked(){
	QString fileName = QFileDialog::getOpenFileName(this, tr("Open File"),".",tr("ADB database (*.adb);;Old XML database (*.xml)"));
	if( fileName.size() > 0 ){
// 		AssumaTextInfoDialog *infodialog = new AssumaTextInfoDialog(this);
// 		infodialog->setText( tr("Please wait while loading subscibers database...") );
// 		infodialog->setTitle( tr("Please wait while loading subscibers database...") );
// 		infodialog->show(); infodialog->raise(); infodialog->update();
// 		update();
		// If the QTreeWidget is not empty clear it
		if( treeWidget->topLevelItemCount() > 0 )
			treeWidget->clear();
		// Loading XML database
		loadDatabase(fileName);
// 		infodialog->hide();
	}
}

void AssumaSubscriberImporterDialog::loadDatabase(const QString& fileName){
	
	QProgressDialog progress(tr("Please wait while loading subscibers database..."), "Abort loading", 0, 3, this);
	progress.setWindowModality(Qt::WindowModal);
	progress.setWindowTitle(tr("Loading database..."));
	progress.setMinimumDuration(0);
	progress.setCancelButton( 0 );
	progress.setValue(0);
	this->update();
	QCoreApplication::processEvents(QEventLoop::AllEvents);
	if(fileName.endsWith(".adb")){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> Openning " << fileName;
#endif
// 		infodialog->setText( tr("Opening database: %1").arg(fileName) );
		progress.setValue(1);
		progress.setLabelText(tr("Opening database: %1").arg(fileName));
		QCoreApplication::processEvents(QEventLoop::AllEvents);
		QFile file(fileName);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> Creating QXmlInputSource";
#endif
		QXmlInputSource inputSource(&file);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> Creating QXmlSimpleReader";
#endif
		QXmlSimpleReader reader;
		AdbHandler handler(treeWidget);
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
		progress.setValue(2);
		progress.update();
		QCoreApplication::processEvents(QEventLoop::AllEvents);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> parsing inputSource";
#endif
// 		infodialog->setText( tr("Processing database...") );
		progress.setLabelText(tr("Processing database..."));
		reader.parse(inputSource);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> inputSource parsed";
#endif
		progress.setValue(3);
		progress.setLabelText(tr("Processing database...done !"));
		QCoreApplication::processEvents(QEventLoop::AllEvents);
// 		infodialog->setText( tr("Processing database...done !") );
	}
	else if(fileName.endsWith(".xml")){
		QMessageBox::warning(this,tr("Association Subscribers Manager"),tr("You are loading an old version of the Association Subscriber Manager database,\nyour subscribers will be imported in the current view\nand you will have to save this database in the new format.\nThe new database files ends with \".adb\" (stands for Association subscribers manager DataBase)."));
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> Openning " << fileName;
#endif
// 		infodialog->setText( tr("Opening database: %1").arg(fileName) );
		QFile file(fileName);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> Creating QXmlInputSource";
#endif
		QXmlInputSource inputSource(&file);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> Creating QXmlSimpleReader";
#endif
		QXmlSimpleReader reader;
		OldXmlHandler handler(treeWidget);
		reader.setContentHandler(&handler);
		reader.setErrorHandler(&handler);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> parsing inputSource";
#endif
// 		infodialog->setText( tr("Processing database...") );
		reader.parse(inputSource);
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaSubscriberImporterDialog-> inputSource parsed";
#endif
// 		infodialog->setText( tr("Processing database...done !") );
	}
	setWindowTitle( QString("Import Subscribers - [%1]").arg(fileName) );
	treeWidget->sortItems( 0, Qt::AscendingOrder );
	for( int i = 0; i < treeWidget->headerItem()->columnCount(); i++){
		treeWidget->resizeColumnToContents(i);
	}
	
	QTreeWidgetItem *rootitem = treeWidget->invisibleRootItem();
	// storing those variables here will considerably speed up the application and reduce memory usage
	int dsc = elementColumn(tr("DuesStatus"));
	int mcsc = elementColumn(tr("MedicalCertificateStatus"));
	//int pc = elementColumn("Picture");
	QIcon ok_icon = QIcon(":/images/images/button_ok.png");
	QIcon nok_icon = QIcon(":/images/images/remove.png");
	
	for(int k=0;k<rootitem->childCount(); k++){
		QTreeWidgetItem *it = rootitem->child(k);
		// Setting all items editable
		it->setFlags(it->flags() | Qt::ItemIsEditable);
		QPixmap pix = QPixmap( it->text(elementColumn(tr("Picture"))) );
		if(!pix)
			pix = QPixmap(":/images/images/user_default.png");
		it->setIcon( 0, QIcon(pix) );
		// Setting text and icon for DuesStatus column
		if( it->text(dsc) == "1" ){
			it->setIcon(dsc, ok_icon );
			it->setText(dsc, tr("Ok") );
		}
		else{
			it->setIcon(dsc, nok_icon );
			it->setText(dsc, tr("Not ok") );
		}
		// Setting text and icon for MedicalCertificateStatus column
		if( it->text(mcsc) == "1" ){
			it->setIcon(mcsc, ok_icon );
			it->setText(mcsc, tr("Ok") );
		}
		else{
			it->setIcon(mcsc, nok_icon );
			it->setText(mcsc, tr("Not ok") );
		}
		setItemTooltip(it);
	}
	
}

int AssumaSubscriberImporterDialog::elementColumn(const QString& columnName){
	QTreeWidgetItem *header = treeWidget->headerItem();
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		if( columnName == header->text(i) )
			return i;
	}
	return -1;
}

void AssumaSubscriberImporterDialog::on_importButton_clicked() {
	QList<QTreeWidgetItem *> item_list =  treeWidget->selectedItems();
	QList<QTreeWidgetItem *> *rl = new QList<QTreeWidgetItem *>() ;
	rl->append( treeWidget->headerItem() );
	for( int i = 0 ; i < item_list.size(); ++i ){
		rl->append( treeWidget->takeTopLevelItem( treeWidget->indexOfTopLevelItem( item_list.at(i) ) ) );
	}
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaSubscriberImporterDialog->on_importButton_clicked() sending a importSubscribers() signal with a "<< rl->size() << " long list.";
#endif
	emit( importSubscribers( rl ) );
}
