
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include <QtGui>
#include "assuma_treewidgetitemeditordelegate.h"
#include "assuma_mainwindow.h"
#include "customfielditem.h"
#include "assuma_customwidgets.h"

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

AssumaTreeWidgetItemEditorDelegate::AssumaTreeWidgetItemEditorDelegate(QWidget * parent) : QItemDelegate(parent) {
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaTreeWidgetItemEditorDelegate -> creation begin";
#endif
	_parent = parent;
	_dateDisplayFormat = "dd/MM/yyyy";
	_coursesList.empty();
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaTreeWidgetItemEditorDelegate -> creation end";
#endif
}

void AssumaTreeWidgetItemEditorDelegate::setCoursesList( const QStringList l ){
	_coursesList = l;
}

void AssumaTreeWidgetItemEditorDelegate::setLicenseLabelsList( const QStringList l ) {
	_licenseLabels = l;
}

void AssumaTreeWidgetItemEditorDelegate::setCoursesLessonsNumberData( const QMap<QString, QStringList> d ){
	_coursesLessonsNumber = d;
}

void AssumaTreeWidgetItemEditorDelegate::setCurrentActivity(const QString & a){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaTreeWidgetItemEditorDelegate::setCurrentActivity -> _currentActivity is now" << a;
#endif
	_currentActivity = a;
}

void AssumaTreeWidgetItemEditorDelegate::setPostCodesMap( const QMap<QString,QString> & map ){
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaTreeWidgetItemEditorDelegate::setPostCodesMap -> " << map;
#endif
	_pcMap = map;
}

QWidget *AssumaTreeWidgetItemEditorDelegate::createEditor(QWidget *parent,const QStyleOptionViewItem & /* option */, const QModelIndex &index) const
 {
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaTreeWidgetItemEditorDelegate::createEditor -> column:" << index.column();
#endif
	QLineEdit *lineEdit = new QLineEdit(parent);
	if( _parent ){
#ifdef ASSUMA_DEBUG
		qDebug() << "AssumaTreeWidgetItemEditorDelegate::createEditor -> we have a _parent";
#endif
		QTreeWidget *tw = static_cast<QTreeWidget*>(_parent);
		if(tw){
			QTreeWidgetItem *head = tw->headerItem();
#ifdef ASSUMA_DEBUG
			qDebug() << "AssumaTreeWidgetItemEditorDelegate::createEditor -> ...and it's a QTreeWidget, column name is:" << head->text( index.column() ) ;
#endif
			if( head->text( index.column() ) == tr("Fee") ){
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaTreeWidgetItemEditorDelegate::createEditor -> and we are editing fees !" ;
#endif
				lineEdit->setValidator( new QDoubleValidator(lineEdit) );
			}
			else if( head->text( index.column() ) == tr("BirthDate") || head->text( index.column() ) == tr("SubscriptionDate") ){
				QDateEdit *date = new QDateEdit(parent);
				date->setDisplayFormat( _dateDisplayFormat );
				date->setCalendarPopup(true);
				return date;
			}
			else if( head->text( index.column() ) == tr("MedicalCertificateStatus") || head->text( index.column() ) == tr("DuesStatus") ){
				QComboBox *combo = new QComboBox(parent);
				QIcon ok_icon;
				QIcon nok_icon;
				if( AssumaMW ){
					ok_icon = QIcon( AssumaMW->getImageFromTheme("button-ok") );
					nok_icon = QIcon( AssumaMW->getImageFromTheme("button-nok") );
				}
				combo->insertItem(0,tr("Ok"));
				combo->setItemIcon(0,ok_icon);
				combo->insertItem(1,tr("Not ok"));
				combo->setItemIcon(1,nok_icon);
				return combo;
			}
			else if( head->text( index.column() ) == tr("Activity") ){
				QComboBox *combo = new QComboBox(parent);
				combo->addItems( _coursesList );
				return combo;
			}
			else if( head->text( index.column() ) == tr("LicenseLabel") ){
				QComboBox *combo = new QComboBox(parent);
				combo->addItems( _licenseLabels );
				return combo;
			}
			else if( head->text( index.column() ) == tr("LessonsNumber") ){
				QComboBox *combo = new QComboBox(parent);
				combo->addItems( _coursesLessonsNumber.value(_currentActivity) );
				return combo;
			}
			else if( head->text( index.column() ) == tr("City") ){
				QLineEdit *lineEdit = new QLineEdit(parent);
				QCompleter *completer = new QCompleter(_pcMap.keys(), parent);
				completer->setCaseSensitivity(Qt::CaseInsensitive);
				lineEdit->setCompleter(completer);
				return lineEdit;
			}
			else if( head->text( index.column() ) == tr("PostalCode") ){
				QLineEdit *lineEdit = new QLineEdit(parent);
				QCompleter *completer = new QCompleter(_pcMap.values(), parent);
				completer->setCaseSensitivity(Qt::CaseInsensitive);
				lineEdit->setCompleter(completer);
				return lineEdit;
			}
			else{
				CustomFieldItem *cfItem = AssumaMW->getCustomFieldItemByName( head->text( index.column() ) );
				if( cfItem ){
					QStringList extra_arguments = cfItem->extraString().split(";");
					if( cfItem->type() == "Multiple line text" ){
						return AssumaCustomWidgetMultipleLine::createEditor(parent);
					}
					else if( cfItem->type() == "Date selector" ){
						return AssumaCustomWidgetDateEdit::createEditor(parent);
					}
					else if( cfItem->type() == "Ok/Not ok combobox" ){
						QComboBox *w = new QComboBox(parent);
						w->addItem( extra_arguments.at(0), QIcon( AssumaMW->getImageFromTheme("button-ok") ) );
						w->addItem( extra_arguments.at(1), QIcon( AssumaMW->getImageFromTheme("button-nok") ) );
						return w;
					}
					else if( cfItem->type() == "Multiple value selector" ){
						QComboBox *w = new QComboBox(parent);
						foreach( QString str, extra_arguments ){
							if( ! str.isEmpty() )
								w->addItem(str);
						}
						return w;
					}
					else if( cfItem->type() == "Fee field" ){
						return AssumaCustomWidgetFeeEditor::createEditor(parent);
					}
				}
			}
		}
	}
	return lineEdit;
 }

void AssumaTreeWidgetItemEditorDelegate::setEditorData(QWidget *editor,const QModelIndex &index) const
{
	QVariant value = index.model()->data(index, Qt::DisplayRole);
#ifdef ASSUMA_DEBUG
	qDebug() << "AssumaTreeWidgetItemEditorDelegate::setEditorData -> about to set data:" << value.toString();
#endif
	if( QDateEdit *date = qobject_cast<QDateEdit *>(editor) )
		date->setDate( QDate::fromString( value.toString(), _dateDisplayFormat ) );
	else if (QComboBox *widget = qobject_cast<QComboBox *>(editor)){
		int idx = widget->findText ( value.toString(), Qt::MatchFixedString );
		if( idx > -1 )
			widget->setCurrentIndex(idx);
		else
			widget->setCurrentIndex(1);
	}
	else if (QLineEdit *lineEdit = qobject_cast<QLineEdit *>(editor))
		lineEdit->setText( value.toString() );
	else if( QTextEdit *text = qobject_cast<QTextEdit *>(editor) )
		text->setText(value.toString());
	else if( QDoubleSpinBox *w = qobject_cast<QDoubleSpinBox *>(editor) )
		w->setValue(value.toDouble());
}

void AssumaTreeWidgetItemEditorDelegate::setModelData(QWidget *editor, QAbstractItemModel *model, const QModelIndex &index) const
{
	if( _parent ){
		QTreeWidget *tw = static_cast<QTreeWidget*>(_parent);
		if(tw){
			QTreeWidgetItem *head = tw->headerItem();
			if( head->text( index.column() ) == tr("BirthDate") || head->text( index.column() ) == tr("SubscriptionDate") ){
				QDateEdit *date = qobject_cast<QDateEdit *>(editor);
#ifdef ASSUMA_DEBUG
				qDebug() << "AssumaTreeWidgetItemEditorDelegate::setModelData -> data were dates:" << date->date().toString( _dateDisplayFormat );
#endif
				model->setData(index, date->date().toString( _dateDisplayFormat ), Qt::DisplayRole);
				return;
			}
			else if(QComboBox *widget = qobject_cast<QComboBox *>(editor)){
				int idx = widget->currentIndex();
				model->setData(index, widget->itemText(idx), Qt::DisplayRole);
				if( ! widget->itemIcon(idx).isNull() )
					model->setData(index, widget->itemIcon(idx), Qt::DecorationRole);
				return;
			}
		}
	}
	QItemDelegate::setModelData(editor,model,index);
}

QSize AssumaTreeWidgetItemEditorDelegate::sizeHint(const QStyleOptionViewItem &/*option*/,const QModelIndex &index) const
{
	if( _parent ){
		QTreeWidget *tw = static_cast<QTreeWidget*>(_parent);
		if(tw){
			QString col_name = tw->headerItem()->text( index.column() );
			if( col_name == tr("Address") || col_name == tr("Picture") || col_name == tr("LicenseLabel") || col_name == tr("Email") || col_name == tr("City") || col_name == tr("Comment") ){
				return QSize(150,32);
			}
		}
	}
	return QSize(50,32);
}
