
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "assuma_updatedelegate.h"
#include <QTextDocument>
#include <QPainter>
#include <QStyleOptionViewItemV2>
#include <QApplication>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#define AU_ICON_SIZE 48
#define AU_BORDER 5
#define AU_SPACING 8
#define AU_CHECK_SIZE 20
#define AU_MINIMUM_TEXT_WIDTH 300
#define AU_MINIMUM_CELL_WIDTH (AU_MINIMUM_TEXT_WIDTH + AU_BORDER + AU_ICON_SIZE + AU_SPACING + AU_BORDER) 

AssumaUpdateDelegate::AssumaUpdateDelegate( QObject *parent ) : QItemDelegate( parent ){
}

void AssumaUpdateDelegate::paint( QPainter *painter, const QStyleOptionViewItem & option, const QModelIndex &index ) const {
	painter->save();
	if (option.state & QStyle::State_Selected)
	{
		QPalette pal=option.palette;
		QBrush brush=pal.highlight();
		QColor col=brush.color();
		col.setAlpha(127);
		brush.setColor(col);
		painter->fillRect(option.rect,brush);
	}
	
	// Test Check
	QRect checkRect;
	Qt::CheckState checkState = Qt::Unchecked;
	QVariant value = index.data(Qt::CheckStateRole);
	if (value.isValid()) {
		QRect r = option.rect ;
		r.setX( r.x() + AU_BORDER );
		checkState = static_cast<Qt::CheckState>(value.toInt());
		checkRect = check(option, r, value);
	}
        drawCheck(painter, option, checkRect, checkState);
	// End test check
	
	QString text=index.data(Qt::DisplayRole).toString();
	QPixmap pixmap;
	QRect decorationRect;
	/*QVariant */value = index.data(Qt::DecorationRole);
	QStyle::State state=option.state;
	QRect rect=option.rect;
	int afterIcon = AU_BORDER + AU_ICON_SIZE + AU_SPACING;
	QIcon ico=QIcon(value.value<QIcon>());
//	painter->drawPixmap(option.rect.x()+AU_BORDER+AU_CHECK_SIZE,option.rect.y()+AU_BORDER,ico.pixmap(AU_ICON_SIZE,AU_ICON_SIZE));
        painter->drawPixmap(option.rect.x()+checkRect.x()+checkRect.width()+AU_BORDER,option.rect.y()+AU_BORDER,ico.pixmap(AU_ICON_SIZE,AU_ICON_SIZE));
	QTextDocument doc;
	doc.setHtml( text );
	painter->translate(option.rect.x()+afterIcon+AU_CHECK_SIZE,option.rect.y()+AU_BORDER);
// 	doc.setTextWidth(option.rect.width()-10);
	doc.setTextWidth(option.rect.width()-110);
// 	QRect cliprect=QRect(QPoint(0,0),QSize(option.rect.width()-afterIcon,option.rect.height()-(AU_BORDER*2)-4));
	QRect cliprect=QRect(QPoint(0,0),QSize(option.rect.width()-afterIcon-100,option.rect.height()-(AU_BORDER*2)-4));
	doc.drawContents(painter, cliprect);
	
	// Test ProgressBar
	int progress = index.data(ProgressRole).toInt();
	if(progress > 0 || progress != index.data(ProgressMaxRole).toInt() ){
		QStyleOptionProgressBar progressBarOption;
		progressBarOption.state = QStyle::State_Enabled;
		progressBarOption.direction = QApplication::layoutDirection();
		//progressBarOption.rect = option.rect;
		QRect pbRect( cliprect.x()+cliprect.width(),0, 75 , (option.rect.height()-(AU_BORDER*2)-4)/2 );
		progressBarOption.rect = pbRect;
		progressBarOption.fontMetrics = QApplication::fontMetrics();
		progressBarOption.minimum = index.data(ProgressMinRole).toInt();
		progressBarOption.maximum = index.data(ProgressMaxRole).toInt();
		progressBarOption.textAlignment = Qt::AlignCenter;
		progressBarOption.textVisible = true;

		// Set the progress and text values of the style option.
		progressBarOption.progress = progress < 0 ? 0 : progress;

		// Draw the progress bar onto the view.
		QApplication::style()->drawControl(QStyle::CE_ProgressBar, &progressBarOption, painter);
		
		// Display the download speed
		QString dl_speed = index.data(DownloadSpeed).toString()+" "+index.data(DownloadSpeedUnit).toString();
		painter->drawText( pbRect.x(),pbRect.y()+pbRect.height()+15, dl_speed );
	}
	// End test ProgressBar
	
	painter->restore(); 
}

QSize AssumaUpdateDelegate::sizeHint ( const QStyleOptionViewItem & /*option*/, const QModelIndex & /*index*/ ) const {
	return QSize( AU_MINIMUM_CELL_WIDTH , AU_BORDER + AU_ICON_SIZE + AU_SPACING );
}


// void drawDecoration ( QPainter * painter, const QStyleOptionViewItem & option, const QRect & rect, const QPixmap & pixmap ) const {
	// QItemDelegate::drawDecoration(painter,option,rect,pixmap);
// }
