
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "federationhandler.h"

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QMessageBox>

#define FEDERATION_XML_SUPPORTED_VERSION "1.0"

FederationHandler::FederationHandler(){
	licences.empty();
#ifdef ASSUMA_DEBUG
	qDebug() << "FederationHandler::FederationHandler() -> object constructed";
#endif
}

bool FederationHandler::startElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString &qName, const QXmlAttributes &attributes ){
	if( qName == "federation" ){
		if( QString( FEDERATION_XML_SUPPORTED_VERSION ) < attributes.value("version","0.01") ){
#ifdef ASSUMA_DEBUG
			qDebug() << "FederationHandler::startElement(...) -> BORK : mismatch versions";
#endif
		}
#ifdef ASSUMA_DEBUG
		else
			qDebug() << "FederationHandler::startElement(...) -> Federations XML file is a supported version";
#endif
	}
	else if( qName == "licence" ){
		licences << new FederationLicenseFee( attributes.value("id"), attributes.value("label") , attributes.value("fee").toFloat() );
#ifdef ASSUMA_DEBUG
		qDebug() << "FederationHandler::startElement(...) -> adding " << attributes.value("label") << " => " << attributes.value("fee").toFloat() << " to licences.";
#endif
	}
	return true;
}

bool FederationHandler::endElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &/*qName*/ ){
	return true;
}

bool FederationHandler::characters(const QString &str){
	currentText += str ;
	return true;
}

bool FederationHandler::fatalError(const QXmlParseException &exception){
	QMessageBox::warning(0,QObject::tr("Association Subscribers Manager federations parser"),
		QObject::tr("parse error at line %1, column %2:\n%3")
		.arg( exception.lineNumber() )
		.arg( exception.columnNumber() )
		.arg( exception.message() )
		
	);
	return false;
}

QList<FederationLicenseFee *> FederationHandler::getLicenceFees(){
	return licences;
}
