
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "oldxmlhandler.h"
#include <QMessageBox>
#include <QTreeWidgetItem>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

#include <QCoreApplication>


OldXmlHandler::OldXmlHandler( QTreeWidget *tree ){
	treeWidget = tree;
	currentText = "";
}

bool OldXmlHandler::startElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString &qName, const QXmlAttributes & /*attributes*/ ){
	if( qName == "membre" ){
		currentItem = new QTreeWidgetItem(treeWidget);
	}
	if( qName != "association-manager" && qName != "membre" ){
		currentText = "";
	}
	return true;
}

bool OldXmlHandler::endElement( const QString & /*namespaceURI*/, const QString & /*localName*/, const QString &qName )
{
	if( qName != "association-manager" && qName != "membre"){
		int col = elementColumn(qName);
		if( col != -1 ){
			if( qName == "cotisation" ){
				if( currentText.contains("non") )
					currentText = "0" ;
				else
					currentText = "1" ;
			}
			if( currentText == "non" )
			    currentText = "0" ;
			if( currentText == "oui" )
			    currentText = "1" ;
#ifdef ASSUMA_DEBUG
			qDebug() << "OldXmlHandler::endElement -> inserting" << currentText << "at column" << col;
#endif
			currentItem->setText(col,currentText);
		}
	}
	return true;
}

bool OldXmlHandler::characters(const QString &str){
	currentText += str ;
	return true;
}

bool OldXmlHandler::fatalError(const QXmlParseException &exception){
	QMessageBox::warning(0,QObject::tr("Association Subscribers Manager XML database parser"),
		QObject::tr("parse error at line %1, column %2:\n%3")
		.arg( exception.lineNumber() )
		.arg( exception.columnNumber() )
		.arg( exception.message() )
		
	);
	return false;
}

int OldXmlHandler::elementColumn(const QString& element){
	QString trElem;
	if( element == "prenom" )
		trElem = QCoreApplication::translate("AssumaMainWindow","FirstName");
	else if( element == "nom-jeune-fille" )
		trElem = QCoreApplication::translate("AssumaMainWindow","MaidenName");
	else if( element == "nom" )
		trElem = QCoreApplication::translate("AssumaMainWindow","LastName");
	else if( element == "date-naissance" )
		trElem = QCoreApplication::translate("AssumaMainWindow","BirthDate");
	else if( element == "ville" )
		trElem = QCoreApplication::translate("AssumaMainWindow","City");
	else if( element == "code-postal" )
		trElem = QCoreApplication::translate("AssumaMainWindow","PostalCode");
	else if( element == "adresse" )
		trElem = QCoreApplication::translate("AssumaMainWindow","Address");
	else if( element == "telephone" )
		trElem = QCoreApplication::translate("AssumaMainWindow","PhoneNumber");
	else if( element == "mail" )
		trElem = QCoreApplication::translate("AssumaMainWindow","Email");
	else if( element == "nombre-cours" )
		trElem = QCoreApplication::translate("AssumaMainWindow","LessonsNumber");
	else if( element == "prix-cotisation" )
		trElem = QCoreApplication::translate("AssumaMainWindow","Dues");
	else if( element == "cotisation" )
		trElem = QCoreApplication::translate("AssumaMainWindow","DuesStatus");
	else if( element == "date-inscription" )
		trElem = QCoreApplication::translate("AssumaMainWindow","SubscriptionDate");
	else if( element == "certificat-medical" )
		trElem = QCoreApplication::translate("AssumaMainWindow","MedicalCertificateStatus");
	else if( element == "numero-licence" )
		trElem = QCoreApplication::translate("AssumaMainWindow","LicenseNumber");
	else if( element == "activitee" )
		trElem = QCoreApplication::translate("AssumaMainWindow","Activity");
	else if( element == "commentaire" )
		trElem = QCoreApplication::translate("AssumaMainWindow","Comment");
	else if( element == "portrait" )
		trElem = QCoreApplication::translate("AssumaMainWindow","Picture");
	else if( element == "label-licence" )
		trElem = QCoreApplication::translate("AssumaMainWindow","LicenseLabel");
	
	QTreeWidgetItem *header = treeWidget->headerItem();
	for (int i = 0 ; i <= header->columnCount() ; i++ ){
		if( trElem== header->text(i) )
			return i;
	}
	return -1;
}
