
//  Copyright (C) 2009 by Arnaud Dupuis
//  a.dupuis@infinityperl.org
//  http://www.infinityperl.org
// 
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 3 of the License, or
//  (at your option) any later version.
// 
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
// 
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the
//  Free Software Foundation, Inc.,
//  59 Temple Place - Suite 330, Boston, MA  02111-1307, USA

#include "postalcodehandler.h"
#include <QMessageBox>

#ifdef ASSUMA_DEBUG
#include <QDebug>
#endif

PostalCodeHandler::PostalCodeHandler( ){
	rmap.clear();
#ifdef ASSUMA_DEBUG
	qDebug() << "PostalCodeHandler-> Constructed";
#endif
}

QMap<QString,QString> PostalCodeHandler::getPostalCodes(){
	return rmap;
}

bool PostalCodeHandler::startElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &qName, const QXmlAttributes &attributes ){
	bool ok;
	if( qName == "Cities" ){
		float version = attributes.value("version").toFloat(&ok);
#ifdef ASSUMA_DEBUG
		qDebug() << "PostalCodeHandler-> Postal Codes DB version: " << version;
#endif
		if( version < 2.0 ){
			QMessageBox::warning(0,QObject::tr("Association Subscribers Manager postal codes parser"),
				QObject::tr("The postal codes database is too old and cannot be read by this version of Association Subscribers Manager.")				
			);
			return false;
		}
	}
	if( qName == "City" ){
		rmap.insert(attributes.value("name").toUpper(),attributes.value("pc"));
	}
	return true;
}

bool PostalCodeHandler::endElement( const QString &/*namespaceURI*/, const QString &/*localName*/, const QString &/*qName*/ )
{
	return true;
}

bool PostalCodeHandler::characters(const QString &/*str*/){
	return true;
}

bool PostalCodeHandler::fatalError(const QXmlParseException &exception){
	QMessageBox::warning(0,QObject::tr("Association Subscribers Manager ADB parser"),
		QObject::tr("parse error at line %1, column %2:\n%3")
		.arg( exception.lineNumber() )
		.arg( exception.columnNumber() )
		.arg( exception.message() )
		
	);
	return false;
}

